/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains Ajax related classes and functions
 * @author Erkan Cengiz
 * @version $Id: Ajax.js 225 2008-09-30 20:56:35Z erkan $
 */


 /** @private */
function hasmmprop() {
  try {
    eval('var x = mmprop');
  } catch(e) {
    return false;
  }
  return true;
}


/** @private */
function BusyImage(node) {

  this.node = node;

  this.init = function() {
    this.node.style.visibility = "hidden";
  };

}


/**
 * @class
 * Represents a TargetArea. A target area is a container in the DOM whose content can be refreshed
 * with {@link MoreMotionAjaxManager#refreshAreas} method. It wrappes a DOM element node that has a "mo:type" attribute
 * with the value "TargetArea".
 */
function TargetArea(node) {

  this.base = MoreMotionObject;
  this.base(node);

  /**
   * Sets the given HTML code to this Target Area and initializes the MoreMotionObject objects in it.
   * @param html The HTML code to set to this target area.
   */

}

/**
 * @class
 * Manager Class that provides static methods for Ajax functionalities.  A static object of this class
 * is avaialable with name <CODE>AjaxMgr</CODE>.
 * <PRE>var AjaxMgr = new MoreMotionAjaxManager();</PRE>
 *
 *
 */
function MoreMotionAjaxManager() {

  /** @private */
  this.getElementsArray = function(elmidsStr) {
    var elms = new Array();
    var elmids = elmidsStr.split(",");
    for (var i = 0; i < elmids.length; i++) {
       var elmid = elmids[i].trim();
      elms[i] = document.getElementById(elmid);
    }
    return elms;
  }

  /**
   * Refreshes the options of an HTML <code>SELECT</code> Element with the current data acquired
   * from a MoreMotion Data Source.
   * This method utilizes the {@link AjaxRequest} class to send a XMLHttpRequest to the MoreMotion Ajax Service
   * on the server.
   * @param prm The parameters object
   * <ul>
   * <li><b>prm.actionName</b> - The name of the <CODE>actionParams</CODE> configuration
   * element in the MoreMotion configuration.
   * </li><li><b>prm.optionNode</b> - The Name of the data node in the datasource that will provide the text of each option
   * </li><li><b>prm.valueNode</b> - The Name of the data node in the datasource that will provide the value of each option
   * </li><li><b>prm.staticOptionsQty</b> - Static Options Quantity. The number of the options on the top of
   * the list that should be preserved while refreshing the others.
   * </li><li><b>prm.targetElement</b> - The id or the object itself of the <code>SELECT</code> Element (ComboBox or ListBox)
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the target <code>SELECT</code> Element
   * is refreshed.
   * The function receives {@link MultiPartAjaxResponse} object as the parameter.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </li></ul>
   * <p>
   * <u>Example:</u><br/><pre>
   * AjaxMgr.refreshOptions(&#123; <br/>
   *   actionName     : 'RefreshCities' <br/>
   *   ,optionNode    : 'NAME' <br/>
   *   ,valueNode     : 'ID'  <br/>
   *   ,targetElement : 'x123' <br/>
   *   ,requestParams : 'CAT=' + this.value <br/>
   * &#125;);<br/>
   * <br/>
   * <u>Configuration:</u><br/>
   * &lt;actionParams name="RefreshCities"&gt;<br/>
   *   &lt;dataSource&gt;cities_of_country&lt;/dataSource&gt;<br/>
   *   &lt;valueNode&gt;ID&lt;/valueNode&gt;<br/>
   *   &lt;encryptTheValue&gt;true&lt;/encryptTheValue&gt;<br/>
   * &lt;/actionParams&gt;<br/>
   *
   * </pre>
   *
   */
  this.refreshOptions = function(prm) {

    var props = new Object();
    var params = new RequestParameterAccumulator(prm.requestParams);

    params.add("_cfg",prm.actionName);

    props.optionNode = prm.optionNode || prm.optionField;
    props.valueNode = prm.valueNode || prm.valueField;
    props.targetElement = prm.targetElement.after ? document.getElementById(prm.targetElement) : prm.targetElement;
    props.staticOptionsQty = prm.staticOptionsQty;

    props.callbackFunc = prm.callbackFunc;
    props.userData = prm.userData;

    request = new AjaxRequest(params,null,props);

    request.service = "RefreshOptions.doms";
    request.pageInfo = OMgr.getPageInfo( props.targetElement );
    request.errorHandler = prm.errorHandler;
    request.busyImage = prm.busyImage;
    request.waitForResponse = (prm.waitForResponse == true);

    request.send();
  };

  /**
   * Clears the options of an HTML <code>SELECT</code> Element.
   * @param elmid The id of the <code>SELECT</code> Element (ComboBox or ListBox)
   * @param soq Static Options Quantity. The number of the options on the top of
   * the list that should be preserved while clearing the others.
   */
  this.clearOptions = function(elmid, soq) {
    var box = elmid.after ? document.getElementById(elmid) : elmid;
    if (box == null) {
      return;
    }
    if (soq) {
      box.options.length = soq;
    } else {
      box.options.length = 0;
    }
    box.selectedIndex = 0;
  };

  /** @private */
  this.refreshOptionsResponseHandler = function(response) {

    var request = response.request;

    var staticOptionsQty = null;
    var targetElement = null;
    var optionNode = null;
    var valueNode = null;

    if (response.isProcessResponse()) {
      staticOptionsQty = response.props.staticOptionsQty;
      targetElement = document.getElementById(response.props.targetElement);
      optionNode = response.props.optionNode;
      valueNode = response.props.valueNode;
    } else {
      staticOptionsQty = request.props.staticOptionsQty;
      targetElement = request.props.targetElement;
      optionNode = request.props.optionNode;
      valueNode = request.props.valueNode;
    }

    AjaxMgr.clearOptions(targetElement,staticOptionsQty);
    var select = targetElement;

    if (!select) {
      var ex = new Exception("REFRESH_OPTIONS_ELEMENT_MISSING");
      if (request.errorHandler) {
        var handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
        handler.run(ex, request);
      } else throw ex;
    }

    var j = staticOptionsQty;
    while (response.hasMore()) {
      var rec = response.next();
      select.options[j++] = new Option( rec.get(optionNode), rec.get(valueNode) );
    }

  };

  /**
   * Refreshes the values of HTML Elements or {@link ProcessField} objects. This method utilizes the
   * {@link AjaxRequest} class to send a XMLHttpRequest to the MoreMotion Ajax Service on the server.
   * @param prm The parameters object.
   * <ul>
   * <li><b>prm.actionName</b> - The name of the <CODE>actionParams</CODE> configuration
   * element in the MoreMotion configuration.
   * </li><li><b>prm.targetElements</b> - Target HTML elements or Process Fields to refresh.
   * It can be a single String that contains the HTML ids of the elements separated with commas or an Array of elements.
   * </li><li><b>prm.targetElementAttrs</b> - A String that contains the names of the attributes
   * of the target elements, separated with commas, to set. If omitted "value" attributes of the target elements are set.
   * Note that the definitions should positionally correspond to the elements defined in targetElements property.
   * </li><li><b>prm.dataNodes</b> - The names of the Data Nodes existing in the result record to take the
   * values for the target elements. If this parameter is omitted it is assumed that the names of the data nodes
   * are identical to the target element names. <br>Note that some target process fields require more than
   * one data nodes, e.g. {@link SuggestBox}. In that case data node names must be separated with "/" character.
   * <br><u>Example:</u> <CODE>NAME/ID,STOCK,PRICE</CODE>
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the target elements are refreshed.
   * The function receives {@link MultiPartAjaxResponse} object as the parameter.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </li></ul>
   * <p>
   * <u>Example:</u><br/><pre>
   *  AjaxMgr.refreshFields(&#123; <br/>
   *    actionName      : 'RefreshProductFields'  <br/>
   *    ,dataNodes      : 'NAME/ID,STOCK,PRICE'  <br/>
   *    ,targetElements : 'PROD_SBOX,STOCK,PRICE'  <br/>
   *    ,requestParams  : 'ID=' + this.value  <br/>
   *  &#125;); <br/>
   * <br/>
   * <u>Configuration:</u> <br/>
   * &lt;actionParams name="RefreshProductFields"&gt; <br/>
   *   &lt;dataSource&gt;product_details&lt;/dataSource&gt; <br/>
   *   &lt;nodesToEncrypt&gt;&lt;/nodesToEncrypt&gt; <br/>
   * &lt;/actionParams&gt; <br/>
   *
   * </pre>
   *
   */
  this.refreshFields = function(prm) {

    var props = new Object();
    var params = new RequestParameterAccumulator(prm.requestParams);

    params.add("_cfg",prm.actionName);

    props.dataNodes = prm.dataNodes;
    props.targetElements = prm.targetElements;
    if (props.targetElements.after) {
      props.targetElements = this.getElementsArray(props.targetElements);
    }

    props.targetElementAttrs = prm.targetElementAttrs;
    props.callbackFunc = prm.callbackFunc;
    props.userData = prm.userData;

    var request = new AjaxRequest(params,null,props);

    request.service = "RefreshFields.doms";
    request.errorHandler = prm.errorHandler;
    request.busyImage = prm.busyImage;
    request.pageInfo = OMgr.getPageInfo( props.targetElements[0] );
    request.waitForResponse = (prm.waitForResponse == true);

    request.send();
  };

  /** @private */
  this.refreshFieldsResponseHandler = function(response) {

    var request = response.request;

    var rec = response.getResponseRecord(0);

    var props = response.request.props;

    var dataNodes = null;
    var targetElements = null;
    var targetElementAttrs = null;
    if (response.isProcessResponse()) {
      dataNodes = response.props.dataNodes;
      targetElements = AjaxMgr.getElementsArray(response.props.targetElements);
      targetElementAttrs = (response.props.targetElementAttrs || "").split(",");
    } else {
      dataNodes = request.props.dataNodes;
      targetElements = request.props.targetElements;
      targetElementAttrs = (request.props.targetElementAttrs || "").split(",");
    }

    if (dataNodes) dataNodes = dataNodes.split(",");

    for (var i = 0; i < targetElements.length; i++ ) {

      var node = dataNodes ? dataNodes[i] : null;
      var elm = targetElements[i];
      var attr = "value";

      if (i < targetElementAttrs.length) {
        attr = targetElementAttrs[i];
        if (attr == "") attr = "value";
      }

      var handler = null;
      var ex = null;

      if (!elm) {
        ex = new Exception("REFRESH_FIELDS_ELEMENT_MISSING",i);
        if (request.errorHandler) {
          handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
          handler.run(ex, response);
        } else throw ex;
      }

      var o = OMgr.getObject(elm);

      if (o) {
        if (o.ajaxRefreshHandler) {
          try {
            o.ajaxRefreshHandler(rec,node,attr);
          } catch (e) {
            if (request.errorHandler) {
              handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
              handler.run(e, response);
            } else throw e;
          }
          continue;
        }
        if (!node) node = o.name;
      }

      if (!node) node = elm.name;

      var value = "";

      if (rec) {
        if (!response.hasField(node)) {

          ex = new Exception("RESPONSE_FIELD_MISSING",i,node);
          if (request.errorHandler) {
            handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
            handler.run(ex, response);
          } else throw ex;

        }
        value = rec.get(node);
      }

      if (o && o.setValue) {
        o.setValue(value);
        continue;
      }

      if ((attr == "checked") || (attr == "readOnly")) {
        eval("elm." + attr + " = " + value);
      } else if (attr == "value") {
        if (elm.type == "checkbox") {
            elm.checked = value.toBool();
        } else if (elm.type == "radio") {
          if (elm.value == value) elm.checked = true; else elm.checked = false;
        } else {
          elm.value = value;
        }
      } else {
        eval("elm." + attr + " = value");
      }

    }

  };


   /**
   * Refreshes the records of the specified ProcessBlock elements in the current page.
   * This method utilizes the {@link AjaxRequest} class to send a XMLHttpRequest to the MoreMotion Ajax Service
   * on the server.
   * @param prm The parameters object.
   * <ul>
   * <li><b>prm.actionName</b> - The name of the <CODE>actionParams</CODE> configuration
   * element in the MoreMotion configuration.
   * </li><li><b>prm.targetBlocks</b> - Target blocks elements to refresh.
   * It can be a single String that contains the HTML ids of the target elements separated with commas or an Array of elements.
   * </li><li><b>prm.fieldMap</b> - By default, the values of the process fields in the specified process blocks are
   * refreshed with the values of the identically named data nodes. If there are exceptions to this rule it can be
   * specified with this parameter in
   * <CODE><i>processFieldName</i>:<i>dataNodeName</i>,[<i>processFieldName</i>:<i>dataNodeName</i> ...]</CODE>
   * format. <br/><u>Example:</u> <CODE>ORDER_ID:ID,UNIT_PRICE:PRICE</CODE>
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the target elements are refreshed.
   * The function receives {@link MultiPartAjaxResponse} object as the parameter.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </li></ul>
   * <p>
   * <u>Example:</u><br/><pre>
   * AjaxMgr.refreshBlocks(&#123; <br/>
   *   actionName     : 'RefreshFormRecords'  <br/>
   *   ,targetBlocks  : 'PF1.ProductsPB,PF2.Customers'  <br/>
   *   ,requestParams : 'ID=' + this.value  <br/>
   * &#125;); <br/>
   * <br/>
   * <u>Configuration:</u> <br/>
   * &lt;actionParams name="RefreshFormRecords"&gt; <br/>
   *   &lt;dataSources&gt;products,customers&lt;/dataSources&gt;<br/>
   *   &lt;nodesToEncrypt&gt;ID&lt;/nodesToEncrypt&gt;<br/>
   * &lt;/actionParams&gt; <br/>
   *  </pre>
   */
  this.refreshBlocks = function(prm) {

    var props = new Object();

    try {
      props.targetBlocks = prm.targetBlocks.after ? this.getBlocksArray(prm.targetBlocks,true) : prm.targetBlocks;
    } catch (ex){
      if (ex.alert) ex.alert(); else alert(ex.message);
    }

    if (prm.callbackFunc) props.callbackFunc = prm.callbackFunc;
    if (prm.userData) props.userData = prm.userData;
    if (prm.fieldMap) props.fieldMap = prm.fieldMap;

    var params = new RequestParameterAccumulator(prm.requestParams);
    params.add("_cfg",prm.actionName);

    request = new AjaxRequest(params,null,props);

    request.service = "RefreshBlocks.doms";
    request.errorHandler = prm.errorHandler;
    request.busyImage = prm.busyImage;
    var blocks = this.getBlocksArray(prm.targetBlocks);
    request.pageInfo = OMgr.getPageInfo( blocks ? blocks[0] : null);
    request.waitForResponse = (prm.waitForResponse == true);

    request.send();
  };

  /** @private */
  this.getFieldMap = function(fieldMapStr) {
    if (!fieldMapStr) return null;
    var map = new Array();
    var defs = fieldMapStr.split(",");
    if (fieldMapStr.indexOf(",") == -1) {
      defs = fieldMapStr.split(";");
    }
    for (var i = 0; i < defs.length; i++) {
     var def = defs[i];
     var pfname = def.before(":",def).trim(); // Process Field Name
     var dnname = def.after(":",def).trim();  // Data Node Name
     map[pfname] = dnname;
    }
    return map;
  };

  /** @private */
  this.getBlocksArray = function(targetBlocks, validate) {

    var blocks = new Array();

    var defs = targetBlocks.split(",");
    for (var i = 0; i < defs.length; i++) {
      var def = defs[i].trim();
      var formName = def.before(".");
      var blockName = def.after(".");

      var block = null;
      try {
        block = PMgr.getForm(formName).getBlock(blockName);
      } catch (e) {}

      if (block) {
        blocks[blocks.length] = block;
      } else {
        if (validate) {
          throw new Exception("PROCESS_BLOCK_MISSING",def);
        }
      }
    }
    return blocks;
   }

   /** @private */
  this.refreshBlocksResponseHandler = function(response) {

    var b = null;
    var fieldMap = null;
    var targetBlocks = null;
    if (response.isProcessResponse()) {
      fieldMap = AjaxMgr.getFieldMap(response.props.fieldMap);
      targetBlocks = AjaxMgr.getBlocksArray(response.props.targetBlock,true);
      b = targetBlocks[0];
    } else  {
      fieldMap = AjaxMgr.getFieldMap(request.props.fieldMap);
      targetBlocks = request.props.targetBlocks;
      b = targetBlocks[response.props.blockSeq];
    }

    b.removeAllRecords();

    // var recnum = 0;
    while (response.hasMore()) {
      var rec = response.next();
      if (!b.boolProp("recordManip")) {
        throw new Exception("TEMPLATE_RECORD_MISSING",b.name);
      }
      var r = b.addRecord().clear();
      var fields = r.getFields();
      for (var i = 0; i < fields.length; i++) {
        var f = fields[i];
        try {
          AjaxMgr.setFieldValue(f,rec,fieldMap);
        } catch (ex) {
          if (ex.code && ex.code == "RESPONSE_FIELD_MISSING") {
            throw new Exception("REFRESH_BLOCKS_RESPONSE_FIELD_MISSING",ex.arg1,i);
          }
          throw ex;
        }
      }
      // recnum++;
    }

  };

  /** @private */
  this.setFieldValue = function(f, rec, fieldMap) {

    var fieldName = f.name;
    if (fieldMap != null && fieldMap[f.name] != null) {
     fieldName = fieldMap[f.name];
    }

    if (f.ajaxRefreshHandler) {
      f.ajaxRefreshHandler(rec, fieldName);

    } else {

      var value = "";

      if (!rec.owner.hasField(fieldName)) {
        throw new Exception("RESPONSE_FIELD_MISSING",fieldName);
      }
      value = rec.get(fieldName);

      if (f.setValue) {
        f.setValue(value);
      }
    }

    if (f.setRecordNumber) {
      f.setRecordNumber(rec.recordIndex);
    }

   };

  /** @private */
  this.refreshPanels = function(prm) {
    prm.sourceAreas = prm.sourcePanels;
    prm.targetAreas = prm.targetPanels;
    this.refreshAreas(prm);
  };

  /** @private */
  this.alreadyUsedSourceAreas = new Array();

  /** @private */
  this.getAlreadyUsedSourceAreas = function() {

    var acc = new Accumulator(",");
    for (var p in AjaxMgr.alreadyUsedSourceAreas) {
      var pv = AjaxMgr.alreadyUsedSourceAreas[p];
      if (pv) {
        acc.add(p + ":" + AjaxMgr.alreadyUsedSourceAreas[p]);
      }
    }
    return acc.toString();
  };

  /** @private */
  this.addNewSourceArea = function(area) {
    var areaName = area.before(":");
    var sequence = area.after(":");
    if (AjaxMgr.alreadyUsedSourceAreas[areaName] == null) {
      AjaxMgr.alreadyUsedSourceAreas[areaName] = sequence;
    }
  };

  /**
   * Refreshes only a part or parts of a page without making a new request for the whole page.
   * For the page parts to be refreshed in the page, the "TargetArea" or "RefreshableArea" elements should be used.
   * This method utilizes the {@link AjaxRequest} class to send a XMLHttpRequest to the MoreMotion Ajax Service
   * on the server.
   * @param prm The parameter object
   * <ul>
   * </li><li><b>prm.actionName</b> - The name of the <CODE>actionParams</CODE> configuration
   * element in the MoreMotion configuration.
   * </li><li><b>prm.targetAreas</b> - Target Areas to be refreshed in the current page.
   * It can be a single String that contains the HTML ids of the target elements separated with commas
   * or an Array of target elements.
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the target elements are refreshed.
   * The function receives {@link MultiPartAjaxResponse} object as the parameter.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </li><li><b>prm.originNode</b> - The handle of the DOM Node that originates this request.
   * This parameter should be supplied only if the call is made from a DOM node that was fetched
   * with a {@link #refreshAreas} method call. It is required to access to the enclosing node that
   * contains a <CODE>mo:pageInfo</CODE> attribute.
   * </li></ul>
   * <p>
   * <u>Example:</u><br/><pre>
   * AjaxMgr.refreshAreas(&#123; <br/>
   *   actionName    : '/RefreshDetails' <br/>
   *   ,targetAreas   : 'Details' <br/>
   *   ,requestParams : 'CAT=' + category <br/>
   *   ,busyImage     : 'bimg1' <br/>
   * &#125;); <br/>
   * <br/>
   * <u>Configuration:</u> <br/>
   * &lt;actionParams name="RefreshDetails"&gt; <br/>
   *   &lt;sourceAreas&gt;/AjaxSourcePage1.Details&lt;/sourceAreas&gt; <br/>
   * &lt;/actionParams&gt; <br/>
   *
   * </pre>
   */
  this.refreshAreas = function(prm) {

    var props = new Object();
    var params = new RequestParameterAccumulator(prm.requestParams);

    params.add("_cfg",prm.actionName);
    params.add("_usedsourceareas",this.getAlreadyUsedSourceAreas());
    params.add("_targetpage",OMgr.getPageInfo().name);

    props.targetAreas = prm.targetAreas;
    if (props.targetAreas.after) {
      props.targetAreas = this.getElementsArray(props.targetAreas);
    }

    props.callbackFunc = prm.callbackFunc;
    props.userData = prm.userData;

    var request = new AjaxRequest(params,null,props);

    request.service = "RefreshAreas.doms";
    request.errorHandler = prm.errorHandler;
    request.busyImage = prm.busyImage;
    request.pageInfo = OMgr.getPageInfo(prm.originNode);
    // request.pageInfo.name = null; // To avoid inclusion of "_originpage" parameter to the request
    request.waitForResponse = (prm.waitForResponse == true);

    request.send();

  };

  /** @private */
  this.refreshAreasResponseHandler = function(response) {

    AjaxMgr.setScripts(response);
    if (!response.pendingJSLoading) {
     AjaxMgr.refreshAreasContinue(response);
    }

  };

  /** @private */
  this.refreshAreasContinue = function(response,loadIndex) {

    if (loadIndex != null) {
      response.pendingJSLoading[loadIndex] = "loaded";
      for (var i = 0; i < response.pendingJSLoading.length; i++) {
        if (response.pendingJSLoading[i] == "pending") {
          return;
        }
      }
    }
    response.pendingJSLoading = null;

    var request = response.request;

    var targetAreas = null;
    if (response.isProcessResponse()) {
      targetAreas = response.props.targetAreas;
      if (targetAreas.after) targetAreas = AjaxMgr.getElementsArray(targetAreas);
    } else {
      targetAreas = request.props.targetAreas;
    }

    if (response.json.html) {
      for (i = 0; i < response.json.html.length; i++ ) {
        var target = OMgr.getObject(targetAreas[i]);
        if (!target) {

          var ex = new Exception("TARGET_AREA_MISSING",i+1);
          if (request.errorHandler) {
            var handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
            handler.run(ex, request);
          } else throw ex;

        } else {
          target.node.innerHTML = response.json.html[i];
          OMgr.initialize(target.node);
        }
      }
    }

    var tas = target.node.getElementsByTagName("textarea");
    for (i = 0; i < tas.length; i++) {
      var ta = tas[i];
      if (ta.value == "$remove-this-from-textarea$") {
       ta.value = "";
      }
    }

    if (response.json.newSourceAreas) {
      for (i = 0; i < response.json.newSourceAreas.length; i++ ) {
        var newSourceArea = response.json.newSourceAreas[i];
        AjaxMgr.addNewSourceArea(newSourceArea);
      }
    }

    if (response.json.styles) {
      AjaxMgr.addStyles(response.json.styles);
    }
  };

  /** @private */
  this.addStyles = function(styles) {

    var head = document.getElementsByTagName("head")[0];
    var styleElement = document.createElement('style');
    styleElement.type = "text/css";
    head.insertBefore(styleElement,head.childNodes[0]);
    if (WinMgr.isIE) {
      var styleSheet = document.styleSheets[0];
      styleSheet.cssText = styles;
    } else {
      styleElement.innerHTML = styles;
    }

  };

  /** @private */
  this.setScripts = function(response) {

    var externals = response.json.extjs;
    var embeddeds = response.json.embjs;
    var extcss = response.json.extcss;

    if (!externals && !embeddeds && !extcss) return false;

    var head = document.getElementsByTagName("head")[0];

    if (externals) {
      for (i = 0; i < externals.length; i++) {
        var src = externals[i];
        if (!AjaxMgr.scriptAlreadyExists(head,src,null)) {
          AjaxMgr.appendScript(head,src,null,response);
        }
      }
    }
    if (embeddeds) {
      for (i = 0; i < embeddeds.length; i++) {
        var text = embeddeds[i];
        if (!AjaxMgr.scriptAlreadyExists(head,null,text)) {
          AjaxMgr.appendScript(head,null,text,response);
        }
      }
    }
    if (extcss) {
      for (i = 0; i < extcss.length; i++) {
        var css = extcss[i];
        if (!AjaxMgr.cssAlreadyExists(head,css)) {
          AjaxMgr.appendExternalCss(head,css);
        }
      }
    }

    return true;

  };

  /** @private */
  this.scriptAlreadyExists = function(head,src,text) {
    for (var i = 0; i < head.childNodes.length; i++) {
      var node = head.childNodes[i];
      if (node.nodeType == 1) {
        if (src && node.src && node.src.endsWith(src)) return true;
        if (text && node.text && node.text.endsWith(text)) return true;
      }
    }
    return false;
  };

  /** @private */
  this.cssAlreadyExists = function(head,css) {
    for (var i = 0; i < head.childNodes.length; i++) {
      var node = head.childNodes[i];
      if (node.nodeType == 1) {
        if (node.href && node.href.endsWith(css)) return true;
      }
    }
    return false;
  };

  /** @private */
  this.appendScript = function(head,src,text,response) {
    var script = document.createElement('script');
    script.type = 'text/javascript';
    if (text) {
      script.text = text;
      head.appendChild(script);
    }
    if (src) {
      script.src = src;
      head.appendChild(script);
      if (!response.pendingJSLoading) response.pendingJSLoading = new Array();
      response.pendingJSLoading[response.pendingJSLoading.length] = "pending";
      var x = response.pendingJSLoading.length;
      if (!WinMgr.isIE) script.onload = function() {AjaxMgr.refreshAreasContinue(response,x-1)};
      else script.onreadystatechange = function() {
        if (script.readyState == "loaded" || script.readyState == "complete") {
          AjaxMgr.refreshAreasContinue(response,x-1);
        }
      }
    }
  };

  /** @private */
  this.appendExternalCss = function(head,css) {
    var link = document.createElement('link');
    link.rel = "stylesheet";
    link.type = 'text/css';
    link.href = css;
    head.appendChild(link);
  };


  /**
   * This method utilizes the {@link AjaxRequest} class to send a XMLHttpRequest to the AjaxActionService on the server.
   * @param prm The parameters object. The AjaxActionService class invokes the configured data service and prepares
   * an AjaxResponse out of the ADOM prepared by it. The AjaxResponse received from the service should be handled
   * by a custom handler function.
   *
   * <ul>
   * <li><b>prm.actionName</b> - The name of the <CODE>actionParams</CODE> configuration
   * element in the MoreMotion configuration.
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE>
   * form
   * </li><li><b>prm.responseHandler</b> - The Javascript function that is responsible for handling the response.
   * The function receives {@link AjaxResponse} object as the parameter. Note that response handler can also be
   * specified in action configuration defined with <CODE>prm.actionName</CODE> property.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </li></ul>
   * <p>
   * <u>Example:</u><br/><pre>
   * AjaxMgr.customRefresh(&#123; <br/>
   *   actionName       : 'RefreshModelNames' <br/>
   *   ,requestParams   : 'CAT=' + this.value <br/>
   * &#125;);<br/>
   * <br/>
   * &lt;actionParams name="RefreshModelNames"&gt;<br/>
   *   &lt;dataSource&gt;model_names&lt;/dataSource&gt;<br/>
   *   &lt;nodesToEncrypt&gt;ID&lt;/nodesToEncrypt&gt;<br/>
   *   &lt;responseHandler&gt;MyResponseHandler&lt;/responseHandler&gt;<br/>
   * &lt;/actionParams&gt;
   *
   * </pre>
   *
   */
  this.customRefresh = function(prm) {

    var props = new Object();
    var params = new RequestParameterAccumulator(prm.requestParams);

    params.add("_cfg",prm.actionName);

    props.userData = prm.userData;

    request = new AjaxRequest(params,null,props);

    request.service = "AjaxActionService.doms";
    request.pageInfo = OMgr.getPageInfo(prm.originNode);
    request.errorHandler = prm.errorHandler;
    request.busyImage = prm.busyImage;
    request.waitForResponse = (prm.waitForResponse == true);

    request.send();
  };



  /** @private */
  this.multiPartResponseHandler = function(mpar) {
    var ra = new Array();
    for (var i = 0; i < mpar.responseCount(); i++) {
      var response = mpar.getResponse(i);
      var handler = response.request.responseHandler;
      if (!handler) handler = eval(response.json.handler);
      if (handler.after) handler = eval(handler);
      handler(response);
      ra[ra.length] = response;
    }

    var callbackFunc = mpar.request.props.callbackFunc;
    if (callbackFunc) {
      AjaxMgr._callCallBack(mpar,ra,callbackFunc);
    }

  };
  
  /** @private */
  this._callCallBack = function(mpar,ra,func) {
    var ready = true;
    for (var i = 0; i < ra.length; i++) {
      var response = ra[i];
      if (response.pendingJSLoading) {
        ready = false;
        break;
      }
    }
    if (ready) {
      func(mpar);  
    } else {
      setTimeout(function(){AjaxMgr._callCallBack(mpar,ra,func)},100);
    }
  };

  /**
   * Cancels the previously sent request with the specified controlId
   * @param controlId The control Id of the request to cancel
   */
  this.cancelRequest = function(controlId) {
    if (controlId) {
      var rs = ajaxRequestSequences[controlId];
      if (rs) {
        rs++;
        ajaxRequestSequences[controlId] = rs;
      }
    }
  };


  /**
   * Prepares an Ajax request out of the specified process form and submits it as an XMLHtttpRequest.
   *
   * @param prm The Parameters Object
   * <ul>
   * <li><b>prm.formName</b> - The name of the {@link ProcessForm}
   * </li><li><b>prm.commandName</b> The name of the {@link ProcessCommand}
   * </li><li><b>prm.requiredBlocks</b> The names of the blocks (separated with commas) to be
   * validated before submitting the form.
   * </li><li><b>prm.beforeSubmitFunc</b> The name of the javascript function to be called before the
   * request is submitted. The function receives {@link ProcessForm} object and the command name.
   * </li><li><b>prm.validateInput</b> Flag that denotes that Field validations should be performed or not.
   * </li><li><b>prm.confirmMessage</b> The confirmation message text to be prompted to the user.
   * </li><li><b>prm.busyImage</b> - The Html Id of the object itself of the image element to be used as the busy image. This image will be
   * displayed on the page until the response is received from the server and processed.
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the target <code>SELECT</code> Element
   * is refreshed.
   * The function receives {@link MultiPartAjaxResponse} object as the parameter.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </ul>
   * <p><b><u>Note:</u> </b>Parameters <CODE>requiredBlocks, beforeSubmitFunc, validateInput, confirmMessage</b></CODE>and <CODE><b>busyImage</b></CODE>
   * can also be defined in the <CODE>mo:props</CODE> attribute of a {@link ProcessCommand} element.
   * If that is the case these parameters can be omitted here.
   * <p>
   * <u>Example:</u><br/><pre>
   * AjaxMgr.submitProcessForm(&#123; <br/>
   *   formName       : 'PF1' <br/>
   *   ,commandName   : 'Delete' <br/>
   *   ,requestParams : 'CAT=' + this.value <br/>
   *   ,busyImage     : 'Busy1' <br/>
   * &#125;);
   *
   * </pre>
   *
   * @return <code>true</code> if form is submitted <code>false</code> if submittion is cancelled because of a
   * validation error.
   *
   * @type boolean
   *
   */
  this.submitProcessForm = function(prm) {
    return PMgr.getForm(prm.formName).ajaxSubmit(prm);
  };


  /** @private - for compatibility */
  this.executeProcessTemplate = function(prm) {
    this.executeAjaxCommand(prm);
  };

  /**
   * Executes the specified Ajax Command on the server.
   * This method utilizes the {@link AjaxRequest} class to send a XMLHttpRequest to the MoreMotion Ajax Service
   * on the server.
   * @param prm The parameter object
   * <ul>
   * <li><b>prm.commandName</b> - The name of the ajaxProcessCommand configuration element.
   * </li><li><b>prm.requestParams</b> - The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the target elements are refreshed.
   * The function receives {@link MultiPartAjaxResponse} object as the parameter.
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.busyImage</b> - The id or the object itself of an Image element to be displayed until a response is received from
   * the server and processed.
   * </li><li><b>prm.errorHandler</b> - The name of the function to be called when an error occurs.
   * The function receives {@link Exception} and {@link AjaxResponse} objects as the parameters.
   * </li><li><b>prm.waitForResponse</b> - Flag to instruct the method to function synchronously. (Not Recommended)
   * </li><li><b>prm.originNode</b> - The handle of the DOM Node that originates this request.
   * This parameter should be supplied only if the call is made from a DOM node that was fetched
   * with a {@link #refreshAreas} method call. It is required to access to the enclosing node that
   * contains a <CODE>mo:pageInfo</CODE> attribute.
   * </li></ul>
   * <p>
   * <u>Example:</u><br/><pre>
   * AjaxMgr.executeAjaxCommand(&#123; <br/>
   *   commandName     : 'CartProds', <br/>
   *   requestParams   : 'CAT=' + category, <br/>
   *   busyImage       : 'bimg1' <br/>
   * &#125;);
   * </pre>
   */
  this.executeAjaxCommand = function(prm) {

    var props = new Object();

    props.callbackFunc = prm.callbackFunc;
    props.userData = prm.userData;

    if (prm.confirmMessage && !confirm(prm.confirmMessage)) {
      return false;
    }

    var params = new RequestParameterAccumulator(prm.requestParams);

    params.add("_cmd",prm.commandName);
    params.add("_usedsourceareas",AjaxMgr.getAlreadyUsedSourceAreas());
    params.add("_targetpage",OMgr.getPageInfo().name);

    var func = prm.beforeSubmitFunc;
    if (func) {
      try {
        if (func.after) func = eval(func);
        if (func(this, prm.commandName) == false) return false;
      } catch (e) {
        alert(OMgr.resource("INVALID_BEFORE_SUBMIT_FUNCTION_DEFINITION",func + " : " + e.message));
        return false;
      }
    }

    var request = new AjaxRequest(params,null,props);

    request.service = "AjaxProcessManager.doms";
    request.errorHandler = prm.errorHandler;
    request.busyImage = prm.busyImage;
    request.pageInfo = OMgr.getPageInfo(prm.originNode);
    request.waitForResponse = (prm.waitForResponse == true);

    request.send();

    return true;
  };

  /**
   * This method is used to store/retrieve states of browser user controls to/from MoreMotion Page Variables on the server 
   * with XMLHttpRequest
   * @param prm The parameter object
   * <ul>
   * <li><b>prm.getVars</b> - String that contains the names of the ViewState variables (Separated with commas) 
   * to retrieve from MoreMotion.
   * </li><li><b>prm.setVars</b> - The property object that contains the names and the values of the 
   * ViewState variables to let MoreMotion to store them.
   * </li><li><b>prm.callbackFunc</b> - The Javascript function to be called after the response to XMLHttpRequest is received. 
   * The function receives the properties object as the first parameter that contains the ViewState variables retrieved 
   * from MoreMotion and the data object as the second parameter. 
   * </li><li><b>prm.userData</b> - The user propertis object to store user data.
   * </li><li><b>prm.originNode</b> - The handle of the DOM Node that originates this request.
   * This parameter should be supplied only if the call is made from a DOM node that was fetched
   * with a {@link #refreshAreas} method call. It is required to access to the enclosing node that
   * contains a <CODE>mo:pageInfo</CODE> attribute.
   * </li></ul>
   * <p>   
   * <u>Examples:</u><br/><pre>
   * AjaxMgr.viewState(&#123; <br/>
   *   getVars      : 'ScrollPosition,SliderIsOpen', <br/>
   *   callbackFunc : function(vars) {alert(vars.ScrollPosition + " " + vars.SliderIsOpen)} <br/>
   * &#125;); <br/>
   * <br/>
   * AjaxMgr.viewState(&#123; <br/>
   *   setVars      : {ScroolPosition:1, SliderIsOpen:document.getElementById('x123').checked} <br/>
   * &#125;);
   * </pre>
   */
  this.viewState = function(prm) {
    
    req = AjaxRequest.getXmlHttpRequest();
    req.onreadystatechange = function() {

      if (req == null) { return; }
      if (req.readyState == 4) {
        if (req.status == 200) {
          try {
            var resp = eval("(" + req.responseText + ")");
            if (prm.callbackFunc) prm.callbackFunc(resp, prm.userData);
          } catch (ex) {alert(ex.message)}
        }  
      }
    };
    
    var pi = OMgr.getPageInfo(prm.originNode);
    var params = new RequestParameterAccumulator(prm.setVars);

    if (pi.name) params.add("_originpage", pi.name);
    if (pi.variant) params.add("_variant", pi.variant);

    if (pi.lang && pi.lang != "ml") params.add("_lang",pi.lang);

    params.add("_resp_enc",pi.charset);

    var reqenc = "";

    if (WinMgr.isIE) {
      reqenc = pi.charset;
      if (reqenc.toLowerCase() == "utf-8") {
        if (pi.clang == "tr") reqenc = "iso-8859-9";
        else reqenc = "iso-8859-1";
      }

    } else if (WinMgr.isGecko) {
      reqenc = pi.charset;

    } else {
      reqenc = "utf-8";
    }

    params.add("_enc",reqenc);
    params.add("_rand",(new Date()).getTime());
    
    if (prm.getVars) params.add("_vars",prm.getVars);
    req.open("GET", "AjaxViewStateService.doms" + "?" + params.toString());
    req.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    req.send(null);
    
  };
  

}

/** Global Singleton MoreMotionAjaxManager object
 * @type MoreMotionAjaxManager
 */
var AjaxMgr = new MoreMotionAjaxManager();


/* Compatibility Functions */
/** @ignore */
function RefreshOptions(elmid, dsname, params, soq, valueNode, optionNode, callbackFunc, customData, waitForResponse) {
  AjaxMgr.refreshOptions({
    targetElement     : elmid
    ,dataSource       : dsname
    ,requestParams    : params
    ,staticOptionsQty : soq
    ,valueNode        : valueNode
    ,optionNode       : optionNode
    ,callbackFunc     : callbackFunc
    ,userData         : customData
    ,waitForResponse  : waitForResponse
  });
}

/** @ignore */
function ClearOptions(elmid, soq) {
  AjaxMgr.clearOptions(elmid, soq);
}

/** @ignore */
function RefreshFields(dsname, params, fields, elmids, attrs, callbackFunc, customData, waitForResponse) {
  AjaxMgr.refreshFields({
    dataSource          : dsname
    ,requestParams      : params
    ,targetElements     : elmids
    ,targetElementAttrs : attrs
    ,callbackFunc       : callbackFunc
    ,userData           : customData
    ,waitForResponse    : waitForResponse
  });
}

/** @ignore */
function RefreshArea(targetAreaIds, sourceAreas, sourcePage, params, callbackFunc, customData, waitForResponse) {
  AjaxMgr.refreshAreas({
    targetAreas      : targetAreaIds
    ,sourceAreas     : sourcePage + '.' + sourceAreas
    ,requestParams   : params
    ,callbackFunc    : callbackFunc
    ,userData        : customData
    ,waitForResponse : waitForResponse
  });
}


/** @private */
var ajaxRequestSequences = new Array();

/**
 * @class
 * Represents a MoreMotion Ajax Request that is responsible for preparing an xmlHttpRequest, sending it,
 * receiving the response and converting the response into the speficied response type.
 * The supported Response types are {@link AjaxResponse} and  {@link MultiPartAjaxResponse}
 * @param requestParams The request parameters.
 * It can be a single string in <CODE>'p1=p1val&p2='+this.value</CODE>form,
 * a properties Object <CODE>{p1:'p1val',p2:this.value}</CODE> form or
 * a {@link RequestParameterAccumulator} object.
 * @param responseHandler The Javascript function that to be called after the
 * response is received from MoreMotion Ajax Service
 * @param props The properties object to store user data
 */
function AjaxRequest(requestParams, responseHandler, props) {

  /** @private */
  this.xmlHttpRequest = null;

  /** Request Parameters.
   * @type RequestParameterAccumulator
   */
  this.requestParams = requestParams.addObject ? requestParams : new RequestParameterAccumulator(requestParams);

  /**
   * The Javascript function responsible for handling the response
   * @type function
   */
  this.responseHandler = responseHandler;

  /**
   * The Javascript function responsible for handling the errors
   * @type function
   */
  this.errorHandler = null;

  /**
   * The properties object to store user data
   */
  this.props = props || new Object();

  /** Html Id or the object of a DOM element that will be visible until the response is received and processed.
   * @type String_or_HTMLElement
   */
  this.busyImage = null;

  /** The Request Encoding. If ignored then <code>OMgr.getPageInfo().charset</code> is used.
   * @type String
   */
  this.requestEncoding = null;

  /** The Response Encoding. If ignored then <code>OMgr.getPageInfo().charset</code> is used.
   * @type String
   */
  this.responseEncoding = null;

  /** Flag that specifies that the {@link #send} should not return until
   * a response is received from the Ajax Service.
   * @type boolean
   */
  this.waitForResponse = false;

  /**
   * The name of the Ajax Service on the server. The default is <code>AjaxService.doms</code>
   * @type String
   */
  this.service = "AjaxActionService.doms";

  /**
   * The Request Method. The default is "GET".
   * @type String
   */
  this.method = "GET";

  this.pageInfo = null;

  /** @private */
  this.setBusy = function() {
    if (this.busyImage) {
      var elm = this.busyImage.after ? document.getElementById(this.busyImage) : this.busyImage;
      if (elm) elm.style.visibility = "visible";
    }
  };

  /** @private */
  this.getUrl = function() {
    return this.service;
  };

  /** @private */
  this.getParams = function(method) {

    if (this.pageInfo) {
      if (this.pageInfo.after) this.pageInfo = new PageInfo(this.pageInfo);
    } else this.pageInfo = OMgr.getPageInfo(null);

    var params = this.requestParams;

    if (this.pageInfo.name) params.add("_originpage", this.pageInfo.name);
    if (this.pageInfo.variant) params.add("_variant", this.pageInfo.variant);

    if (this.pageInfo.lang && this.pageInfo.lang != "ml") params.add("_lang",this.pageInfo.lang);

    var respenc = this.responseEncoding;
    if (respenc == null) {
      respenc = this.pageInfo.charset;
      this.responseEncoding = respenc;
    }
    params.add("_resp_enc",respenc);

    var reqenc = "utf-8";

    if (method != "POST") {
     reqenc = this.requestEncoding;
     if (reqenc == null) {

       if (WinMgr.isIE) {
         reqenc = this.pageInfo.charset;
         if (reqenc.toLowerCase() == "utf-8") {
           if (this.pageInfo.clang == "tr") reqenc = "iso-8859-9";
           else reqenc = "iso-8859-1";
         }

       } else if (WinMgr.isGecko) {
         reqenc = this.pageInfo.charset;

       } else {
         reqenc = "utf-8";
       }

       this.requestEncoding = reqenc;
     }
    }

    params.add("_enc",reqenc);
    params.add("_rand",(new Date()).getTime());

    return params.toString();
  };


  /**
   * Sends the request to the Ajax Service that is specified with member field {@link #service}
   * @param controlId The identity that is used to make this request cancelable.
   * By using the value of this parameter, the delayed responses can be ignored or a request can be explicitly
   * cancelled with {@link MoreMotionAjaxManager#cancelRequest} method
   */
  this.send = function(controlId) {
    this.controlId = controlId;
    SendAjaxRequest(this);
  };


}


/** @ignore */
function SendAjaxRequest(request) {

  if (request.controlId) {
    var rs = ajaxRequestSequences[request.controlId];
    if (!rs) rs = 1; else rs++;
    request.requestSequence = rs;
    ajaxRequestSequences[request.controlId] = rs;
  }

  request.xmlHttpRequest = AjaxRequest.getXmlHttpRequest();
  request.xmlHttpRequest.onreadystatechange = function() {

    if (request == null || request.xmlHttpRequest == null) { return; }
    if (request.xmlHttpRequest.readyState == 4) { AjaxRequestStatusChange(request); }

  };

  var async = !(request.waitForResponse && WinMgr.isIE);

  request.setBusy();

  if (request.method.toUpperCase() == "GET") {

   request.xmlHttpRequest.open("GET", request.getUrl() + "?" + request.getParams("GET"), async);
   // request.xmlHttpRequest.setRequestHeader("Content-Type","text/plain; charset=" + request.requestEncoding);
   request.xmlHttpRequest.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
   request.xmlHttpRequest.send(null);

  } else if (request.method.toUpperCase() == "POST") {

   var params = request.getParams("POST");
   request.xmlHttpRequest.open("POST", request.getUrl(), async);
   request.xmlHttpRequest.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
   request.xmlHttpRequest.setRequestHeader("Content-Length", params.length);
   if (!WinMgr.isIE) request.xmlHttpRequest.setRequestHeader("Connection", "close");
   request.xmlHttpRequest.send(params);

  } else {
   alert("Invalid method!" + request.method);
  }

}

/** @ignore */
function AjaxRequestStatusChange(request) {

  var status = "No Response!";
  try {
    status = request.xmlHttpRequest.status;
  } catch(e) {}

  if (status == 200) {

    try {

      var responseText = request.xmlHttpRequest.responseText;

      if (responseText.startsWith("!ERROR:")) {

        var ex = new Exception("AJAX_SERVICE_ERROR", responseText.after("!ERROR:"));
        if (request.errorHandler) {
          var handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
          handler.run(ex, request);
        } else throw ex;
      }

      if (request.controlId) {
        var rs = ajaxRequestSequences[request.controlId];
        if (request.requestSequence < rs) return;
      }

      var resp = GetMultiPartAjaxResponse(responseText, request);
      AjaxMgr.multiPartResponseHandler(resp);

    } catch(e) {
      if (e.alert) e.alert();
      else throw e; // alert(e.message);

    } finally {

      if (request.busyImage) {
        var elm = request.busyImage.after ? document.getElementById(request.busyImage) : request.busyImage;
        if (elm) elm.style.visibility = "hidden";
        else alert( OMgr.resource("BUSY_IMAGE_MISSING") );
      }

    }

  } else {

    /*

    var ex = new Exception("AJAX_REQUEST_ERROR",status);
    if (request.errorHandler) {
      var handler = new ErrorHandler(request.errorHandler,"Ajax Request Error Handler");
      try {
        handler.run(ex, request);
      } catch (e) {
        if (e.alert) e.alert();
        else alert(e.message);
      }
    } else ex.alert();

    */

  }

}


/** @private */
function GetAjaxResponse(responseText, request) {

  var dlm = "$_T_E_X_T_$";
  var x = responseText.indexOf(dlm);

  var response = null;
  if (x == -1) {
    response = new AjaxResponse(responseText);
  } else {
    response = new AjaxResponse(responseText.substring(0,x));
    response.setText(responseText.substring(x + dlm.length+1));
  }
  response.request = request;
  return response;

}


/** @private */
function GetMultiPartAjaxResponse(responseText, request) {

  var mpar = new MultiPartAjaxResponse();
  mpar.request = request;

  var responseParts = responseText.split("$_RESPONSE_SEPARATOR_$");
  for (var i = 0; i < responseParts.length; i++) {
    var responsePart = responseParts[i];
    if (responsePart.trim().length > 0) {
      var response = GetAjaxResponse(responsePart,request);
      response.owner = mpar;
      if (response.json && response.json.type == "*ResultFields") {
        mpar.setResultFields(response);
      } else if (response.json && response.json.type == "*PagingInfo") {
        mpar.setPagingInfo(response);
      } else {
        mpar.addResponse(response);
      }
    }
  }

  return mpar;
}


/** @ignore */
function tme() {
  return (new Date()).getTime();
}

/** @ignore */
AjaxRequest.getXmlHttpRequest = function() {

 if (window.XMLHttpRequest) {
   return new XMLHttpRequest();
 }
 else if (window.ActiveXObject) {
   try {
     return new ActiveXObject("Msxml2.XMLHTTP");
   } catch (e) {
     try {
       return new ActiveXObject("Microsoft.XMLHTTP");
     } catch (e) {
       return null;
     }
   }
 }
 else {
   return null;
 }

};


/** @ignore */
function ShowDropDowns(div) {

  if (!WinMgr.isIE) return;

  var s = document.getElementsByTagName("SELECT");
  for (var i = 0; i < s.length; i++) {
    var dd = s[i];
    var zIndex = dd.style.zIndex;
    if (zIndex > 9000) {
      dd.style.visibility = "";
      dd.style.zIndex = zIndex - 10000;
    }
  }

}

/** @ignore */
function HideDropDowns(div, divL, divT) {

  if (!WinMgr.isIE) return;

  var divR = 0 + divL + div.childNodes[0].offsetWidth;
  var divB = 0 + divT + div.childNodes[0].offsetHeight;

  var s = document.getElementsByTagName("SELECT");
  for (var i = 0; i < s.length; i++) {
    var dd = s[i];
    if (dd.style.visibility == "" || dd.style.visibility == "visible")
    {
      var ddL = findPosX(dd);
      var ddT = findPosY(dd);
      var ddR = ddL + dd.offsetWidth;
      var ddB = ddT + dd.offsetHeight;

      if ((ddL >= divL && ddL <= divR) || (ddR >= divL && ddR <= divR) || (ddL < divL && ddR > divR) )
        if ((ddT >= divT && ddT <= divB) || (ddB >= divT && ddB <= divB) || (ddT < divT && ddB > divB) ) {
          dd.style.zIndex += 10000;
          dd.style.visibility = "hidden";
        }
    }
  }

}


/** @ignore */
function findPosX(obj) {
  var curleft = 0;
  if (obj.offsetParent) {
    while (obj.offsetParent) {
      curleft += obj.offsetLeft;
      obj = obj.offsetParent;
    }
  } else if (obj.x) {
    curleft += obj.x;
  }
  return curleft - 1;
}

/** @ignore */
function findPosY(obj) {
  var ie = WinMgr.isIE;
  var curtop = 0;
  if (obj.offsetParent) {
    while (obj.offsetParent) {
      var top = ie && obj.tagName == "DIV" && obj.offsetTop % 2 == 0 ? obj.offsetTop / 2 : obj.offsetTop;
      curtop += top;
      obj = obj.offsetParent;
    }
  } else if (obj.y) {
    curtop += obj.y;
  }
  return curtop;
}


/**
 * @class
 * Represents a Response Record that is created by {@link AjaxResponse#get} and  {@link AjaxResponse#next} methods
 * @param owner The owner {@link AjaxResponse} object
 * @param recordIndex The index number of this ResponseRecord in the AjaxResponse
 */
function ResponseRecord(owner, recordIndex) {

  /** The AjaxResponse object that owns this ResponseRecord
   * @type AjaxResponse
   */
  this.owner = owner;

  /** The index number of this ResponseRecord in the owner {@link AjaxResponse}
   * @type int
   */
  this.recordIndex = recordIndex;


  /** Returns the names of the fields that exist in this Response Item
   * @type String[]
   */
  this.fields = function() {
    var flds = null;
    try {
      flds = this.owner.json.item[this.recordIndex];
    } catch (e) {}
    return flds;
  };

  /** Returns <code>true</code> if specified field exists in this Response Item <code>false</code> otherwise
   * @type boolean
   */
  this.hasField = function(fld) {
    return this.owner.hasField(fld);
  };

  /** Returns the value of the specified field.
   * @param fieldName the name of the field in this ResponseRecord
   * @type String
   */
  this.getFieldValue = function(fieldName) {
    var s = null;
    try {
      var fidx = -1;
      if (!isNaN(fieldName)) {
        fidx = fieldName + 0;
      } else {
        var meta = this.owner.json.meta;
        for (var i = 0; i < meta.length; i++) {
          if (meta[i] == fieldName) {
            fidx = i;
            break;
          }
        }
      }
      if (fidx == -1) return null;
      s = this.owner.json.item[this.recordIndex][fidx];
    } catch (e) {}
    return s;
  };

  /** @private */
  this.get = function(fld) {
    return this.getFieldValue(fld);
  };


}

/**
 * class Paging Information
 * The objects of this class are created by {@link AjaxResponse} class
 */
function PagingInfo(str) {

  this.currentPage = null; //-<
  this.itemsPerPage = null; //-<
  this.pagesPerBlock = null; //-<
  this.numberOfItems = null; //-<
  this.lastPage = null; //-<
  this.morePages = null; //-<
  this.fromPage = null; //-<
  this.toPage = null; //-<

  this.parse = function(str) {
    var settings = str.split(";");
    for (var i = 0; i < settings.length; i++) {
      var s = settings[i].trim();
      var n = s.before(":","").trim();
      var v = s.after(":","").trim();
      if (n && v) {
        if (n == "cp") this.currentPage = v * 1;
        else if (n == "ipp") this.itemsPerPage = v * 1;
        else if (n == "ppb") this.pagesPerBlock = v * 1;
        else if (n == "noi") this.numberOfItems = v * 1;
        else if (n == "lp") this.lastPage = v == "true";
        else if (n == "mp") this.morePages = v == "true";
        else if (n == "fp") this.fromPage = v * 1;
        else if (n == "tp") this.toPage = v * 1;
      }
    }

  };

  if (str) this.parse(str);

}

/**
 * class Response Data Structure
 * The objects of this class are created by {@link AjaxResponse} class
 */
function JsonResponse() {

  /** Response Type set by AjaxProcess classes
   * @type String
   */
  this.type = null;

  /**
   * Response properties provided by AjaxProcess classes
   * @type Object
   */
  this.props = null;
  /**
   * Result field names array. Array[fieldIndex]
   * @type String[]
   */
  this.meta = null;

  /**
   * Result fields data. Array[recordIndex][fieldIndex]
   * @type String[][]
   */
  this.item = null;
}

/**
 * @class
 * Represents an Ajax Response. Object of this class is created by the {@link MultiPartAjaxResponse} object.
 * An object of this class contains information only for one type of refresh action.
 * If more than one refresh actions is requested in one go then multiple objects of this
 * class will be created and contained in {@link MultiPartAjaxResponse} object.
 */
function AjaxResponse(responseText) {

  /**
   * The owner MultiPartAjaxResponse object
   * @type MultiPartAjaxResponse
   */
  this.owner = null;
  /**
   * The AjaxRequest object that originates this response
   * @type AjaxRequest
   */
  this.request = null;

  /**
   * Returns the value of the specified Result Field. The result fields are set
   * by the "mor.ajax.setResultField(name,value)" Ajax MScript function during the execution of the request on the server.
   * @param fieldName The name of the result field
   */
  this.getResultField = function(fieldName) {
    return this.owner ? this.owner.getResultField(fieldName) : null;
  };

  /**
   * Returns the PagingInfo object associated with the given data soure name
   * @param dsname The name of the data source
   * @type PagingInfo
   */
  this.getPagingInfo = function(dsname) {
    return this.owner ? this.owner.getPagingInfo(dsname) : null;
  };

  /** @private */
  this.recordIndex = -1;

  /** @private */
  this.itemCount = 0;
  /** The number of the {@link ResponseRecord} objects contained in this Response
   * @type int
   */
  this.recordCount = 0;

  /**
   * Response properties set by the process classes executed on the server
   */
  this.props = null;

  /**
   * The response data received from the Ajax Service in Json structure
   * @type JsonResponse
   */
  this.json = null;
  try {
    this.json = eval("(" + responseText + ")");
    if (this.json.item) {
     this.recordCount = this.json.item.length;
     this.itemCount = this.json.item.length; // For compatibility
    }
    this.props = this.json.props;
  } catch (e) {alert(OMgr.resource("NOT_VALID_JSON_RESPONSE") + "\n" +  e.message)}


  /**
   * Returns <CODE>true</CODE> if this response is created by a process
   * @type boolean
   */
  this.isProcessResponse = function() {
    return this.json.name && this.json.name != ""  ;
  };

  /** @private */
  this.text = null;

  /** @private */
  this.setText = function(text) {
    this.text = text;
  };

  /**
   * Returns the plain text part of the response.
   * @type String
   */
  this.getText = function() {
    return this.text;
  };

  /**
   * Returns <code>true</code> when there are more ResponseRecord objects to retrieve with
   * {@link #next} method.
   * @type boolean
   */
  this.hasMore = function() {
    return this.recordIndex+1 < this.itemCount;
  };

  /**
   * Returns the next ResponseRecord object in this response
   * @type ResponseRecord
   */
  this.next = function() {
    this.recordIndex++;
    if (this.recordIndex == this.itemCount) {
      this.recordIndex--;
      return null;
    }
    return new ResponseRecord(this, this.recordIndex);
  };

  /**
   * Returns the ResponseRecord object referenced with the index number.
   * @param recordIndex The index number of the ResponseRecord
   * @type ResponseRecord
   */
  this.getResponseRecord = function(recordIndex) {
    if (recordIndex >= this.itemCount) {
      return null;
    }
    return new ResponseRecord(this, recordIndex);
  };

  /** @private */
  this.get = function(recordIndex) {
    return this.getResponseRecord(recordIndex);
  };


  /**
   * Wraps the current record index pointer to the top.
   */
  this.top = function() {
    this.recordIndex = -1;
  };

  /** Returns the names of the fields that exist in this Response
   * @type String[]
   */
  this.fieldNames = function() {
    return this.json.meta;
  };

  /** Returns <code>true</code> if specified field exists in this Response <code>false</code> otherwise
   * @type boolean
   */
  this.hasField = function(fld) {
    var meta = this.json.meta;
    for (var i = 0; i < meta.length; i++) {
      if (meta[i] == fld) {
        return true;
      }
    }
    return false;
  };

}

/**
 * @class
 * The Object of this class is created always by the {@link AjaxRequest} object upon receiving
 * a response to its XMLHttpRequest. It may contain one of more {@link AjaxResponse} objects and it contains
 * all the necessary properties and methods to access and manage the request and the response information.
 */
function MultiPartAjaxResponse() {

  /**
   * The AjaxRequest object that originates this response
   * @type AjaxRequest
   */
  this.request = null;

  /** @private */
  this.resultFields = null;

  /** @private */
  this.setResultFields = function(response) {
    this.resultFields = response.json.props;
  };
  /**
   * Returns the value of the specified Result Field. The result fields are set
   * by the "mor.Ajax.setResultField(name,value)" Ajax MScript function during the
   * execution of the request on the server.
   * @param fieldName The name of the result field
   * @type String
   */
  this.getResultField = function(fieldName) {
    return this.resultFields[fieldName];
  }

  /** @private */
  this.pagingInfo = null;
  /** @private */
  this.setPagingInfo = function(response) {
    this.pagingInfo = response.json.props;
  };
  /**
   * Returns the PagingInfo object associated with the given data soure name
   * @param dsname The name of the data source
   * @type PagingInfo
   */
  this.getPagingInfo = function(dsname) {
    var pistr = this.pagingInfo[dsname];
    return pistr ? new PagingInfo(pistr) : null;
  };

  /** @private */
  this.getPagingInfoStr = function(dsname) {
    return this.pagingInfo[dsname];
  };

  /**
   * Array of {@link AjaxResponse} objects
   * @type AjaxResponse[]
   */
  this.responses = new Array();

  /** @private */
  this.addResponse = function(response) {
    this.responses[this.responses.length] = response;
  }

  /**
   * Returns the number of the AjaxResponses contained in this object
   * @type integer
   */
  this.responseCount = function() {
    return this.responses.length;
  };

  /**
   * Returns the AjaxResponse object associated with the specified name
   * @param indexOrName The index number or the name name of the AjaxResponse. If not specified
   * the firts response is returned.
   * @type AjaxResponse
   */
  this.getResponse = function(indexOrName) {
    if (!indexOrName) return this.responses[0];
    if (indexOrName.after) {
      for (var i = 0; i < this.responses.length; i++) {
        var response = this.responses[i];
        if (response.name == indexOrName) {
          return response;
        }
      }
      return null;
    } else {
      return this.responses[indexOrName];
    }
  };

  /**
   * Returns the AjaxRequest object that originates this response
   * @type AjaxRequest
   */
  this.getRequest = function() {
    return this.responses.length > 0 ? this.responses[0].request : null;
  };


}
