/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the 
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains common utility classes.
 * @author Erkan Cengiz
 * @version $Id: Common.js 193 2008-08-14 16:09:44Z erkan $
 */

var PENDING_EVENT_Q = 0;

/**
 * Prints the given string to an editarea element on the current page.
 * @param s The string to print
 * Note: This function requires an <CODE>textarea</CODE> element on the current page
 * whose html id is "alert".
 */
function alrt(s) {
  var elm = document.getElementById('alert');
  if (elm) {
    if (elm.value == "") elm.value = s;
    else {
      elm.value += "\n" + s;
    }
  }    
}
/** @private */
function aleert(s) {
  alert(s);
}

/** @private */
var isIEResult = null;
/** @private */
function isIE() {
  if (isIEResult == null) isIEResult = navigator.userAgent.toLowerCase().indexOf("msie") > -1;
  return isIEResult;
}

/** @private - for compatibility */
function RefreshLink(href) {
  OMgr.refreshLink(href);
}

/** @private - for compatibility */
function RefreshHref(node) {
  OMgr.refreshHref(node); 
}

/** @private - for compatibility */
function RefreshAction(frm) {
  OMgr.refreshAction(frm);
}

/** @private */
function FocusAndSelect(elm) {
  try {
    if (!elm.id) elm = document.getElementById(elm);
    elm.focus();
    elm.select();
  } catch (e) {}
}

var re_25 = new RegExp( "%", "g" );
var re_26 = new RegExp( "&", "g" );
var re_2B = new RegExp( "[+]", "g" );


/** @private */
function escapex(str) {
  if (str.after) {
    return str.replace(re_25,"%25").replace(re_26,"%26").replace(re_2B,"%2B");
  } else return str;
}

function Stack() {

  this.array = new Array();

  this.push = function(o) {
    this.array.push(o);
  };

  this.set = function(o) {
    if (this.array.length == 0) this.array.push(o);
    else this.array[this.array.length-1] = o;
  };

  this.pop = function() {
    return this.array.pop();
  };

  this.peek = function(offset) {
    if (!offset) offset = 0;
    return this.array[this.array.length - 1 - offset];
  };

  this.hasItems = function() {
    return this.array.length > 0;
  };

  this.length = function() {
    return this.array.length;
  };

}

/**
 * Trims the space characters on both sides of this String
 * @return Trimmed String
 * @type String
 */
String.prototype.trim = function() {
  var s = this;
  var i = 0;
  var j = s.length - 1;
  while (i < s.length && s.charAt(i) == " ") i++;
  while (j > i && s.charAt(j) == " ") j--;
  return s.substring(i,j+1);
};

/**
 * Converts this String to boolean and returns it
 * @type boolean
 */
String.prototype.toBoolean = function() {
  return this.toLowerCase() == "true" || this.charAt(1) == "1";
};

/** @ignore */
String.prototype.toBool = function() {
  return this.toLowerCase() == "true" || this.charAt(1) == "1";
};

/**
 * Test if this String contains a valid number
 * @return true if this is a valid number false otherwise
 * @type boolean
 */
String.prototype.isNumber = function() {
  return !isNaN(this);
};

/**
 * Searches the first occurence of the specified delimiter string in this string and returns
 * the substring after it.
 * @param delimiter
 * @param defaultValue
 * @return the substring after the first occurence of the delimiter or the default value if delimiter is not found
 * @type String
 */
String.prototype.after = function(delimiter, defaultValue) {
  var i = this.indexOf(delimiter);
  if (i == -1) return defaultValue;
  return this.substring(i + delimiter.length);
};

/**
 * Searches the first occurence of the specified delimiter string in this string and returns
 * the substring before it.
 * @param delimiter
 * @param defaultValue
 * @return the substring before the first occurence of the delimiter or the default value if delimiter is not found
 * @type String
 */
String.prototype.before = function(delimiter,defaultValue) {
  var i = this.indexOf(delimiter);
  if (i == -1) return defaultValue;
  return this.substring(0,i);
};

/**
 * Searches the last occurence of the specified delimiter string in this string and returns
 * the substring after it.
 * @param delimiter
 * @param defaultValue
 * @return the substring after the last occurence of the delimiter or the default value if delimiter is not found
 * @type String
 */
String.prototype.afterLast = function(delimiter,defaultValue) {
  var i = this.lastIndexOf(delimiter);
  if (i == -1) return defaultValue;
  return this.substring(i + delimiter.length);
};

/**
 * Searches the last occurence of the specified delimiter string in this string and returns
 * the substring before it.
 * @param delimiter
 * @param defaultValue
 * @return the substring before the last occurence of the delimiter or the default value if delimiter is not found
 * @type String
 */
String.prototype.beforeLast = function(delimiter,defaultValue) {
  var i = this.lastIndexOf(delimiter);
  if (i == -1) return defaultValue;
  return this.substring(0,i);
};

/**
 * Test if this String starts with the given substring.
 * @return <code>true</code> if it does <code>false</code> otherwise
 * @type boolean
 */
String.prototype.startsWith = function(subString) {
  return subString == this.substring(0,subString.length);
};

/**
 * Test if this String ends with the given substring.
 * @return <code>true</code> if it does <code>false</code> otherwise
 * @type boolean
 */
String.prototype.endsWith = function(subString) {
  var diff = this.length - subString.length;
  return diff > -1 && this.substring(diff) == subString;
};

/**
 * Test if the length of this String is greter than zero.
 * @return <code>true</code> if it is <code>false</code> otherwise
 * @type boolean
 */
String.prototype.hasValue = function() {
  return this.trim().length > 0;
};


function GhostForm() {

  this.form = document.createElement("FORM");
  this.form = document.body.appendChild(this.form);

  this.addParameter = function(name, value) {

    var elm = OMgr.createElement("INPUT", name);
    elm.type = "hidden";
    elm.value = value;
    this.form.appendChild(elm);

  };

  this.submit = function(method) {
    if (!method) method = "POST";
    this.form.method = method;
    this.form.submit();
  };


}

/**
 * @class
 * This class represents an URL Address that includes MoreMotion specific parameters and it provides methods to
 * manipulate its parameters.
 *
 * @param url The full URL string
 * @return UrlAdress object
 *
 */
function UrlAddress(url) {

  /** The base part of the URL excluding the query string that contains the parameters */
  this.base = null;

  /**
   * Property that indicates a "_rand" parameter with a unique value is included into the parameters
   * when converting this instance to String.
   * @type boolean
   */
  this.includeRandParameter = true;

  /**
   * Property that indicates that the "_sendhtml=true" parameter to be included into the parameters to force the
   * MoreMotion to response as HTML not XML.
   * @type boolean
   */
  this.asHtml = true;

  /** @private */
  this.paramNames = new Array();
  /** @private */
  this.paramValues = new Array();

  /**
   * Adds a new parameter to this object
   * @param name The name of the parameter
   * @param value The value of the parameter
   */
  this.addParam = function(name, value) {
    this.paramNames[this.paramNames.length] = name;
    this.paramValues[this.paramValues.length] = value;
  };

  /**
   * Return the value of the given parameter
   * @param name The name of the parameter
   * @return The value of the parameter or <code>null</code> if the parameter is not found.
   * @type String
   */
  this.getParam = function(name) {
    for (var i = 0; i < this.paramNames.length; i++) {
      if (name == this.paramNames[i]) {
        return this.paramValues[i];
      }
    }
    return null;
  };

  /**
   * Removes the given parameter from this object
   * @param name The name of the parameter
   */
  this.removeParam = function(name) {
    for (var i = 0; i < this.paramNames.length; i++) {
      if (name == this.paramNames[i]) {
        this.paramNames[i] = null;
        this.paramValues[i] = null;
        break;
      }
    }
  };

  /**
   * Parses the given URL string and re-initializes this object.
   * @param url The full URL string
   */
  this.parse = function(url) {

    if (url.startsWith("http") || url.endsWith(".html") || url.indexOf("?") > -1) {
    } else {
      url = "display.doms?pg=" + url;
    }

    this.base = url.before("?",url);
    var paramsStr = url.after("?",null);
    if (paramsStr != null) {
      var params = paramsStr.split("&");
      for (var i = 0; i < params.length; i++) {
        var param = params[i];
        var name = param.before("=").trim();
        var value = param.after("=").trim();
        this.addParam(name,unescape(value));
      }
    }
  };

  if (url) this.parse(url);

  /**
   * Converts this object to full URL String and returns it
   * @type String
   */
  this.toString = function() {

    var s = this.base;
    var first = true;
    for (var i = 0; i < this.paramNames.length; i++) {
      if (this.paramNames[i] != null) {
        if (first) {first = false; s = s + "?";} else s = s + "&";
        s = s + this.paramNames[i] + "=" + escapex(this.paramValues[i]);
      }
    }

    if (this.asHtml) {
      if (first) {first = false; s = s + "?";} else s = s + "&";
      s = s + "_sendhtml=true";
    }

    if (this.includeRandParameter) {
      if (first) {first = false; s = s + "?";} else s = s + "&";
      s = s + "_rand=" + (new Date()).getTime();
    }

    return s;
  };

}


/** @ignore */
function Iterator() {

  /** @private */
  this.elements = new Array();

  /** @private */
  this.index = -1;

  /** @private */
  this.add = function(object) {
    this.elements[this.elements.length] = object;
  };

  this.hasMore = function() {
    return this.index < this.elements.length;
  };

  this.nextElement = function() {
    this.index++;
    return this.element[this.index];
  };

  this.rollUp = function() {
    this.index = -1;
  };

}

/**
 * @class
 * A utility class to accumulate strings delimited with the specified delimiter.
 * <p><u>Example:</u></p><code>
 * &nbsp;&nbsp;var a = new Accumulator(",");<br/>
 * &nbsp;&nbsp;a.add("AAA");<br/>
 * &nbsp;&nbsp;alert(a.toString()); // result "AAA"<br/>
 * &nbsp;&nbsp;a.add("BBB");<br/>
 * &nbsp;&nbsp;a.add("CCC");<br/>
 * &nbsp;&nbsp;alert(a.toString()); // result "AAA,BBB,CCC"<br/>
 * </code>
 */
function Accumulator(dlm) {
  /** @private */
  this.str = "";
  /** @private */
  this.dlm = dlm;

  /**
   * Adds the given string to this accumulator
   * @param s The string to add to the accumulator
   */
  this.add = function(s) {
    if (this.str.length > 0) {
      this.str += this.dlm;
    }
    this.str += s;
  };

  /**
   * Returns the accumulated string
   */
  this.toString = function() {
    return this.str;
  };

}


/** @private */
var RegExp_Quotes = new RegExp( "'", "g" );
/** @private */
var RegExp_NewLines = new RegExp( "\n", "g" );

function PropertyAccumulator() {

  /** @private */
  this.str = "";

  this.add = function(name, value) {
    if (this.str.length > 0) {
      this.str += ", ";
    }
    if (value) {
      if (value.indexOf) {
        if (value.indexOf("'") > -1) value = value.replace(RegExp_Quotes,"\\'");
        if (value.indexOf("\n") > -1) value = value.replace(RegExp_NewLines,"\\n");
      }
      this.str += name + ":'" + value + "'";
    }
  };

  /**
   * Returns the accumulated string
   */
  this.toString = function() {
    return this.str;
  };
}


/**
 * @class
 * This class is used to accumulate the parameter names and values of the process fields.
 * @param params The initial value.
 * It can be a single string in <CODE>'p1=p1val&p2='+this.value</CODE>form or
 * a properties Object <CODE>{p1:'p1val',p2:this.value}</CODE> form.
 * It is optional.
 */
function RequestParameterAccumulator(params) {

  /** @private */
  this.str = "";


  /**
   *  Adds the given parameter name and the value to this accumulator
   */
  this.add = function(name, value) {
   if (value != null) {
     if (this.str.length > 0) {
       this.str += "&";
     }
     this.str += name + "=" + escapex(value);
   }
  };

  /**
   * Adds the properties in the given property object to this accumulator
   */
  this.addObject = function(o) {
    if (o) {
      if (o.after) this.append(o);
      else {
        for (var p in o) {
          var pv = o[p];
          if (pv) {
            this.add(p,pv);
          }
        }
      }
    }
  };

  /** Appends the given string to this accumulator */
  this.append = function(str) {
    if (str) {
      if (this.str.length > 0) {
        this.str += "&";
      }
      this.str += str;
    }
  };


  /**
   * Returns the accumulated string
   */
  this.toString = function() {
    return this.str;
  };

  this.addObject(params);

}

/**
 * @class
 * Page Info Structure
 * @param str The page information string that contains the information attributes separated with semicolumns
 * Example string :
 * <CODE>"name:/products;lang:en;charset:iso-8859-1;GS:.;DS;,;requestEncoding:iso-8859-1</CODE>
 */
function PageInfo(str) {

  /** Page Name */
  this.name = null;

  /** Page Variant*/
  this.variant = null;

  /** Page Language */
  this.lang = null;

  /** Current Page Language. Significant if lang == "ml" */
  this.clang = null;

  /** Page Character Set (Encoding) */
  this.charset = null;

  /** Grouping Separator */
  this.GS = null;

  /** Decimal Separator */
  this.DS = null;

  if (str) {
    var defs = str.split(";");
    for (var i = 0; i < defs.length; i++) {
      var def = defs[i];
      var n = def.before(":");
      var v = def.after(":");
      if (n && v) {
        if (n == "name") this.name = v;
        else if (n == "variant") this.variant = v;
        else if (n == "lang") this.lang = v;
        else if (n == "clang") this.clang = v;
        else if (n == "charset") this.charset = v;
        else if (n == "GS") this.GS = v;
        else if (n == "DS") this.DS = v;
      }
    }
  }

  if (!this.lang) this.lang = "en";
  if (!this.clang) this.clang = this.lang == "ml" ? "en" : this.lang;
  if (!this.charset) this.charset = "iso-8859-1";
  if (!this.GS) this.GS = ".";
  if (!this.DS) this.DS = ",";

}

/**
 * @class
 * Represents a MoreMotion Object. This object can be instantiated by giving an DOM Element node to the
 * constructor. The DOM node may have "mo:name" and "mo:props" attributes.
 * @param node the DOM Element node that is wrapped by this object
 */
function MoreMotionObject(node) {

  /** The DOM Element node wrapped by this object */
  this.node = node;

  /** The name of this object. It is extracted from "mo:name" attribute of the DOM Element node. */
  this.name = OMgr.getAttribute(node,"mo:name") || OMgr.getAttribute(node,"name");

  /** The type of this object
   * @type String
   */
  this.type = null;

  this.isMoreMotionObject = true;

  /**
   * The properties object. It is extracted from "mo:props" attribute of the DOM element node.
   */
  this.props = OMgr.getProps(node);

  if (!this.props) this.props = new Object();

  /*
  this.getType = function() {
    return (this.constructor + "").before("(").after("function").trim();
  };
  */

  /**
   * Saves the properties in the props object to the "mo:props" attribute of the DOM element node
   */
  this.saveProps = function() {

    if (this.node) {
      var pa = new PropertyAccumulator();
      for (var p in this.props) {
        var pv = this.props[p];
        if (pv) {
          pa.add(p,this.props[p]);
        }
      }
      this.node.setAttribute("mo:props",pa.toString());
    }
  };

  /**
   * Returns the value of the given property as a boolean value.
   * @param propName The name of the property in the props object.
   * @return <code>true</code> if given property exists in props object and its value is 'true',
   * <code>false</code> otherwise
   * @type boolean
   */
  this.boolProp = function(propName) {
    var prop = eval("this.props." + propName);
    if (prop != null) {
      if (prop.toBool) return prop.toBool();
      return prop == true;
    }
    return false;
  };

  /**
   * Returns the value of the given property as a number value.
   * @param propName The name of the property in the props object.
   * @param def The default value to use if the given prop is missing in the props object.
   * @return The property value or the def value as a number
   * @type number
   */
  this.numberProp = function(propName, def) {
    var prop = eval("this.props." + propName);
    if (prop == null) prop = def;
    return prop * 1;
  };

  /**
   * Returns the value of the given property as a char (1-byte string)
   * @param propName The name of the property in the props object.
   * @param def The default value to use if the given prop is missing in the props object.
   * @return The property value or the def value as 1-byte string;
   * @type String
   */
  this.charProp = function(propName, def) {
    var prop = eval("this.props." + propName);
    if (prop == null) prop = def || "?";
    return prop.charAt(0);
  };
 
  /** @ignore */
  this.dynProp = function(propName, def) {
    var prop = eval("this.props." + propName);
    if (prop == null) prop = def;
    if (prop && prop.startsWith("*")) {
      try {
        eval("prop = " + prop.substring(1));
      } catch(e) {}
    }
    return prop;
  };
 
 
  /** @private */
  this.setModified = function() {};

  /** @private */
  this._iValue = null;

  /**
   * Returns the initial value of this object
   */
  this.getIValue = function() {
    if (this._iValue == null) {
      this._iValue = OMgr.getAttribute(this.node,"mo:iValue");
      if (this._iValue == null) this._iValue = "";
    }
    return this._iValue;
 };

  /**
   * Sets the initial value of this object
   */
  this.setIValue = function(value) {
    this._iValue = null;
    OMgr.setAttribute(this.node,"mo:iValue",value);
  };


}


/**
 * @class
 * Manager Class that provides methods to navigate through the DOM structure and manage {@link MoreMotionObject} objects.
 * A static object of this class is available with name <CODE>OMgr</CODE>.
 * <PRE>var OMgr = new MoreMotionObjectManager();</PRE>
 */

 function MoreMotionObjectManager() {

  /** @private */
  this.idCounter = 0;

  /**
   * Returns a unique id
   * @param prefix A prefix for the id. If not spefivied "x" is assumed.
   */
  this.getUniqueId = function(prefix) {
    if (prefix == null) prefix = "x";
    return prefix + (new Date()).getTime() + "" + ++this.idCounter;
  };

  /**
   * Returns the value of the given attribute safely. Provided to solve the unpredictable errors of IE.
   * @param node The node to extract the attribute value from
   * @param attrName The name of the attribute
   * @return The value of the attribute
   * @type String
   */
  this.getAttribute = function(node,attrName) {
    try {
      var attr = node.getAttribute(attrName);
    } catch (e) {}
    return attr;
  };

  this.setAttribute = function(node, attrName, attrValue) {
    try {
      node.setAttribute(attrName, attrValue);
    } catch (e) {}
  };


  /**
   * Returns the first child node with the given node name.
   * @param parentNode The HTML node whose childs will be searched
   * @param nodeName The name of the node to find.
   * @param elmName The element name. It is optional.
   * @param elmSuffix Optional parameter. When specified elmName parameter is considered to be the
   * element prefix and a child node element is searched whose name mathed with these prefix and suffix.search is made with these tThe element name. It is optional.
   * @type HTMLElement
   */
  this.getChildNode = function(parentNode, nodeName, elmName, elmSuffix) {

    nodeName = nodeName.toUpperCase();

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        if (!nodeName || node.nodeName == nodeName) {
          if (!elmName) return node;
          if (!elmSuffix) {
            if (node.name == elmName) return node;  
          } else {
            if (node.name.startsWith(elmName) && node.name.endsWith(elmSuffix)) return node;  
          }
        }
        if (node.hasChildNodes) {
          var n = this.getChildNode(node, nodeName, elmName, elmSuffix)
          if (n) return n;
        }
      }
    }
    return null;
  };

  /**
   * Returns the first parent node with the given node name.
   * @param node The HTML node whose parent will be searched
   * @param nodeName The name of the node to find.
   * @type HTMLElement
   */
  this.getParentNode = function(node, nodeName) {

    nodeName = nodeName.toUpperCase();
    node = node.parentNode;
    while (node) {
      if (node.nodeName == nodeName) {
        return node;
      }
      node = node.parentNode;
    }
    return null;
  };

  this.findChildObject = function(parentNode, nodeName, attrName, attrValue, depth) {
    var node = this.findChildNode(parentNode, nodeName, attrName, attrValue, depth);
    if (node) {
      return this.getObject(node);
    }
    return null;
  }

  /**
   * Searches the specified HTML Node under the given node.
   * @param parentNode The HTML node whose childs will be searched
   * @param nodeName The name of the node to focus the search. If specified the search will be perfomed only
   * on the nodes that has this name.
   * @param attrName The name of the attribute to find a matching value
   * @param attrValue The value to match in the attribute value. If this parameter is omitted or is null then
   * the analyzed node is selected in case of it has the attribute with the given name regardless of its value.
   * @param depth The depth of the search. If omitted 999 is assumed.
   * @type HTMLElement
   */
  this.findChildNode = function(parentNode, nodeName, attrName, attrValue, depth) {

    var prm = new Object();

    if (nodeName && nodeName.startsWith("type:")) {
      prm.motype = nodeName.after(":");
      if (prm.motype.startsWith(":")) {
        prm.stopOnSame = true;
        prm.motype = prm.motype.substring(1);
      }
      nodeName = null;
    } else {
      if (nodeName) nodeName = nodeName.toUpperCase();
    }
    prm.nodeName = nodeName;

    prm.attrName = attrName;

    if (attrValue.endsWith("*")) {
      prm.generic = true;
      prm.attrValue = attrValue.before("*");
    } else {
      prm.attrValue = attrValue;
    }

    if (depth == null) depth = 999;

    return this._findChildNode(parentNode, prm, depth);

  };


  /** @private */
  this._findChildNode = function(parentNode, prm, depth) {

    var attr;
    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];

      if (node.nodeType == 1) {

        var typeMatch = false;
        var doit = false;
        if (prm.nodeName) doit = node.nodeName == prm.nodeName;
        else if (prm.motype) {
          attr = this.getAttribute(node,"mo:type");
          if (attr && (attr == prm.motype)) typeMatch = true;
          doit = typeMatch;
        } else doit = true;

        if (doit) {
          attr = this.getAttribute(node,prm.attrName);
          if (attr) {
            if ( (prm.generic && attr.startsWith(prm.attrValue)) || attr == prm.attrValue) {
              return node;
            }
          }
        }
        if (typeMatch == false && node.hasChildNodes && depth > 0) {
          var result = this._findChildNode(node, prm, depth - 1);
          if (result) {
            return result;
          }
        }

      }
    }

    return null;
  };


  this.findParentObject = function(parentNode,attrName,attrValue) {
    var node = this.findParentNode(parentNode,attrName,attrValue);
    if (node) {
      return this.getObject(node);
    }
    return null;
  }

  /**
   * Returns the first parent node that has the specified attribute and attribute value.
   * @param node The HTML node whose parent will be searched
   * @param attrName The name of the attribute to find a matching value
   * @param attrValue The value to match in the attribute value. If this parameter is omitted or is null then
   * the analyzed node is selected in case of it has the attribute with the given name regardless of its value.
   * @type HTMLElement
   */
  this.findParentNode = function(node,attrName,attrValue) {
    node = node.parentNode;
    while (node) {
      var attr = this.getAttribute(node,attrName);
      if (attr && (!attrValue || attr.startsWith(attrValue))) {
        return node;
      }
      node = node.parentNode;
    }
    return null;
  };

  /** @private */
  this.collectInputElements = function(elems, collector) {
    for (var i = 0; i < elems.length; i++) {
      collector[collector.length] = elems[i];
    }
  };

  /**
   * Returns an array of all form elements ("input", "select" and "editarea") that exits under the
   * given node.
   * @type HTMLElement[]
   * @return An array of DOM elements
   */
  this.getChildInputElements = function(node) {
    var elms = new Array();
    this.collectInputElements(node.getElementsByTagName("input"),elms);
    this.collectInputElements(node.getElementsByTagName("select"),elms);
    this.collectInputElements(node.getElementsByTagName("textarea"),elms);
    return elms;
  };

  /**
   * Searches the child nodes of the given node and returns an array of MoreMotion Objects
   * that have the specified attribute and value
   * @param parentNode The HTML node whose childs will be searched
   * @param attrName The name of the attribute to find a matching value
   * @param attrValue The value to match in the attribute value.
   * @type MoreMotionObject[]
   * @return An array of MoreMotion Objects
   */
  this.getObjects = function(parentNode, attrName, attrValue) {
    var objects = new Array();
    this._collectObjects(objects, parentNode, attrName, attrValue);
    return objects;
  };

  /** @private */
  this._collectObjects = function(objects, parentNode, attrName, attrValue) {

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        var attr = OMgr.getAttribute(node,attrName);
        if (attr && (!attrValue || attr == attrValue)) {
          objects[objects.length] = OMgr.getObject(node);
        }
        else if (node.hasChildNodes) {
          this._collectObjects(objects, node, attrName, attrValue);
        }
      }
    }

  };

  /**
   * Creates a props object from the value of the "mo:props" attribute of the given node and returns it.
   * @param node The DOM element node
   * @type props
   */
  this.getProps = function(node) {

    var attr = this.getAttribute(node,"mo:props");
    if (attr) {
     try {
       if (attr.startsWith("*")) {
         eval("var props = " + attr.substring(1));
       } else {
         eval("var props = {" + attr + "}" );
       }
     } catch (e) {
       if (!OMgr._alerted) {
         alert('invalid mo:props definition. mo:props="' + attr + '"\n\n' + e.message);
         OMgr._alerted = true;
       }
       return null;
     }
      return props;
    }
    return null;

  };

  /**
   * Creates the {@link MoreMotionObject} defined in the "mo:type" attribute of the given DOM element node and
   * returns it.
   * @param node The DOM element node that contains "mo:type" attribute.
   */
  this.getObject = function(node) {
    if (!node) return null;
    var type = this.getAttribute(node,"mo:type");
    if (type) {
      try {
        eval("var obj = new " + type + "(node)");
        obj.type = type;
        return obj;
      } catch (e) {}
    }
    return null;
  };

  /**
   * Creates the {@link MoreMotionObject} defined in the "mo:type" attribute of the DOM element whose
   * id is given and returns it.
   * @param id The id of the element node that contains "mo:type" attribute.
   */
  this.getObjectById = function(id) {
    var node = document.getElementById(id);
    if (node) {
      return this.getObject(node);
    }
    return null;
  };

  /** @private */
  this.initObjects = function(parentNode) {

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
       var attr = this.getAttribute(node,"mo:needsInit");
        if (attr != null) {
          var o = this.getObject(node);
          if (o && o.init) {
            o.init();
          }
        }
        if (node.hasChildNodes) {
          this.initObjects(node);
        }
      }
    }

  };

  /** @private */
  this.setClass = function(node, classPrefix, classSuffix) {

    var buf = "";
    var classNames = node.className.split(' ');
    for (var i = 0; i < classNames.length; i++) {
      var cls = classNames[i];

      if (!cls.startsWith(classPrefix + "_") ) {
        if (buf == "") buf = cls; else buf += " " + cls;
      }
    }

    node.className = buf + " " + classPrefix + "_" + classSuffix;
  };


  /**
   * Returns the page info by extracting from the document.body node.
   * @type PageInfo
   */
  this.getThisPageInfo = function() {
    return new PageInfo( this.getAttribute(document.body,"mo:pageInfo") );
  };

  /**
   * Searches the parents of the given node to find a node that has
   * "mo:pageInfo" attribute to extract extracts and return the page info from there.
   * @param node The DOM element node to start the searching off. If it is null then page info is
   * taken from document.body node.
   * @type PageInfo
   */
  this.getPageInfo = function(node) {
    if (node && node.after) node = document.getElementById(node);
    if (node == null) return this.getThisPageInfo();
    var pinode = this.findParentNode(node,"mo:pageInfo");
    if (pinode) return new PageInfo( this.getAttribute(pinode,"mo:pageInfo") );
    return this.getThisPageInfo();
  };


  /**
   * Submits the specified form with the given action.
   * @param nodeOrFormName Either a DOM node existing in the form the name of the form to submit
   * @param action The action to set to the from object before submit
   */
  this.submitForm = function(nodeOrFormName, action) {
    var frm = null;
   if (nodeOrFormName.after) frm = document.forms[nodeOrFormName];
    else frm = this.getParentNode(nodeOrFormName,"form");
    if (frm) {
      if (action) frm.action = action;
      if (frm._rand) frm._rand.value = (new Date()).getTime();
      frm.submit();
    }
  };

  /**
   * Resets the specified form
   * @param nodeOrFormName Either a DOM node existing in the form the name of the form to submit
   */
  this.resetForm = function(nodeOrFormName) {
    var frm = null;
   if (nodeOrFormName.after) frm = document.forms[nodeOrFormName];
    else frm = this.getParentNode(nodeOrFormName,"form");
    if (frm) frm.reset();
  };

  this.refreshHref = function(node) {
    if (node.href != "") {
      var ri = node.href.indexOf("_rand=");
      if (ri > -1) {
        var bi = node.href.indexOf("#");
        var bm = bi == -1 ? "" : node.href.substring(bi);
        node.href = node.href.substring(0,ri+6) + (new Date()).getTime() + bm;
      }
    }
  };

  this.refreshLink = function(href) {
    if (href == "") return "#";
    var ri = href.indexOf("_rand=");
    if (ri > -1) {
      var bi = href.indexOf("#");
      if (bi == -1) bm = ""; else bm = href.substring(bi);
      return href.substring(0,ri+6) + (new Date()).getTime() + bm;
    } else return href;
  };


  this.refreshAction = function(frm) {
    if (frm._rand) frm._rand.value = (new Date()).getTime();
  };

  /**
   * Sets the initial value of the given <CODE>SELECT</CODE> element
   * @param elm The handle or the id of a HTML <CODE>SELECT</CODE> element
   * @param value The initial value
   */
  this.setInitialOption = function(elm, value) {
   if (elm.after) elm = document.getElementById(elm);
    if (elm && value) {
      for (var i = 0; i < elm.options.length; i++) {
        if (value == elm.options[i].value) {
          elm.options[i].selected = true;
          elm.options[i].setAttribute("selected","true");
          return;
        }
      }
    }
  };

  /**
   * Initializes all the {@link MoreMotionObject} objects existing in the given DOM element node.
   * @param node The DOM element node to start with initializing. If it is omitted document.body is used.
   */
  this.initialize = function(node) {
    if (!node) {
      node = document.body;
    }
    this.initObjects(node);
  };

  /**
   * Searches the parent nodes of the given DOM element node to find a
   * MoreMotionObject node. If it finds then it creates an instance of it and returns it.
   * @param node The DOM element node to start searching the parent nodes.
   * @type MoreMotionObject
   */
  this.getEnclosingObject = function(node) {
    node = this.findParentNode(node,"mo:name");
    if (node) {
      return this.getObject(node);
    }
    return null;
  };


/** @private */
var RegExp_parm1 = new RegExp( "%0", "g" );
/** @private */
var RegExp_parm2 = new RegExp( "%1", "g" );
/** @private */
var RegExp_parm3 = new RegExp( "%2", "g" );

  /**
   * Returns the resource text associated with the specified resId. The %0, %1, %2 symbols
   * in the resource text are replaces with the values of parm1, parm2 and parm3 parameters.
   * @param resId the Resource id
   * @param parm1 Resource parameter 1. (Optional)
   * @param parm2 Resource parameter 2. (Optional)
   * @param parm3 Resource parameter 3. (Optional)
   */
  this.resource = function(resId, parm1, parm2, parm3) {

    var result = CommonResources[resId];

    if (!result) return resId;

    try {
      if (parm1 != null) result = result.replace( RegExp_parm1, parm1);
    } catch (e) {}
    try {
      if (parm2 != null) result = result.replace(RegExp_parm2, parm2);
    } catch (e) {}
    try {
      if (parm3 != null) result = result.replace(RegExp_parm3, parm3);
    } catch (e) {}

    return result;

  };


  this.createElement = function(tagName, elmName) {
    if (WinMgr.isIE) {
      var s = "<" + tagName + " name='" + elmName + "' />";
      return document.createElement(s);
    } else {
      var elm = document.createElement(tagName);
      elm.name = elmName;
      return elm;
    }
  };


}

/** Global Singleton MoreMotionObjectManager object
 * @type MoreMotionObjectManager
 */
 var OMgr = new MoreMotionObjectManager();

/**
 * @class
 * This class provides utility methods related to the current browser window. A static object of this class
 * is avaialable with name <CODE>WinMgr</CODE>.
 * <PRE>var WinMgr = new WindowManager();</PRE>
 *
 *
 */
function WindowManager() {

  var ua = navigator.userAgent.toLowerCase();

  /** <CODE>true</CODE> means the current browser is Gecko
   * @type boolean
   **/
  this.isGecko = ua.indexOf("gecko") > -1;

  /** <CODE>true</CODE> means the current browser is Internet Explorer
   * @type boolean
   **/
  this.isIE = ua.indexOf("msie") > -1;

  /** <CODE>true</CODE> means the current browser is Opera
   * @type boolean
   **/
  this.isOpera = ua.indexOf("opera") > -1;

  /**
   * Returns the value of the specified request parameter.
   * @param parmName The name of the request parameter
   * @param defValue The value to return in case of the specified parameter is missing in the request
   * @type String
   *
   */
  this.getRequestParam = function(parmName, defValue) {
     var parms = unescape(self.location.search).substring(1).split("&");
     for (var i = 0; i < parms.length; i++) {
       var oneparm = parms[i].split("=");
       if (oneparm[0] == parmName)
         return oneparm[1];
     }
     return defValue;
  };

  /** @private */
  this._boolParam = function(name, value) {
    return name + "=" + (value ? "1" : "0");
  };

  /** @private */
  this._param = function(name, value) {
    return name + "=" + value;
  };

  /**
   * Saves the windows position and dimensions to the cookie and closes the current browser window.
   * @param pageName The name of the current page. This parameter will be used as the cookie key to save the
   * current top, left, width, height values of the active window. If omitted then no saving is done.
   *
   */
  this.close = function(pageName) {
    if (pageName) {
      try {
        var winId = this.normalizeWinId(pageName);
        window.opener.WinMgr.saveDims(window, winId);
      } catch (e) {}
    }
    window.close();
    window.opener.focus();
  };

  /** @private */
  this.saveDims = function(w, winId) {
    /* alert(winId + " " + (w.screenTop || w.screenY)); */
    if (!winId) return;
    this.setCookie(winId + "_left",w.screenLeft || w.screenX);
    this.setCookie(winId + "_top",w.screenTop || w.screenY);
  };

  this.currentWinId = null;

  /** @private */
  this.normalizeWinId = function(winId) {
    var s = "";
    for (var i = 0; i < winId.length; i++) {
      var c = winId.charAt(i);
      if ("/.\\".indexOf(c) > -1) {
        if (s.length > 0) s += "_";
      } else s += c;
    }
    return s;
  };

  /**
   * Opens a new Popup window with the specified parameters.
   * @param props The property array that contains the function parameters in
   * <CODE>{prop1:"Value1", prop2:"Value2"}</CODE> format.
   * <ul>
   * <li><b>prop.url</b> - Either a {@link UrlAddress} object or a url string.
   * </li><li><b>prop.winId</b> - An identity for the popup window. For the calls that have the same winId,
   * the same popup window is used. If a window is already open it is replaced with new content.
   * </li><li><b>prop.left</b> - The distance on the left of the window. If not defined the window
   * is aligned vertically on the center.
   * </li><li><b>prop.top</b> - The distance on the top of the window. If not defined the window
   * is aligned horizontally on the middle.
   * </li><li><b>props.toolbar</b> - Toolbar is visible
   * </li><li><b>props.scrollbars</b> - Scroolbars are visible
   * </li><li><b>props.location</b> - URL location is visible
   * </li><li><b>props.status</b> - Status bar is visible
   * </li><li><b>props.menubar</b> - Menubar is visible
   * </li><li><b>props.resizable</b> - Window is resizable
   * </li><li><b>props.status</b> - Status bar is visible
   * </li></ul>
   * <p>
   * <p><u>Example:</u></p>
   * <PRE><CODE>
   *      WinMgr.openWindow({ <br/>
   *        url:"/NewWindow/PopUp", height:400, width:700, <br/>
   *        scrollbars:true, toolbar:true, menubar:true, resizable:true, <br/>
   *        status:true, location:true, resizeable:true, <br/>
   *        top:50, left:50 }); <br/>
   * </CODE></PRE>
   * </p>
   * @type window Object
   * @return The handle of the new popup window
   */
  this.openWindow = function( props ) {

    var url = props.url;

    if (!url.addParam) url = new UrlAddress(url);

    var winId = props.winId;
    if (winId == null) winId = url.getParam("pg");
    if (winId == null) winId = url.base;

    winId = this.normalizeWinId(winId);

    var x = props.left;
    if (!x) x = this.getCookie(winId + "_left");
    if (!x || x == null) x = (screen.width - props.width) / 2;

    var y = props.top;
    if (!y) y = this.getCookie(winId + "_top");
    if (!y || y == null) y = (screen.height - props.height) / 2;

    var a = new Accumulator(",");

    a.add(this._boolParam("toolbar",props.toolbar));
    a.add(this._boolParam("scrollbars",props.scrollbars));
    a.add(this._boolParam("location",props.location));
    a.add(this._boolParam("status",props.status));
    a.add(this._boolParam("menubar",props.menubar));
    a.add(this._boolParam("resizable",props.resizable));
    a.add(this._boolParam("status",props.status));
    a.add(this._param("width",props.width));
    a.add(this._param("height",props.height));
    a.add(this._param("left",x));
    a.add(this._param("top",y));

    var w = window.open(url.toString(), winId, a.toString());
    w.focus();
    return w;

  };

  /** @private */
  this.getExpires = function(m) {
    if (m == null) return "";
    if (!m) m = 365 * 24 * 60;
    var d = new Date();
    var ed = d.getTime() + (m * 60 * 1000);
    d.setTime(ed);
    return ";expires=" + d.toGMTString();
  };

  /**
   * Sets a cookie.
   * @param name Cookie Name
   * @param value Cookie Value
   * @param minutes The life of the cookie in minutes
   * @param domain The domain string
   * @param path The path string
   * @param secure
   */
  this.setCookie = function(name, value, minutes, domain, path, secure) {

    var cookie = name + '=' + escape(value) +
      (path   ? (";path="   + path)   : "") +
      (domain ? (";domain=" + domain) : "") +
      this.getExpires(minutes) +
      (secure ? (";secure")           : "");

    try {
      document.cookie = cookie;
    } catch (e) {return false;}

    return true;
  };

  /**
   * Returns the value of the specified cookie
   * @param name The cookie name
   * @type String
   */
  this.getCookie = function(name) {

   var cookie = document.cookie;
   var start = cookie.indexOf(name);
   if(start == -1) return null;
   start += name.length + 1;
   var end = cookie.indexOf(";", start);
   if(end == -1) end = cookie.length;
   return unescape(cookie.substring(start, end));
  };

  /**
   * Expires the specified cookie
   * @param name The cookie name
   * @param domain The domain name
   * @param path The path string
   */
  this.expireCookie = function(name, domain, path) {
    var value = this.get(name);
    if (value == null) return false;
    var cookie = name + '=' + value + '; expires=Fri, 13-Apr-1970 00:00:00 GMT' +
      (path   ? (";path="   + path)   : "") +
      (domain ? (";domain=" + domain) : "");
    try {
     document.cookie = cookie;
    } catch (e) {return false;}

    return true;
  };

}
/** Static {@link WindowManager} Object */
var WinMgr = new WindowManager();


/**
 * @class
 * Represents a MoreMotion Face Exception
 * @param code Exception Code
 * @param arg1 Exception Argument 1
 * @param arg2 Exception Argument 2
 * @param arg3 Exception Argument 3
 */
 function Exception(code, arg1, arg2, arg3) {

  /** The exception code */
  this.code = code;
  /** Exception Argument 1 */
  this.arg1 = arg1;
  /** Exception Argument 1 */
  this.arg2 = arg2;
  /** Exception Argument 1 */
  this.arg3 = arg3;

  /** The object causing or related to this exception */
  this.object = null;

  /** The DOM Element to focus when this exception is alerted */
  this.elementToFocus = null;

  /**
   * Returns the localized message text.
   * @type String
   */
  this.getMessageText = function() {
    return OMgr.resource(this.code , this.arg1, this.arg2, this.arg3);
  };

  /**
   * Alerts the localized text to the user. If elementToFocus is not null then the
   * focus is set to it before alerting.
   */
  this.alert = function() {

    if (this.elementToFocus && this.object) {
      try {
        TabMgr.focusEnclosingTab(this.object.node);
      } catch(e) {}
      this.elementToFocus.focus();
    }

    alert( this.getMessageText());
  };

}


/**
 * @private
 * @class
 * Class to call Javascript handler functions
 * @param name The name of the Javascript handler function without any paranthesis and/or parameters
 * @param caption The caption of the handler function
 */
function Handler(name, caption) {

  /**
   * Executes the handler function
   * @param prm1 The first objects to be passed to the handler funtion
   * @param prm2 The second objects to be passed to the handler funtion
   * @param prm3 The thisd objects to be passed to the handler funtion
   * @return The result of the executed handler
   * @type boolean
   */
  this.run = function(prm1, prm2, prm3) {
    return this.handler(prm1, prm2, prm3);
  };

  /** @private */
  this.setHandler = function(name, caption) {

    if (!name.substring) {
      this.handler = name;
      this.handlerName = ("" + name).before("{").after("function").trim();
      this.handlerCaption = caption;
      return;
    }

    this.handlerName = name.before("(",name);
    if (caption) this.handlerCaption = caption;

    try {
      this.handler = eval(this.handlerName);
    } catch (e) {
      var parm1 = caption || this.handlerName;
      var parm2 = e.message;
      alert( OMgr.resource("INVALID_HANDLER", parm1, parm2) );
      return;
    }
    return;

  };

  this.setHandler(name, caption);

}

/**
 * @private
 * @class
 * Is used to call Event Handlers
 * @extends Handler
 * @param name The name of the Javascript event handler function without any paranthesis and/or parameters
 * @param caption The caption of the handler function
 *
 */
function EventHandler(name, caption) {
  /** @private */
  this.base = Handler;
  this.base(name, caption);
  /**
   * Executes the event handler. The error handler should return <CODE>true</CODE> if the caller should
   * continue normally and <CODE>false</CODE> to cancel the action.
   * @param obj The object related to or cause of the event.
   * @type boolean
   */
  this.run = function(obj) {
    return this.handler(obj);
  };
}

/**
 * @private
 * @class
 * Is used to call Error Handlers
 * @extends Handler
 * @param name The name of the Javascript error handler function without any paranthesis and/or parameters
 * @param caption The caption of the handler function
 *
 */
function ErrorHandler(name, caption) {
  /** @private */
  this.base = Handler;
  this.base(name, caption);
  /**
   * Executes this error handler. The error handler should return normally if the error is handled.
   * If the error cannot be handled it should throw the Exception passed to it as the parameter.
   * @param ex The {@link Exception} object holding information about the error occured
   * @param obj The data object related to the action in error.
   */
  this.run = function(ex, obj) {
    this.handler(ex, obj);
  };
}


/**
 * @class
 * Wrapper class of a DOM Node that contains text data.
 * @extends MoreMotionObject
 */
function TextNode(node) {

  /** @private */
  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isTextNode = true;

  /** @private */
  this.aNode = null;

  /**
   * The HTML <code>TD</code> Element node wrapped by this field object.
   */
  this.textNode = null;

  /** @private */
  this.getChildNode = function(parentNode) {
    var nodes = parentNode.childNodes;
    for (var i = 0; i < nodes.length; i++) {
     var n = nodes[i];
     if (n.nodeType == 3 || (n.nodeType == 1 && n.tagName == "A") ) {
       return n;
     }
    }
    return null;
  }

  /** @private */
  this.wrapElements = function() {

    var n = this.getChildNode(this.node);
    if (n) {
      if (n.nodeType == 3) {
        this.textNode = n;
      } else {
        if (n.tagName && n.tagName == "A") {
          this.aNode = n;
          n = this.getChildNode(n);
          if (n) {
            this.textNode = n;
          }
        }
      }
    }
  };

  this.wrapElements();

  /**
   * Returns the HTML <code>A</code> Element (Anchor) node wrapped by this field object.
   */
  this.getANode = function() {
    if (!this.aNode) {
      var s = this.textNode ? this.textNode.nodeValue : "";
      var aNode = document.createElement("A");
      this.aNode = this.node.appendChild(aNode);
      if (this.textNode) this.node.removeChild(this.textNode);
      this.textNode = this.aNode.appendChild(document.createTextNode(s));
    }
    return this.aNode;
  };

  /**
   * Removes the HTML <code>A</code> Element (Anchor) node wrapped by this field object.
   */
  this.removeANode = function() {
    if (this.aNode) {
      var s = this.textNode.nodeValue;
      this.node.removeChild(this.aNode);
      this.aNode = null;
      this.textNode = this.node.appendChild(document.createTextNode(s));
    }
  };

  /**
   * Sets a text value this field object
   */
  this.setValue = function(value) {
    this._setValue(value);
  };

  /** @ignore */
  this._setValue = function(value) {

    if (this.textNode) {
      this.textNode.nodeValue = value;
    } else {
      if (this.aNode) {
        this.textNode = this.aNode.appendChild(document.createTextNode(value));
      } else {
        this.textNode = this.node.appendChild(document.createTextNode(value));
      }
    }
  };

  /** Returns the text value of this field object*/
  this.getValue = function() {
    return this.textNode ? this.textNode.nodeValue : "";
  };

  /** Returns the Hyperlink value of this field object*/
  this.getHref = function() {
    return this.aNode ? this.aNode.href : "";
  };


  /** Sets a hyperlink value to this field object.
   * @param value The hyperlink value to set
   */
  this.setHref = function(value) {

    if (value == "") {
      if (this.aNode) this.removeANode();
    } else {
      this.getANode().href = value;
    }
  };

  /**
   * Clears text and hyperlink values of this field object.
   */
  this.clear = function() {
    this.setHref("");
    this.setValue("");
  };


}


/** @ignore */
/* For Javascript Docs */
function _ignore() {
  new TextNode();
  new Exception();
  new ErrorHandler();
  new EventHandler();
}
