<?xml version="1.0" encoding="UTF-8"?>
<!-- Version $Id: MMTemplates.xsl 7 2008-03-05 16:18:02Z erkan $ -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <!-- ============ -->
  <!-- BUILD BREAKS -->
  <!-- ============ -->
  <xsl:template name="mm-build-breaks">
    <xsl:param name="text"/>
    <xsl:choose>
      <xsl:when test="contains($text, '&#xa;')">
        <xsl:call-template name="mm-build-spaces">
          <xsl:with-param name="text" select="substring-before($text, '&#xa;')"/>
        </xsl:call-template>
        <br/>
        <xsl:call-template name="mm-build-breaks">
          <xsl:with-param name="text" select="substring-after($text,'&#xa;')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="mm-build-spaces">
          <xsl:with-param name="text" select="$text"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ============ -->
  <!-- BUILD SPACES -->
  <!-- ============ -->
  <xsl:template name="mm-build-spaces">
    <xsl:param name="text"/>
    <xsl:choose>
      <xsl:when test="not(boolean($text)) or $text = ''">
        <xsl:value-of select="'&#xa0;'" disable-output-escaping="yes"/>
      </xsl:when>
      <xsl:when test="substring($text,1,1) = ' '">
        <xsl:value-of select="'&#xa0;'" disable-output-escaping="yes"/>
        <xsl:if test="string-length($text) > 1">
          <xsl:call-template name="mm-build-spaces-recurse">
            <xsl:with-param name="text" select="substring($text,2)"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="mm-build-spaces-recurse">
          <xsl:with-param name="text" select="$text"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="mm-build-spaces-recurse">
    <xsl:param name="text"/>
    <xsl:choose>
      <xsl:when test="$text = ' '">
        <xsl:value-of select="'&#xa0;'" disable-output-escaping="yes"/>
      </xsl:when>
      <xsl:when test="contains($text, '  ')">
        <xsl:value-of select="substring-before($text, '  ')"/>
        <xsl:value-of select="'&#xa0; '" disable-output-escaping="yes"/>
        <xsl:if test="string-length(substring-after($text,'  ')) > 0">
          <xsl:call-template name="mm-build-spaces">
            <xsl:with-param name="text" select="substring-after($text,'  ')"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="substring($text,string-length($text),1) = ' '">
            <xsl:value-of select="substring($text,1,string-length($text) - 1)"/>
            <xsl:value-of select="'&#xa0;'" disable-output-escaping="yes"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$text"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- =========== -->
  <!-- REPLACE ALL -->
  <!-- =========== -->
  <xsl:template name="mm-replace-all">
    <xsl:param name="text"/>
    <xsl:param name="from"/>
    <xsl:param name="to"/>
    <xsl:choose>
      <xsl:when test="contains($text, $from)">
        <xsl:value-of select="substring-before($text, $from)"/>
        <xsl:value-of select="$to"/>
        <xsl:call-template name="mm-replace-all">
          <xsl:with-param name="text" select="substring-after($text,$from)"/>
          <xsl:with-param name="from" select="$from"/>
          <xsl:with-param name="to" select="$to"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- =============== -->
  <!-- FORMAT CURRENCY -->
  <!-- =============== -->
  <xsl:template name="mm-format-currency">
    <xsl:param name="value"/>
    <xsl:param name="format"/>
    <xsl:choose>
      <xsl:when test="contains($value,' ') and contains($format, '?')">
        <xsl:variable name="sign" select="substring-after($value,' ')"/>
        <xsl:variable name="pure-price" select="substring-before($value, ' ')"/>
        <xsl:variable name="formatted-pure-price" select="format-number($pure-price, $format, 'std')"/>
        <xsl:variable name="formatted-price" select="concat(substring-before($formatted-pure-price, '?'), $sign, substring-after($formatted-pure-price, '?'))"/>
        <xsl:value-of select="$formatted-price"/>
      </xsl:when>
      <xsl:when test="number($value)">
        <xsl:value-of select="format-number($value, $format, 'std')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="not(boolean($value)) or $value = ''">
            <xsl:value-of select="'&#xa0;'" disable-output-escaping="yes"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$value"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ============= -->
  <!-- FORMAT NUMBER -->
  <!-- ============= -->
  <xsl:template name="mm-format-number">
    <xsl:param name="value"/>
    <xsl:param name="format"/>
    <xsl:variable name="df">
      <xsl:choose>
        <xsl:when test="boolean(substring-after($format,'//'))"><xsl:value-of select="substring-after($format,'//')"/></xsl:when>
        <xsl:otherwise>std</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="fmt">
      <xsl:choose>
        <xsl:when test="boolean(substring-before($format,'//'))"><xsl:value-of select="substring-before($format,'//')"/></xsl:when>
        <xsl:otherwise><xsl:value-of select="$format"/></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="number($value)">
        <xsl:value-of select="format-number($value, $fmt, $df)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="not(boolean($value)) or $value = ''">
            <xsl:value-of select="'&#xa0;'" disable-output-escaping="yes"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$value"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ============= -->
  <!-- RIGHT TRIM    -->
  <!-- ============= -->
  <xsl:template name="mm-trim-right">
    <xsl:param name="text" select="''"/>
    <xsl:choose>
      <xsl:when test="substring($text,string-length($text),1) = ' '">
        <xsl:call-template name="mm-trim-right">
          <xsl:with-param name="text" select="substring($text,1,string-length($text) - 1)"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ============= -->
  <!-- LEFT TRIM     -->
  <!-- ============= -->
  <xsl:template name="mm-trim-left">
    <xsl:param name="text" select="''"/>
    <xsl:choose>
      <xsl:when test="substring($text,1,1) = ' '">
        <xsl:call-template name="mm-trim-left">
          <xsl:with-param name="text" select="substring($text,2)"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ============= -->
  <!-- TRIM          -->
  <!-- ============= -->
  <xsl:template name="mm-trim">
    <xsl:param name="text" select="''"/>
    <xsl:call-template name="mm-trim-left">
      <xsl:with-param name="text">
        <xsl:call-template name="mm-trim-right">
          <xsl:with-param name="text" select="$text"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

</xsl:stylesheet>