/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * AjaxProcess.java
 *
 * Created on 05 September 2007 Wednesday, 23:10
 *
 */

package org.moremotion.mmcomp.ajax;

import org.moremotion.adom.ADOM;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessException;
import org.moremotion.process.ProcessRecord;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: AjaxProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
public class AjaxProcess extends Process {

  private ConfigParameter dataSource;
  private ConfigParameter nodesToEncrypt;
  private ConfigParameter responseHandler;
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      
      dataSource = processConfig.getParameter("dataSource");
      nodesToEncrypt = processConfig.getParameter("nodesToEncrypt");
      responseHandler = processConfig.getParameter("responseHandler");
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("AJAX_PROCESS_PARAMETERS", 
            new String[] {dataSource+"", nodesToEncrypt+"", responseHandler+""});
      }
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
  }

  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
      }
      
      AjaxResponse response = new AjaxResponse();
      response.setName(XString.afterLast(callDef.getTargetName(),'.') + "." + prec.getIndex());
      
      String DataSource = dataSource.resolve(request).stringValue();
      String NodesToEncrypt = nodesToEncrypt.resolve(request).stringValue(null);
      String ResponseHandler = responseHandler.resolve(request).stringValue();
      
      if (logger.LEVEL2) {
        
        if (dataSource.hasMScript() || nodesToEncrypt.hasMScript() || responseHandler.hasMScript() ) {
          logger.printres("*PROCESS_PARAMETERS_LINE_RESOLVED");
          logger.printres("REFRESH_FIELDS_PROCESS_PARAMETERS", 
              new String[] {DataSource, NodesToEncrypt, ResponseHandler});
        }
      }
      
      
      response.addReponseProperty("responseHandler",ResponseHandler);
      
      ADOM adom = Utils.generateADOM(DataSource,request,NodesToEncrypt);
      response.setADOM(adom);
      response.setResponseHandler(ResponseHandler);
      
      AjaxProcessManager.addResponse(request,response);
      if (logger.LEVEL2) logger.printres("RESPONSE_CREATED");
      
    } catch (ConfigParameterException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    } catch (AjaxException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
    
  }

  public void finalizeProcess() throws ProcessException {
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    }
  }
  
 
}
