/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * AjaxResponse.java
 *
 * Created on 05 September 2007 Wednesday, 21:44
 *
 */

package org.moremotion.mmcomp.ajax;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: AjaxResponse.java 217 2008-09-14 14:57:27Z erkan $
 */
public class AjaxResponse {
  
  private String name = "";
  private String type = "";
  private String handler = "";
  private Properties responseProps = null;
  private ADOM adom = null;
  private ArrayList html = null;
  private ArrayList extjs = null;
  private ArrayList extcss = null;
  private HashMap embjs = null;
  private HashMap styles = null;
  private ArrayList areas = null;
  
  public void setADOM(ADOM adom) {
    this.adom = adom;
  }
  
  public void setName(String name) {
    this.name = name;
  }
  
  public void addExternalScript(String src) {
    if (extjs == null) {
      extjs = new ArrayList();
    }
    extjs.add(src);
  }
  
  public void addExternalCss(String src) {
    if (extcss == null) {
      extcss = new ArrayList();
    }
    extcss.add(src);
  }
  
  public void addEmbededScript(String text) {
    if (embjs == null) {
      embjs = new HashMap();
    }
    embjs.put(text,"");
  }
  
  public void addStyle(String name, String value) {
    if (this.styles == null) {
      this.styles = new HashMap();
    }
    this.styles.put(name, value);
  }
  
  public HashMap getStyles() {
    return this.styles;
  }
  
  public void setStyles(HashMap styles) {
    this.styles = styles;
  }
  
  public void addHtml(String html) {
    if (this.html == null) {
      this.html = new ArrayList();
    }
    this.html.add(html);
  }
  
  
  public void setAreas(ArrayList areas) {
    this.areas = areas;
  }
  
  public ArrayList getAreas() {
    return this.areas;
  }
  
  public void setResponseHandler(String handler) {
    this.handler = handler;
  }
  
  public void setType(String type) {
    this.type = type;
  }
  
  public void addReponseProperty(String name, String value) {
    if (responseProps == null) responseProps = new Properties();
    responseProps.setProperty(name, value);
  }
  
  // "{ 'props':{'blockName':!'Block1','formName':'Form1'},'meta':['ID','NAME'],'item':[['12','Computer'],['33','Printer']] }";
  
  public String getResponseString() {
    StringBuffer buf = new StringBuffer(1024);
    buf.append("{");
    buf.append("'name':'" + this.name + "',");
    buf.append("'type':'" + this.type + "',");
    buf.append("'handler':'" + this.handler + "',");
    buf.append(getPropsJson());
    // if (adom != null) 
    {
      buf.append("," + getMetaJson() + "," + getItemJson());
    }
    
    if (styles != null) {
      buf.append(getNewSourceAreaNamesJson());
      buf.append(getStylesJson());
    }
    
    if (html != null) {
      buf.append("," + getHtmlJson());
    }
    buf.append("}");
    
    return buf.toString();
  }
  
  
  // "'props':{'blockName':'Block1','formName':'Form1'}";
  private String getPropsJson() {
    StringBuffer buf = new StringBuffer();
    if (responseProps != null) {
      Enumeration keys = responseProps.keys();
      while (keys.hasMoreElements()) {
        String key = (String)keys.nextElement();
        String value = responseProps.getProperty(key);
        if (buf.length() > 0) buf.append(',');
        buf.append("'" + key + "':'" + escape(value) + "'");
      }
    }
    
    return " 'props':{ " + buf.toString() + "}";
    
  };
  
  
  // "'meta':['ID','NAME']";
  private String getMetaJson() {
    
    int cnt = 0;
    StringBuffer buf = new StringBuffer();
    
    if (adom != null) {
      
      try {
        Iterator itr = adom.getNodes();
        ADOMNode item = (ADOMNode)itr.next();
        itr = item.getNodes();
        while (itr.hasNext()) {
          ADOMNode n = (ADOMNode)itr.next();
          String nn = n.getName();
          if (buf.length() > 0) buf.append(',');
          buf.append("'" + nn + "'");
        }
      } catch (Exception e) {}

    }
    
    return "'meta':[" + buf.toString() + "]";
    
  }
  
  
  
  private String getItemJson() {
    
    boolean firstItem = true;
    StringBuffer buf = new StringBuffer();
    
    // "item':[['12','Computer'],['33','Printer']]"
    
    if (adom != null) {
      
      Iterator itr = adom.getNodes();
      while (itr.hasNext()) {
        
        ADOMNode item = (ADOMNode)itr.next();
        if (item.getName().equals("item")) {
          
          if (!firstItem) buf.append(",");
          buf.append("[");
          firstItem = false;
          
          
          boolean firstNode = true;
          
          Iterator items = item.getNodes();
          while (items.hasNext()) {
            
            ADOMNode node = (ADOMNode)items.next();
            
            String nv = node.getValue();
            if (nv == null || nv.length() == 0) {
              nv = "";
            } else {
              nv = escape(nv);
            }
            
            if (!firstNode) buf.append(',');
            buf.append("'" + nv + "'" );
            firstNode = false;
            
          }
          buf.append("]");
        }
      }
      
    }
    
    return "'item':[" + buf.toString() + "]";
    
  }
  
  private String getHtmlJson() {
    
    StringBuffer buf = new StringBuffer(1024);
    
    /*
     * External Java Scripts
     */
    if (extjs != null) {
      if (buf.length() > 0) buf.append(',');
      buf.append("'extjs':[");
      for (int i = 0; i < extjs.size(); i++) {  
        if (i > 0) buf.append(',');
        buf.append('\'');
        buf.append(escape((String) extjs.get(i)));
        buf.append('\'');
      }
      buf.append(']');
    }
    
    /*
     * External Css
     */
    if (extcss != null) {
      if (buf.length() > 0) buf.append(',');
      buf.append("'extcss':[");
      for (int i = 0; i < extcss.size(); i++) {  
        if (i > 0) buf.append(',');
        buf.append('\'');
        buf.append(escape((String) extcss.get(i)));
        buf.append('\'');
      }
      buf.append(']');
    }
    
    /*
     * Embeded Java Scripts
     */
    if (embjs != null) {
      if (buf.length() > 0) buf.append(',');
      buf.append("'embjs':[");
      Iterator itr = embjs.entrySet().iterator();
      boolean first = true;
      while (itr.hasNext()) {
        Map.Entry e = (Map.Entry)itr.next();
        if (!first) buf.append(',');
        first = false;
        buf.append('\'');
        buf.append(escape((String) e.getKey()));
        buf.append('\'');
      }
      buf.append(']');
    }
    
    /*
     * HTML
     */
    if (buf.length() > 0) buf.append(',');
    buf.append("'html':[");
    
    for (int i = 0; i < this.html.size(); i++) {
      if (i > 0) buf.append(',');
      buf.append('\'');
      buf.append(escape((String)this.html.get(i)));
      buf.append('\'');
    }
    
    buf.append(']');
    
    
    return buf.toString();
  }
  
  private String getNewSourceAreaNamesJson() {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < this.areas.size(); i++) {
      SourceArea area = (SourceArea)this.areas.get(i);
      if  (area.firstTimeForStyles()) {
        if (buf.length() > 0) buf.append(',');
        buf.append('\'');
        buf.append(area.getPageName());
        buf.append('.');
        buf.append(area.getAreaName());
        buf.append(':');
        buf.append(area.getStyleSequence());
        buf.append('\'');
      }
    }      
    
    if (buf.length() == 0) return "";
    return (", 'newSourceAreas':[" + buf.toString() + "]");
  }
  
  private String getStylesJson() {
    
    /*
     * Styles
     */
    StringBuffer buf = new StringBuffer(1024);
    if (styles != null) {
      Iterator itr = this.styles.entrySet().iterator();
      while (itr.hasNext()) {
        Map.Entry e = (Map.Entry)itr.next();
        buf.append("\\n");
        buf.append("    .");
        buf.append(fixedLenStr((String)e.getKey()));
        buf.append('{');
        buf.append(escape((String)e.getValue()));
        buf.append('}');
      }
    }
    
    if (buf.length() == 0) return "";
    return ",'styles':'" + buf.toString() + '\'';
  }
  
  private static String fixedLenStr(String s) {
    return s + "        ".substring(s.length());
  }
  
  private static boolean has(ArrayList al, String str) {
    for (int i = 0; i < al.size(); i++) {
      String s = (String)al.get(i);
      if (s.equals(str)) {
        return true;
      }
    }
    return false;
  }
  
  public static String escape(String s) {
    
    StringBuffer b = null;
    
    int ln = s.length();
    
    for (int i = 0; i < ln; i++) {
      
      char c = s.charAt(i);
      char nc = i+1 < ln ? s.charAt(i+1) : ' ';
      
      switch (c) {
        
        case '\\':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          b.append("\\\\");
          break;
          
        case '\'':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          b.append("\\'");
          break;
          
        case '\r':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          if (nc == '\n') break;
          b.append("\\n");
          break;
          
        case '\n':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          b.append("\\n");
          break;
          
        default:
          if (b != null) {
            b.append(c);
          }
          
      }
    }
    
    if (b == null) return s;
    return b.toString();
    
  }
  
}
