/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * RefreshAreas.java
 *
 * Created on 06 December 2004 Monday, 17:50
 */

package org.moremotion.mmcomp.ajax;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.moremotion.config.ConfigurationException;
import org.moremotion.datasrc.Paging;
import org.moremotion.page.PageGenerationException;
import org.moremotion.page.PageGenerator;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RefreshAreas.java 51 2008-03-10 11:46:54Z erkan $
 */
public class RefreshAreas {
  
  
  private static void putPage(ArrayList pages, String pageName) {
    for (int i = 0; i < pages.size(); i++) {
      if (pageName.equals(pages.get(i))) return;
    }
    pages.add(pageName);
  }
  
  public static void extractSourceAreas(String areaDefsStr, MoreMotionRequest request, AjaxResponse response) throws AjaxException {
    
    try {
      
      ArrayList areas = new ArrayList();
      ArrayList pages = new ArrayList();
      
      String[] areaDefs = XString.split(areaDefsStr,',');
      
      for (int i = 0; i < areaDefs.length; i++ ) {
        String areaDef = areaDefs[i];
        String pageName = XString.before(areaDef,'.',null);
        String areaName = XString.after(areaDef,'.',null);
        if (pageName == null || areaName == null) {
          throw new AjaxException("INVALID_SOURCE_DEFINITION",new String[]{areaDef},null);
        }
        SourceArea area = new SourceArea();
        area.setPageName(pageName);
        area.setAreaName(areaName);
        areas.add(area);
        putPage(pages,pageName);
      }
      
      assignSourceAreaSequences(request,areas);
      String targetPage = request.getParameter("_targetpage","");
      
      for (int itr = 0; itr < pages.size(); itr++) {
        String pageName = (String) pages.get(itr);
        
        String areaNames = null;
        for (int i = 0; i < areas.size(); i++) {
          SourceArea area = (SourceArea) areas.get(i);
          if (area.getPageName().equals(pageName)) {
            if (areaNames == null) areaNames = area.getAreaName();
            else areaNames = "," + area.getAreaName();
          }
        }
        
        request.setSourceAreaNames(areaNames);
        
        PageGenerator pgen = new PageGenerator();
        Node node = pgen.generateDom(request,pageName);
        
        if (node == null) {
          throw new AjaxException("SOURCE_PAGE_NOT_FOUND",new String[]{pageName},null);
        }
        
        collectSourceAreaNodes(node.getFirstChild(),areas,pageName);
        // if (!allRefreshableAreas(areas,pageName)) {
        if (!pageName.equals(targetPage)) {
          extractJavaScripts(node.getFirstChild(),response);
          extractExternalCss(node.getFirstChild(),response);
          extractStyles(node.getFirstChild(),areas,response,pageName);
        }
        
        for (int i = 0; i < areas.size(); i++) {
          SourceArea area = (SourceArea) areas.get(i);
          if (area.getPageName().equals(pageName)) {
            if (area.getNode() != null) {
              response.addHtml( getXml(area.getNode()) );
            } else {
              response.addHtml("");
            }
          }
        }
        
        request.setCurrentPage(null);
        
      }
      
      response.setAreas(areas);
      
    } catch (PageGenerationException ex) {
      throw new AjaxException(ex);
    } catch (TransformerConfigurationException ex) {
      throw new AjaxException(ex);
    } catch (TransformerException ex) {
      throw new AjaxException(ex);
    }
  }
  
//  private static boolean allRefreshableAreas(ArrayList areas, String pageName) {
//    for (int i = 0; i < areas.size(); i++) {
//      SourceArea area = (SourceArea)areas.get(i);
//      if (area.getNode() != null) {
//        Element elm = (Element)area.getNode();
//        String type = elm.getAttribute("mo:type");
//        if (!type.equals("RefreshableArea")) return false;
//      }
//    }
//    return true;
//  }
  
  private static void assignSourceAreaSequences(MoreMotionRequest request, ArrayList areas) {
    int maxSequence = 0;
    String usedSourceAreas = request.getParameter("_usedsourceareas");
    if (usedSourceAreas != null) {
      StringParser p = new StringParser(usedSourceAreas,',');
      while (p.hasMore()) {
        String def = p.nextItem();
        String sadef = XString.before(def,':');
        String saseq = XString.after(def,':');
        
        String pageName = XString.before(sadef,'.');
        String areaName = XString.after(sadef,'.');
        
        int seq = -1;
        try {
          seq = Integer.parseInt(saseq);
        } catch(Exception ex) {};
        if (seq > maxSequence) maxSequence = seq;
        
        for (int i = 0; i < areas.size(); i++) {
          SourceArea area = (SourceArea)areas.get(i);
          if (area.equals(pageName,areaName)) {
            area.setStyleSequence(seq);
          }
        }
      }
    }
    
    for (int i = 0; i < areas.size(); i++) {
      SourceArea area = (SourceArea)areas.get(i);
      if (area.getClassPrefix() == null) {
        area.setStyleSequence(++maxSequence, true);
      }
    }
    
  }
  
//  private static String extractSourceArea(String areaName, Node rootNode) throws TransformerConfigurationException, TransformerException {
//    
//    Node node = findSourceAreaNode(rootNode.getFirstChild(),areaName);
//    if (node != null) {
//      return getXml(node);
//    }
//    return "";
//    
//  }
//  
//  private static Node findSourceAreaNode(Node rootNode, String areaName) {
//    NodeList nlist = rootNode.getChildNodes();
//    for (int i = 0; i < nlist.getLength(); i++) {
//      Node node = nlist.item(i);
//      if (node.getNodeType() == Node.ELEMENT_NODE ) {
//        Element e = (Element)node;
//        if (e.getAttribute("mo:type").equals("SourceArea") && e.getAttribute("mo:name").equals(areaName)) {
//          return node;
//        }
//        Node result = findSourceAreaNode(node, areaName);
//        if (result != null) {
//          return result;
//        }
//      }
//    }
//    return null;
//  }
  
  private static void collectSourceAreaNodes(Node rootNode, ArrayList areas, String pageName) {
    _collectSourceAreaNodes(rootNode, areas, pageName);
  }
  
  private static void _collectSourceAreaNodes(Node rootNode, ArrayList areas, String pageName) {
    NodeList nlist = rootNode.getChildNodes();
    for (int i = 0; i < nlist.getLength(); i++) {
      Node node = nlist.item(i);
      if (node.getNodeType() == Node.ELEMENT_NODE ) {
        Element e = (Element)node;
        String type = e.getAttribute("mo:type");
        if (type.equals("SourceArea") || type.equals("RefreshableArea") ) {
          String name = e.getAttribute("mo:name");
          for (int j = 0; j < areas.size(); j++) {
            SourceArea area = (SourceArea)areas.get(j);
            if (area.equals(pageName,name)) {
              area.setNode(node);
              if (!hasUnassignedAreaOfPage(areas, pageName)) {
                return;
              }
            }
          }
        }
      }
      _collectSourceAreaNodes(node, areas, pageName);
    }
  }
  
  private static boolean hasUnassignedAreaOfPage(ArrayList areas, String pageName) {
    for (int i = 0; i < areas.size(); i ++) {
      SourceArea area = (SourceArea)areas.get(i);
      if (area.getNode() != null) {
        if (area.getPageName().equals(pageName) && area.getNode() == null) {
          return true;
        }
      }
    }
    return false;
  }
  
  private static NodeList extractJavaScripts(Node node, AjaxResponse response) {
    
    Node headNode = findNode(node,"head");
    
    if (headNode != null) {
      NodeList scripts = ((Element)headNode).getElementsByTagName("script");
      
      if (scripts != null) {
        for (int i = 0; i < scripts.getLength(); i++) {
          Element script = (Element)scripts.item(i);
          String src = script.getAttribute("src");
          if (src.length() > 0) {
            response.addExternalScript(src);
          } else {
            String s = script.getFirstChild().getNodeValue();
            response.addEmbededScript(s);
          }
        }
      }
      
    }
    return null;
    
  }
  
  private static NodeList extractExternalCss(Node node, AjaxResponse response) {
    
    Node headNode = findNode(node,"head");
    
    if (headNode != null) {
      NodeList links = ((Element)headNode).getElementsByTagName("link");
      
      if (links != null) {
        for (int i = 0; i < links.getLength(); i++) {
          Element link = (Element)links.item(i);
          if (link.getAttribute("type").equalsIgnoreCase("text/css")) {
            String href = link.getAttribute("href");
            if (href.length() > 0) {
              response.addExternalCss(href);
            } 
          }
        }
      }
      
    }
    return null;
    
  }
  

  private static void extractStyles(Node node, ArrayList areas, AjaxResponse response, String pageName) {
    HashMap styles = collectStyles(node);
    modifyClasses(styles,areas,response,pageName);
  }
  
  private static void modifyClasses(HashMap styles, ArrayList areas, AjaxResponse response, String pageName) {
    
    HashMap usedStyles = response.getStyles();
    if (usedStyles == null) {
      usedStyles = new HashMap();
      response.setStyles(usedStyles);
    }
    
    for (int i = 0; i < areas.size(); i++) {
      SourceArea area = (SourceArea)areas.get(i);
      if (area.getNode() != null) {
        if (area.getPageName().equals(pageName)) {
          _modifyClasses(area.getNode(),styles,usedStyles,area);
        }
      }
    }
    
    mergeClasses(styles,usedStyles,areas,pageName);
  }
  
  private static void mergeClasses(HashMap styles, HashMap usedStyles, ArrayList areas, String pageName) {
    
    for (int i = 0; i < areas.size(); i++) {
      SourceArea area = (SourceArea)areas.get(i);
      if (area.getPageName().equals(pageName)) {
        if (area.firstTimeForStyles()) {
          String pfx = area.getClassPrefix();
          Iterator si = styles.entrySet().iterator();
          while (si.hasNext()) {
            Map.Entry e = (Map.Entry)si.next();
            String s = pfx + (String) e.getKey();
            if (usedStyles.get(s) == null) {
              usedStyles.put(s,e.getValue());
            }
          }
        }
      }
    }
  }
  
  private static void _modifyClasses(Node node, HashMap styles, HashMap usedStyles, SourceArea area ) {
    
    Element e = (Element)node;
    
    Attr attr = e.getAttributeNode("class");
    if (attr != null) {
      StringParser classNames = new StringParser(attr.getValue(),' ');
      StringBuffer buf = new StringBuffer();
      while (classNames.hasMore()) {
        String className = classNames.nextItem();
        if (usedStyles.get(area.getClassPrefix()+className) == null) {
          String style = (String)styles.get(className);
          if (style != null) {
            if (area.firstTimeForStyles()) {
              usedStyles.put(area.getClassPrefix()+className,style);
            }
            className = area.getClassPrefix() + className;
          }
        } else {
          className = area.getClassPrefix() + className;
        }
        if (buf.length() > 0) buf.append(' ');
        buf.append(className);
        
      }
      attr.setValue(buf.toString());
    }
    
    if (node.hasChildNodes()) {
      NodeList nlist = node.getChildNodes();
      for (int i = 0; i < nlist.getLength(); i++) {
        Node n = nlist.item(i);
        if (n.getNodeType() == Node.ELEMENT_NODE) {
          _modifyClasses(n,styles,usedStyles,area);
        }
      }
    }
    
    if (e.getNodeName().equalsIgnoreCase("textarea")) {
      if (!e.hasChildNodes()) {
        e.appendChild( e.getOwnerDocument().createTextNode("$remove-this-from-textarea$"));
      }
    }
    
  }
  
  
  private static HashMap collectStyles(Node node) {
    
    HashMap styles = new HashMap();
    
    Node headNode = findNode(node,"head");
    
    if (headNode != null) {
      NodeList styleNodes = ((Element)headNode).getElementsByTagName("style");
      
      if (styleNodes != null) {
        for (int i = 0; i < styleNodes.getLength(); i++) {
          Element styleNode = (Element)styleNodes.item(i);
          parseStyles(styles, styleNode);
        }
      }
      
    }
    
    return styles;
  }
  
  
  private static void parseStyles(HashMap styles, Node node) {
    
    String stylesText = node.getFirstChild().getNodeValue();
    
    StringBuffer buf = new StringBuffer(stylesText);
    
    for (int i = 0; i < buf.length(); i++) {
      char c = buf.charAt(i);
      if (c == '\n') buf.setCharAt(i,' ');
      else if (c == '{') buf.setCharAt(i,':');
      else if (c == '}') buf.setCharAt(i,'^');
    }
    
    StringParser p = new StringParser(buf.toString(),'^');
    
    while (p.hasMore()) {
      String def = p.nextItem();
      int x = def.indexOf(':');
      if (x > -1) {
        String name = def.substring(0,x).trim();
        String style = def.substring(x+1).trim();
        if (name.length() > 0) {
          if (name.charAt(0) == '.') {
            name = name.substring(1);
            styles.put(name,style);
          }
        }
      }
    }
    
  }
  
  
  private static Node findNode(Node rootNode, String nodeName) {
    NodeList nlist = rootNode.getChildNodes();
    for (int i = 0; i < nlist.getLength(); i++) {
      Node node = nlist.item(i);
      if (node.getNodeName().equals(nodeName)) {
        return node;
      }
      Node result = findNode(node, nodeName);
      if (result != null) {
        return result;
      }
    }
    return null;
  }
  
  
  private static String getXml(Node node) throws TransformerConfigurationException, TransformerException {
    
    Transformer trans = TransformerFactory.newInstance().newTransformer();
    trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
    
    StringWriter sink = new StringWriter();
    trans.transform( new DOMSource(node),  new StreamResult(sink));
    
    return sink.toString();
  }
  
//  private static String getPagingInfo(MoreMotionRequest request, String dsname) throws ConfigurationException {
//    Paging paging = new Paging(request,dsname,null);
//    if (paging != null) {
//      return paging.getPagingInfo();
//    }
//    return null;
//  }
  
}