/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.mailer;

/*
 * TextMailProcess.java
 *
 * Created on February 10, 2003, 1:04 PM
 */

import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigurationException;
import org.moremotion.process.ProcessException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessRecord;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: TextMailProcess.java 89 2008-03-21 20:22:00Z erkan $
 */
public class TextMailProcess extends Process {
  
  private String smtpHost = null;
  private String smtpUser = null;
  private String smtpFrom = null;
  private String smtpPassword = null;
  private int smtpPort = 25;
  private boolean smtpAuth = false;
  private boolean smtpSecure = false;
  
  MailTask mtask = null;
  
  private ConfigParameter to;
  private ConfigParameter cc;
  private ConfigParameter bcc;
  private ConfigParameter subject;
  private ConfigParameter message;
  private ConfigParameter attachs;
  
  private int tQ = 0;
  private int seqnr = 0;
  
  private boolean hasVars;
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      
      ConfigParameter smtpConfigName = processConfig.getParameter("smtpAccount"); 
      String SmtpConfigName = smtpConfigName.resolve(request).stringValue();

      Config config = ConfigManager.getConfig("smtpAccount", SmtpConfigName);

      if (config == null) {
        throw new MailerException("SMTP_CONFIG_MISSING",new String[]{SmtpConfigName},null);
      }

      smtpHost = config.getParameter("host").resolve(request).stringValue();
      smtpPort = config.getParameter("port").resolve(request).intValue();
      smtpUser = config.getParameter("user").resolve(request).stringValue();
      smtpPassword = config.getParameter("password").resolve(request).passwordValue();
      smtpFrom = config.getParameter("from").resolve(request).stringValue(smtpUser);
      smtpAuth = config.getParameter("auth").resolve(request).booleanValue(false);
      smtpSecure = config.getParameter("secure").resolve(request).booleanValue(false);
        
      if (logger.LEVEL2) { 
        logger.printres("SMTP_PARAMETERS", 
            new String[] {smtpHost+"", smtpPort+"", smtpUser+"", smtpFrom+"", smtpAuth+"", smtpSecure+""} );
      }
      
      to = processConfig.getParameter("to");
      cc = processConfig.getParameter("cc");
      bcc = processConfig.getParameter("bcc");
      subject = processConfig.getParameter("subject");
      message = processConfig.getParameter("message");
      attachs = processConfig.getParameter("attachs");

      mtask = new MailTask(smtpHost,smtpPort,smtpAuth,smtpSecure,smtpUser,smtpPassword,smtpFrom);

      mtask.setLogger(logger);

      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("TEXT_MAIL_PROCESS_PARAMETERS",
            new String[] {to+"", cc+"", bcc+"", subject+"", attachs+""});
      }
      
      
      hasVars = to.hasMScript() || cc.hasMScript() ||
          bcc.hasMScript() || subject.hasMScript() || message.hasMScript() ||
          attachs.hasMScript();
      
      String charset = request.getParameter("_enc","iso-8859-1");
      mtask.setCharset(charset);
      
      
    } catch (ConfigurationException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    } catch (MailerException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
    
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
     
    if (logger.LEVEL2) logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
    
    try {
      
      String To   = to.resolve(request).stringValue("");
      String Cc   = cc.resolve(request).stringValue("");
      String Bcc  = bcc.resolve(request).stringValue("");
      String Subject = subject.resolve(request).stringValue("");
      String Message = message.resolve(request).stringValue("");
      String Attachs = attachs.resolve(request).stringValue("");
      
      if (hasVars) {
        
        if (logger.LEVEL3) {
          logger.printres("*PROCESS_PARAMETERS_LINE_RESOLVED");
          logger.printres("TEXT_MAIL_PROCESS_PARAMETERS",
              new String[] {To, Cc, Bcc, Subject, Attachs});
        } 
        
      }
      
      Attachment[] attachments = Attachment.getAttachments(request, prec, Attachs);
      mtask.addEntry(++seqnr, smtpFrom, To, Cc, Bcc, Subject, Message, attachments, null);
      
    } catch (Exception ex) {
      if (mtask != null) mtask.setEntriesComplete();
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
  }
  
  public void finalizeProcess() throws ProcessException {
    
    mtask.setEntriesComplete();
    
    if (getCompletionCode() > 8) {
      mtask.clear();
      return;
    }
    
    if (logger.LEVEL2) logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    
    try {
      if (logger.LEVEL2) logger.printres("BEGINING_TO_SEND");
      ForegroundMailer.send(mtask,logger);
      
    } catch (Exception ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    }
    
  }
  
}