/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * BlobExporter.java
 *
 * Created on 08 October 2005 Saturday, 13:38  
 */

package org.moremotion.mmcomp.reldb;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Blob;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Hashtable;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: BlobExporter.java 215 2008-09-13 10:27:24Z erkan $
 */
public class BlobExporter {
  
  protected static Hashtable blobpool = null;
  
  static {
    blobpool = new Hashtable();
  }
  
  private static void createFile(File file, Blob blob) throws SQLException, FileNotFoundException, IOException {
    file.deleteOnExit();
    file.getParentFile().mkdirs();
    InputStream is = blob.getBinaryStream();
    FileOutputStream fos = new FileOutputStream(file);
    
    int len = 0;
    byte[] buff = new byte[1024 * 1000];
    while ((len = is.read( buff )) != -1 ) {
//      System.out.println("File : " + file.getAbsolutePath() + ", len : " + len);
      fos.write(buff,0,len);
    }
    fos.flush();
    fos.close();
    
  }
  
  public static String exportBlob(MoreMotionRequest request, ResultSet rs, int ci, BlobExportDefinition bed) 
  throws BlobExportException {
    
    try {
      
      ResultSetMetaData rsmd = rs.getMetaData();
      BlobPoolEntry bpe = new BlobPoolEntry(rsmd, rs, ci, bed);
      
      BlobPoolEntry e = (BlobPoolEntry)blobpool.get(bpe.key);
      if (e == null || !e.lastModified.equals(bpe.lastModified)) {
        
        synchronized (blobpool) {
          Blob blob = rs.getBlob(ci);
          if (blob == null || blob.length() == 0) {
            File f = request.getRealFile(bpe.exportName);
            if (f.exists()) {
              f.delete();
            }
            return "";
          } else {
            createFile( request.getRealFile(bpe.exportName),  blob );
            blobpool.put(bpe.key,bpe);
            return bpe.exportName;
          }
        }
      } else {
        long diff = System.currentTimeMillis() - e.lastChecked;
        if (diff > (1000L * 10)) {
          synchronized (blobpool) {
            e.lastChecked = System.currentTimeMillis();
            File f = request.getRealFile(e.exportName);
            boolean create = e.lastModified.length() == 0; 
            if (!create) {
              create = !f.exists();
            }
            if (create) {
              Blob blob = rs.getBlob(ci);
              if (blob.length() > 0) {
                createFile( f,  blob );
              }
            }
          }
        }
        return e.exportName;
      }
      
    } catch (Exception ex) {
      throw new BlobExportException(ex);
    }
    
  }
  
}


class BlobPoolEntry {
  protected String key;
  protected String lastModified;
  protected String exportName;
  protected long lastChecked;
  
  public BlobPoolEntry(ResultSetMetaData rsmd, ResultSet rs, int ci, BlobExportDefinition bed) throws SQLException {
    lastChecked = System.currentTimeMillis();
    String blobColName = rsmd.getColumnName(ci);
    key = new String(blobColName);
    for (int i = 0; i < bed.keyCols.length; i++) {
      String colName = bed.keyCols[i];
      String colValue = rs.getString(colName);
      key = key + colValue;
    }
    key = key + rsmd.getTableName(ci);
    key = key + bed.dburl + bed.exportDir;
    
    lastModified = "";
    for (int i = 0; i < bed.lmCols.length; i++) {
      String colName = bed.lmCols[i];
      String colValue = rs.getString(colName);
      lastModified = lastModified + colValue;
    }
    
    String ext = "jpg";
    for (int i = 0; i < bed.blobCols.length; i++) {
      String s = bed.blobCols[i];
      if (blobColName.equals(bed.blobCols[i])) {
        ext = bed.exts[i];
        break;
      }
    }
    
    exportName = bed.exportDir + "/" + key.hashCode() + "." + ext;
  }
  
}

