/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ReldbQueryDataService.java
 *
 * Created on January 25, 2001, 3:59 PM
 */

package org.moremotion.mmcomp.reldb;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMException;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataServiceContext;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.Paging;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.ServiceException;


/**
 * Queries a Relational Database and generates an {@link org.moremotion.adom.ADOM ADOM}
 * object using relational database query definition.
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ReldbQueryDataService.java 215 2008-09-13 10:27:24Z erkan $
 */
public class ReldbQueryDataService implements DataService {
  
  /**
   * Creates a new ReldbQueryDataService.
   * @param datasource Relational database query definition object.
   * @param request MoreMotionRequest Object
   */
  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    adom.clear();
    
    MoreMotionRequest request = dsc.getRequest();
    DataSourceConfig dsconfig = dsc.getDataSourceConfig();
    Paging paging = dsc.getPaging();
    
    ReldbQuery reldbQuery = new ReldbQuery(request, dsconfig, paging, dsc.logger);
    
    try {
      reldbQuery.execute(adom);
    } catch (ADOMException e) {
      throw new ServiceException(e);
    }

  }
  
  
}


