/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ResourceFromReldbDataService.java
 *
 * Created on 18 July 2006 Tuesday, 12:45
 */

package org.moremotion.mmcomp.reldb;

import java.sql.Clob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.servlet.ServiceException;
import org.moremotion.datasrc.DataServiceContext;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.JdbcConnection;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ResourceFromReldbDataService.java 215 2008-09-13 10:27:24Z erkan $
 */
public class ResourceFromReldbDataService implements DataService {
  
  
  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    try {
      
      adom.clear();
      
      MoreMotionRequest request = dsc.getRequest();
      DataSourceConfig dsconfig = dsc.getDataSourceConfig();
      
      String conName = dsconfig.getParameter("conn").resolve(request).stringValue();
      String sql = dsconfig.getParameter("SQL").resolve(request).stringValue();
      
      boolean mscript = dsconfig.getParameter("MScript").resolve(request).booleanValue(false);
      boolean isClob = dsconfig.getParameter("Clob").resolve(request).booleanValue(false);
      
      ADOMNode rootNode = adom;
    
      Connection con;
      try {
        con = JdbcConnection.getConnection(request, conName);
      } catch (Exception e) {
        throw new ReldbException("UNABLE_TO_CONNECT_WITH_CON",new String[]{conName},e);
      }
      
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(sql);
      
      while (rs.next()) {
        String rcode = rs.getString(1);
        String rtext;
        
        if (isClob) {
          Clob clob = rs.getClob(2);
          if (clob == null) rtext = "";
          else {
            long pos = 1;
            Long len = new Long(clob.length());
            rtext = clob.getSubString(pos,len.intValue());
          }
        } else {
          rtext = rs.getString(2);
        }
        
        if (mscript) {
          try {
            rtext = request.resolve(rtext);
          } catch (Exception e) {
            throw new ReldbException("RESOURCE_TEXT_MSCRIPT_ERROR",new String[]{rtext},e);
          }
        }
        
        if (rcode != null && rcode.length() > 0) {
          ADOMNode node = new ADOMNode(rcode,safe(rtext));
          rootNode.addNode(node);
        }
      }
      
      rs.close();
      stmt.close();
      
    } catch (SQLException e) {
      throw new ReldbException("RESOURCE_FROM_RELDB_SQL_ERROR",null,e);
      
    } catch (ConfigParameterException e) {
      throw new ReldbException("RESOURCE_FROM_RELDB_CONFIGURATION_ERROR",null,e);
    } 
    
    
  }
  
  private String safe(String s) {
    return s == null ? "" : s;
  }
  
}
