/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;

import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ForgotPassword.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ForgotPassword {

  protected static void sendMessage(MoreMotionRequest request, Config sdconfig, String email, String url) throws SecurityException, ConfigurationException {
    
    try {

      Session session = null;

      String smtpHost = sdconfig.getParameter("fpmail.smtp.host").resolve(request).stringValue();
      int smtpPort = sdconfig.getParameter("fpmail.smtp.port").resolve(request).intValue();
      String smtpUser = sdconfig.getParameter("fpmail.smtp.user").resolve(request).stringValue();
      String smtpPassword = sdconfig.getParameter("fpmail.smtp.password").resolve(request).passwordValue(null);
      String smtpFrom = sdconfig.getParameter("fpmail.smtp.from").resolve(request).stringValue(smtpUser);
      boolean smtpAuth = sdconfig.getParameter("fpmail.smtp.auth").resolve(request).booleanValue(false);

      Properties props = new Properties();
      props.put("mail.smtp.host", smtpHost);
      props.put("mail.from", smtpFrom);
      props.put("mail.smtp.auth", String.valueOf(smtpAuth));

      if (smtpAuth) {
        session = Session.getDefaultInstance(props, new SimpleAuthenticator(smtpUser, smtpPassword));
      } else {
        session = Session.getDefaultInstance(props);
      }

      String subject = sdconfig.getParameter("fpmail.subject").resolve(request).stringValue();
      String body = sdconfig.getParameter("fpmail.mailtext").resolve(request).stringValue();
      String keyword = "{CHANGE_PASSWORD_LINK}";
      int i = body.indexOf(keyword);
      if (i > -1) {
        String newbody = body.substring(0, i) + url + body.substring(i + keyword.length());
        body = newbody;
      }

      Transport trans = null;
      Message msg = new MimeMessage(session);
      InternetAddress[] toAddr = new InternetAddress[1];
      toAddr[0] = new InternetAddress(email);
      msg.setRecipients(javax.mail.Message.RecipientType.TO, toAddr);
      String charset = request.getParameter("_enc", "iso-8859-1");
      msg.setSubject(MimeUtility.encodeText(subject, charset, "B"));
      msg.setSentDate(new Date());
      msg.setContent(body, "text/plain;charset=\"" + charset + "\"");
      InternetAddress[] fromAddr = new InternetAddress[1];
      fromAddr[0] = new InternetAddress(smtpFrom);
      msg.setFrom(fromAddr[0]);
      msg.saveChanges();
      trans = session.getTransport(toAddr[0]);
      trans.connect(smtpHost, smtpPort, smtpUser, smtpPassword);
      trans.sendMessage(msg, toAddr);
      trans.close();
      
    } catch (UnsupportedEncodingException ex) {
      throw new SecurityException(ex);
    } catch (MessagingException ex) {
      throw new SecurityException(ex);
    } catch (ConfigParameterException ex) {
      throw new SecurityException(ex);
    }
      
  }


  static class SimpleAuthenticator extends javax.mail.Authenticator {
    String username, password;

    /** Simple authenticator for connecting SMTP server.
     * @param username User name.
     * @param password Password.
     */
    public SimpleAuthenticator(String username, String password) {
      this.username = username;
      this.password = password;
    }
    /** Returns PasswordAuthentication object to connect to SMPT server.
     * @return PasswordAuthentication object.
     */
    protected PasswordAuthentication getPasswordAuthentication() {
      return new PasswordAuthentication(username, password);
    }
  }
  
}