/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Map;
import java.util.Properties;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.DataSourceManager;
import org.moremotion.security.AuthenticationException;
import org.moremotion.security.CheckPoint;
import org.moremotion.security.DeleteSecurityEntityException;
import org.moremotion.security.MissingSecurityEntityException;
import org.moremotion.security.Role;
import org.moremotion.security.SaveSecurityEntityException;
import org.moremotion.security.User;
import org.moremotion.security.broker.BrokerCommunicationContext;
import org.moremotion.security.broker.SecurityBroker;
import org.moremotion.security.broker.SecurityBrokerException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.adom.ADOMUtils;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.JdbcConnection;
import org.moremotion.util.MutableInt;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RelDBSecurityBroker.java 152 2008-06-25 16:19:36Z erkan $
 */
public class RelDBSecurityBroker implements SecurityBroker {
  
  private String _encryptPassword(String password) {
    return SystemContext.encryptor().oneWayEncrypt(password);
  }
  
  private void _authenticateUser(User user, String password) throws AuthenticationException {
    
    if (SystemContext.encryptor().isEncrypted(password,null)) {
      throw new AuthenticationException("Invalid Password for user " + user.getName());
    }
    
    String upass = user.getPassword();
    
    password = SystemContext.encryptor().oneWayEncrypt(password);
    if (password.equals(upass)) return;
    
    upass = SystemContext.encryptor().oneWayEncrypt(upass);
    if (password.equals(upass)) return;
    
    throw new AuthenticationException("Invalid Password for user " + user.getName());
  }
  
  private static final char[] hexDigit = {
    '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
  };
  
  private static char toHex(int nibble) {
    return hexDigit[(nibble & 0xF)];
  }
  
//  private String enc2(String pass) {
//    return SystemContext.encryptor().encrypt(pass);
//  }
  
  private String safeval(String s) {
    if (s == null) return "";
    return s;
  }
  
  //
  // USER INTERFACE METHODS
  //
  
  private User openUser(BrokerCommunicationContext bcc, ResultSet rs, boolean full) throws Exception {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    String id = rs.getString("ID");
    String name = rs.getString("NAME");
    String password = rs.getString("PASSWORD");
    String fullname = rs.getString("FULLNAME");
    String email = rs.getString("EMAIL");
    String disabled = rs.getString("DISABLED");
    User user = new User(name,password,email,fullname,(disabled != null && disabled.equals("true")) );
    user.setId(id);
    user.setBroker(this);
    
    ResultSetMetaData md = rs.getMetaData();
    
    int columnCount = md.getColumnCount();
    
    for(int i = 1; i <= columnCount; i++) {
      String colName = md.getColumnName(i);
      if (!colName.equals("ID") && !colName.equals("NAME") && !colName.equals("PASSWORD")  &&
          !colName.equals("FULLNAME") && !colName.equals("EMAIL") && !colName.equals("DISABLED") ) {
        String colValue = rs.getString(i);
        user.setAttribute(colName,safeval(colValue));
      }
    }
    
    if (!full) {
      return user;
    }
    
    bcc.pool.setProperty("userid", id);
    ResultSet rrs = p.execQ(p.c.sql_GetRolesofUser);
    while (rrs.next()) {
      Role role = getRole(bcc,rrs.getString("NAME"));
      user.addRole(role);
    }
    rrs.getStatement().close();
    
    return user;
  }
  
  public User getUser(BrokerCommunicationContext bcc, String userName) throws MissingSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    try {
      // System.out.println("FIRM_ID:" + safeval(bcc.pool.getProperty("FIRM_ID")));
      
      bcc.pool.setProperty("username",userName);
      ResultSet rs = p.execQ(p.c.sql_GetUserInfoByName);
      if (rs.next()) {
        User user = openUser(bcc,rs,true);
        rs.getStatement().close();
        return user;
      } else {
        rs.getStatement().close();
        throw new MissingSecurityEntityException("User " + userName + " not found.");
      }
    } catch(Exception e) {
      throw new MissingSecurityEntityException(e.getMessage());
    }
  }
  
  public User getUserByEmail(BrokerCommunicationContext bcc, String email) throws MissingSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    try {
      bcc.pool.setProperty("email",safeval(email));
      ResultSet rs = p.execQ(p.c.sql_GetUserInfoByEmail);
      if (rs.next()) {
        User user = openUser(bcc,rs,true);
        rs.getStatement().close();
        return user;
      } else {
        rs.getStatement().close();
        throw new MissingSecurityEntityException("User with Email '" + email + "' not found.");
      }
    } catch(Exception e) {
      throw new MissingSecurityEntityException(e.getMessage());
    }
  }
  
  public User[] getUsers(BrokerCommunicationContext bcc) throws MissingSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    User[] result = null;
    try {
      LinkedList ll = new LinkedList();
      
      ResultSet rs = p.execQ(p.c.sql_GetAllUsers);
      while (rs.next()) {
        ll.addLast(openUser(bcc,rs,false));
      }
      rs.getStatement().close();
      result = new User[ll.size()];
      ll.toArray(result);
    } catch(Exception e) {
      throw new MissingSecurityEntityException(e);
    }
    
    return result;
  }
  
  public User[] getUsers(BrokerCommunicationContext bcc, int from, int amount, MutableInt noi) throws MissingSecurityEntityException {
    
    User[] result = null;
    
    try {
      
      if (from < 0) from = 0;
      int to = from + amount;
      
      PreparedContext p = (PreparedContext)bcc.preparedObject;
      
      LinkedList ll = new LinkedList();
      ResultSet rs = p.execQ(p.c.sql_GetAllUsers);
      int count = 0;
      while (rs.next()) {
        count++;
        if (count > from && count <= to) {
          User user = openUser(bcc, rs, false ) ;
          ll.addLast(user);
        }
      }
      rs.getStatement().close();
      noi.value = count;
      
      result = new User[ll.size()];
      ll.toArray(result);
      
    } catch(Exception e) {
      throw new MissingSecurityEntityException(e);
    }
    
    return result;
  }
  
  public User[] searchUsers(BrokerCommunicationContext bcc, String searchCriteria, int from, int amount, MutableInt noi) throws MissingSecurityEntityException {
    
    User[] result = null;
    
    try {
      
      if (from < 0) from = 0;
      int to = from + amount;
      
      PreparedContext p = (PreparedContext)bcc.preparedObject;
      String username = StringParser.extractParameter(searchCriteria,"username");
      String disabled = StringParser.extractParameter(searchCriteria,"disabled");
      String fullname = StringParser.extractParameter(searchCriteria,"fullname");
      String email = StringParser.extractParameter(searchCriteria,"email");
      
      //    bcc.pool.clear();
      bcc.pool.setProperty("username", username);
      bcc.pool.setProperty("fullname", safeval(fullname));
      bcc.pool.setProperty("email", safeval(email));
      bcc.pool.setProperty("disabled", safeval(disabled));
      
      StringParser sp = new StringParser(searchCriteria,';');
      while (!sp.eos()) {
        String def = sp.nextItem();
        String prmName = XString.before(def,'=',"");
        String prmValue = XString.after(def,'=',"");
        if (prmName.length() > 0 &&
            !prmName.equals("username") && !prmName.equals("disabled") &&
            !prmName.equals("fullname") && !prmName.equals("email")) {
          bcc.pool.setProperty(prmName, safeval(prmValue));
        }
      }
      
      LinkedList ll = new LinkedList();
      ResultSet rs = p.execQ(p.c.sql_SearchUsers);
      int count = 0;
      while (rs.next()) {
        count++;
        if (count > from && count <= to) {
          User user = openUser(bcc, rs, false ) ;
          ll.addLast(user);
        }
      }
      rs.getStatement().close();
      noi.value = count;
      
      result = new User[ll.size()];
      ll.toArray(result);
      
    } catch(Exception e) {
      throw new MissingSecurityEntityException(e);
    }
    
    return result;
  }
  
  public void deleteUser(BrokerCommunicationContext bcc, String userName) throws DeleteSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    try {
      bcc.pool.setProperty("username",userName);
      p.exec(p.c.sql_RemoveUser);
    } catch (Exception e) {
      throw new DeleteSecurityEntityException(e);
    }
    
  }
  
  public void saveUser(BrokerCommunicationContext bcc, User user) throws SaveSecurityEntityException {
    
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    try {
      
      bcc.pool.setProperty("username", user.getName());
      bcc.pool.setProperty("password", user.getPassword());
      bcc.pool.setProperty("fullname", safeval(user.getFullname()));
      bcc.pool.setProperty("email", safeval(user.getEmail()));
      bcc.pool.setProperty("disabled", "" + user.isAccountDisabled());
      
      Properties attrs = user.getAttributes();
      if (attrs != null) {
        Enumeration en = attrs.propertyNames();
        while (en.hasMoreElements()) {
          String attrName = (String)en.nextElement();
          String attrValue = safeval(attrs.getProperty(attrName));
          bcc.pool.setProperty(attrName, attrValue);
        }
      }
      
      if (user.getId() != null) {
        bcc.pool.setProperty("userid",user.getId());
        p.exec(p.c.sql_ClearRolesofUser);
        p.exec(p.c.sql_UpdateUser);
      } else {
        p.exec(p.c.sql_InsertUser);
        ResultSet rs = null;
        if (p.c.sql_GetUserIdByName != null) {
          rs = p.execQ(p.c.sql_GetUserIdByName);
        } else {
          rs = p.execQ(p.c.sql_GetUserInfoByName);
        }
        rs.next();
        user.setId(rs.getString("ID"));
        rs.getStatement().close();
      }
      
      bcc.pool.setProperty("userid", user.getId());
      Role[] roles = user.getRoles();
      if (roles != null) {
        for (int i = 0; i < roles.length; i++) {
          bcc.pool.setProperty("roleid", roles[i].getId());
          p.exec(p.c.sql_AddRoleToUser);
        }
      }
      
    } catch(Exception ex) {
      throw new SaveSecurityEntityException(ex);
    }
  }
  
  public void authenticateUser(User user, String password) throws AuthenticationException {
    _authenticateUser(user, password);
  }
  
  public String encryptPassword(String password) {
    return _encryptPassword(password);
  }
  
  
  //
  // ROLE INTERFACE METHODS
  //
  
  private Role openRole(BrokerCommunicationContext bcc, ResultSet rs) throws Exception {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    String id = rs.getString("ID");
    String name = rs.getString("NAME");
    String descr = rs.getString("DESCR");
    Role role = new Role(name,descr);
    role.setId(id);
    
    ResultSetMetaData md = rs.getMetaData();
    int columnCount = md.getColumnCount();
    for(int i = 1; i <= columnCount; i++) {
      String colName = md.getColumnName(i);
      if (!colName.equals("ID") && !colName.equals("NAME") && !colName.equals("DESCR")) {
        String colValue = rs.getString(i);
        role.setAttribute(colName,safeval(colValue));
      }
    }
    
    bcc.pool.setProperty("roleid",id);
    rs = p.execQ(p.c.sql_GetCheckPointsofRole);
    while (rs.next()) {
      //      id = rs.getString("ID");
      //      name = rs.getString("NAME");
      //      descr = rs.getString("DESCR");
      CheckPoint cp = getCheckPoint(bcc,rs.getString("NAME"));
      //      cp.setId(id);
      role.addCheckPoint(cp);
    }
    rs.getStatement().close();
    
    return role;
  }
  
  
  public Role getRole(BrokerCommunicationContext bcc, String roleName) throws MissingSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    Role result = null;
    
    try {
      bcc.pool.setProperty("rolename",roleName);
      ResultSet rs = p.execQ(p.c.sql_GetRoleInfoByName);
      if (rs.next()) {
        result = openRole(bcc,rs);
        rs.getStatement().close();
        
      } else {
        rs.getStatement().close();
        throw new MissingSecurityEntityException("Role " + roleName + " not found");
      }
    } catch (Exception e) {
      throw new MissingSecurityEntityException(e);
    }
    
    return result;
    
  }
  
  public Role[] getRoles(BrokerCommunicationContext bcc) {
    
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    Role[] result = null;
    
    try {
      ResultSet rs = p.execQ(p.c.sql_GetAllRoles);
      LinkedList ll = new LinkedList();
      while (rs.next()) {
        Role role = openRole(bcc,rs);
        ll.addLast(role);
      }
      rs.getStatement().close();
      
      result = new Role[ll.size()];
      ll.toArray(result);
      
    } catch (Exception e) {
      e.printStackTrace();
    } 
    
    return result;
    
  }
  
  public void saveRole(BrokerCommunicationContext bcc, Role role) throws SaveSecurityEntityException {
    
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    try {
      bcc.pool.setProperty("rolename",role.getName());
      bcc.pool.setProperty("roledescr",safeval(role.getDescription()));
      
      Properties attrs = role.getAttributes();
      if (attrs != null) {
        Enumeration en = attrs.propertyNames();
        while (en.hasMoreElements()) {
          String attrName = (String)en.nextElement();
          String attrValue = safeval(attrs.getProperty(attrName));
          bcc.pool.setProperty(attrName, attrValue);
        }
      }
      
      if (role.getId() != null) {
        bcc.pool.setProperty("roleid",role.getId());
        p.exec(p.c.sql_ClearCheckPointsofRole);
        p.exec(p.c.sql_UpdateRole);
      } else {
        p.exec(p.c.sql_InsertRole);
        ResultSet rs = p.execQ(p.c.sql_GetRoleInfoByName);
        rs.next();
        role.setId(rs.getString("ID"));
        rs.getStatement().close();
      }
      
      CheckPoint[] cps = role.getCheckPoints();
      if (cps != null) {
        for (int i = 0; i < cps.length; i++) {
          bcc.pool.setProperty("cpid", cps[i].getId());
          p.exec(p.c.sql_AddCheckPointToRole);
        }
      }
    } catch (Exception e) {
      throw new SaveSecurityEntityException(e);
    }
  }
  
  public void deleteRole(BrokerCommunicationContext bcc, String roleName) throws DeleteSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    try {
      
      bcc.pool.setProperty("rolename",roleName);
      p.exec(p.c.sql_RemoveRole);
      
    } catch(Exception e) {
      throw new DeleteSecurityEntityException(e);
    }
  }
  
  //
  // CHECKPOINTS INTERFACE METHODS
  //
  
  private CheckPoint openCheckPoint(BrokerCommunicationContext bcc, ResultSet rs) throws Exception {
    String id = rs.getString("ID");
    String name = rs.getString("NAME");
    String descr = rs.getString("DESCR");
    CheckPoint cp = new CheckPoint(name,descr);
    cp.setId(id);
    
    ResultSetMetaData md = rs.getMetaData();
    int columnCount = md.getColumnCount();
    for(int i = 1; i <= columnCount; i++) {
      String colName = md.getColumnName(i);
      if (!colName.equals("ID") && !colName.equals("NAME") && !colName.equals("DESCR")) {
        String colValue = rs.getString(i);
        cp.setAttribute(colName,safeval(colValue));
      }
    }
    
    return cp;
  }
  
  public CheckPoint getCheckPoint(BrokerCommunicationContext bcc, String checkPointName) throws MissingSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    CheckPoint result = null;
    try {
      bcc.pool.setProperty("cpname",checkPointName);
      ResultSet rs = p.execQ(p.c.sql_GetCheckPointInfoByName);
      if (rs.next()) {
        result = openCheckPoint(bcc,rs);
        rs.getStatement().close();
        
      } else {
        rs.getStatement().close();
        throw new MissingSecurityEntityException("CheckPoint " + checkPointName + " not found.");
      }
      
    } catch (Exception e) {
      throw new MissingSecurityEntityException(e);
    }
    return result;
  }
  
  public CheckPoint[] getCheckPoints(BrokerCommunicationContext bcc) {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    CheckPoint[] result = null;
    
    try {
      
      LinkedList ll = new LinkedList();
      ResultSet rs = p.execQ(p.c.sql_GetAllCheckPoints);
      while (rs.next()) {
        CheckPoint cp = openCheckPoint(bcc,rs);
        ll.addLast(cp);
      }
      rs.getStatement().close();
      
      result = new CheckPoint[ll.size()];
      ll.toArray(result);
      
    } catch (Exception ex) {ex.printStackTrace();}
    
    return result;
  }
  
  public void saveCheckPoint(BrokerCommunicationContext bcc, CheckPoint cp) throws SaveSecurityEntityException {
    
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    if (cp == null || cp.getName().trim().equals("")) {
      return;
    }
    
    try {
      if (cp.getId() == null) {
        bcc.pool.setProperty("cpname",cp.getName());
        bcc.pool.setProperty("cpdescr",safeval(cp.getDescription()));
        
        Properties attrs = cp.getAttributes();
        if (attrs != null) {
          Enumeration en = attrs.propertyNames();
          while (en.hasMoreElements()) {
            String attrName = (String)en.nextElement();
            String attrValue = safeval(attrs.getProperty(attrName));
            bcc.pool.setProperty(attrName, attrValue);
          }
        }
        
        p.exec(p.c.sql_InsertCheckPoint);
        ResultSet rs = p.execQ(p.c.sql_GetCheckPointInfoByName);
        rs.next();
        cp.setId(rs.getString("ID"));
        rs.getStatement().close();
      } else {
        bcc.pool.setProperty("cpid",cp.getId());
        bcc.pool.setProperty("cpdescr",safeval(cp.getDescription()));
        p.exec(p.c.sql_UpdateCheckPoint);
      }
      
    } catch (Exception e) {
      throw new SaveSecurityEntityException(e);
    }
    
  }
  
  public void deleteCheckPoint(BrokerCommunicationContext bcc, String checkPointName) throws DeleteSecurityEntityException {
    PreparedContext p = (PreparedContext)bcc.preparedObject;
    
    try {
      bcc.pool.setProperty("cpname",checkPointName);
      p.exec(p.c.sql_RemoveCheckPoint);
    } catch (Exception e) {
      throw new DeleteSecurityEntityException(e);
    }
    
  }
  
  //
  // INIT, PREPARE, CLEANUP INTERFACE METHODS
  //
  public Object init(MoreMotionRequest request, String conName, Map brokerParams) throws SecurityBrokerException {
    
    String conds = XString.before(conName,';');
    String addParms = XString.after(conName,';',null);
    
    try {
      
      Config conConfig = ConfigManager.getConfig("jdbcConnection", conName);
      
      if (conConfig == null) {
        throw new Exception("Connection configuration  " + conConfig + " not found!");
      }
      
      BrokerInitialContext c = new BrokerInitialContext(conConfig);
      
      c.pool.putAll(brokerParams);
      
      // Not functional!
      if (addParms != null) {
        StringParser sp = new StringParser(addParms,';');
        while (!sp.eos()) {
          String parmDef = sp.nextItem();
          String parmName = XString.before(parmDef,'=',null);
          String parmValue = XString.after(parmDef,'=',null);
          if (parmName != null & parmValue != null) {
            c.pool.setProperty(parmName.trim(), parmValue.trim());
          }
        }
      }
      
      String debug = c.pool.getProperty("debug");
      if (debug != null && debug.equalsIgnoreCase("true")) {
        c.debug = true;
        System.out.println("RelDBSecurityBroker debug is set to 'true' ...");
      } else {
        c.debug = false;
      }
      
      /*
       * Load SQL Patterns
       */
      String fileName = c.pool.getProperty("sqlPatternFile");
      String filePath = SystemConfig.mmHomePath() + "/config/security/bsp";
      File file = new File(filePath,fileName);
      loadSQLPatterns(file,c);
      
      
      BrokerCommunicationContext bcc = (BrokerCommunicationContext)prepare(request,c);
      PreparedContext p = (PreparedContext)bcc.preparedObject;
      
      ResultSet rs = p.execQ(c.sql_CheckTables);
      boolean exist = false;
      if (rs.next()) {
        if (rs.getString(1) != null) {
          exist = true;
        }
      }
      rs.getStatement().close();
      if (!exist) {
        p.exec(c.sql_CreateTables);
        p.exec(c.sql_FillInInitialData);
      }
      
      rs = p.execQ(c.sql_CheckInitialData);
      if (!rs.next() || rs.getString(1) == null) {
        p.exec(c.sql_FillInInitialData);
      }
      rs.getStatement().close();
      
      return c;
      
    } catch (Exception e) {
      throw new SecurityBrokerException(e);
    }
    
  }
  
  private void loadSQLPatterns(File file, BrokerInitialContext c) throws Exception {
    BufferedReader br = new BufferedReader(new FileReader(file));
    String line = null;
    int state = 0;
    String name = null;
    String pattern = "";
    while ((line = br.readLine()) != null) {
      
      if (state == 0 && line.startsWith("#")) {
        name = line;
        state = 1;
      } else if (state == 1 && line.equals("{")) {
        state = 2;
      } else if (state == 2 && !line.equals("}")) {
        pattern = pattern + "\n" + line;
      } else if (state == 2 && line.equals("}")) {
        if (false);
        else if (name.equals("#Update CheckPoint")) c.sql_UpdateCheckPoint = pattern;
        else if (name.equals("#Insert CheckPoint")) c.sql_InsertCheckPoint = pattern;
        else if (name.equals("#Remove CheckPoint")) c.sql_RemoveCheckPoint = pattern;
        else if (name.equals("#Get CheckPoint Info By Name")) c.sql_GetCheckPointInfoByName = pattern;
        else if (name.equals("#Get All CheckPoints")) c.sql_GetAllCheckPoints = pattern;
        else if (name.equals("#Update Role")) c.sql_UpdateRole = pattern;
        else if (name.equals("#Insert Role")) c.sql_InsertRole = pattern;
        else if (name.equals("#Remove Role")) c.sql_RemoveRole = pattern;
        else if (name.equals("#Get Role Info By Name")) c.sql_GetRoleInfoByName = pattern;
        else if (name.equals("#Get All Roles")) c.sql_GetAllRoles = pattern;
        else if (name.equals("#Add CheckPoint To Role")) c.sql_AddCheckPointToRole = pattern;
        else if (name.equals("#Clear CheckPoints of Role")) c.sql_ClearCheckPointsofRole = pattern;
        else if (name.equals("#Get CheckPoints of Role")) c.sql_GetCheckPointsofRole = pattern;
        else if (name.equals("#Update User")) c.sql_UpdateUser = pattern;
        else if (name.equals("#Insert User")) c.sql_InsertUser = pattern;
        else if (name.equals("#Remove User")) c.sql_RemoveUser = pattern;
        else if (name.equals("#Get User Id By Name")) c.sql_GetUserIdByName = pattern;
        else if (name.equals("#Get User Info By Name")) c.sql_GetUserInfoByName = pattern;
        else if (name.equals("#Get User Info By Email")) c.sql_GetUserInfoByEmail = pattern;
        else if (name.equals("#Get All Users")) c.sql_GetAllUsers = pattern;
        else if (name.equals("#Add Role To User")) c.sql_AddRoleToUser = pattern;
        else if (name.equals("#Clear Roles of User")) c.sql_ClearRolesofUser = pattern;
        else if (name.equals("#Get Roles of User")) c.sql_GetRolesofUser = pattern;
        else if (name.equals("#Search Users")) c.sql_SearchUsers = pattern;
        
        else if (name.equals("#Check Tables")) c.sql_CheckTables = pattern;
        else if (name.equals("#Create Tables")) c.sql_CreateTables = pattern;
        else if (name.equals("#Check Initial Data")) c.sql_CheckInitialData = pattern;
        else if (name.equals("#Fill In Initial Data")) c.sql_FillInInitialData = pattern;
        else {
          br.close();
          throw new Exception("Invalid SQL Pattern Name in File " + file.getName());
        }
        state = 0;
        pattern = "";
      }
      
    }
    br.close();
  }
  
  public BrokerCommunicationContext prepare(MoreMotionRequest request, Object initialObject) throws SecurityBrokerException {
    // System.out.println("prepare called...");
    try {
      BrokerCommunicationContext bcc = new BrokerCommunicationContext();
      BrokerInitialContext c = (BrokerInitialContext)initialObject;
      Connection con = JdbcConnection.getConnection(request, c.conConfig);
      bcc.pool = new Properties(c.pool);
      bcc.preparedObject = new PreparedContext(c, con, bcc.pool);
      
      return bcc;
    } catch (Exception e) {
      throw new SecurityBrokerException(e);
    }
  }
  
  public void cleanUp(Object preparedObject) {
  }
  
}

class BrokerInitialContext {
  
  protected Config conConfig = null;
  protected Properties pool = null;
  protected boolean debug = false;
  
  protected static String sql_CheckTables = null;
  protected static String sql_CreateTables = null;
  protected static String sql_CheckInitialData = null;
  protected static String sql_FillInInitialData = null;
  
  protected static String sql_UpdateCheckPoint = null;
  protected static String sql_InsertCheckPoint = null;
  protected static String sql_RemoveCheckPoint = null;
  protected static String sql_GetCheckPointInfoByName = null;
  protected static String sql_GetAllCheckPoints = null;
  protected static String sql_UpdateRole = null;
  protected static String sql_InsertRole = null;
  protected static String sql_RemoveRole = null;
  protected static String sql_GetRoleInfoByName = null;
  protected static String sql_GetAllRoles = null;
  protected static String sql_AddCheckPointToRole = null;
  //  protected static String sql_RemoveCheckPointFromRole = null;
  protected static String sql_ClearCheckPointsofRole = null;
  protected static String sql_GetCheckPointsofRole = null;
  protected static String sql_UpdateUser = null;
  protected static String sql_InsertUser = null;
  protected static String sql_RemoveUser = null;
  protected static String sql_GetUserIdByName = null;
  protected static String sql_GetUserInfoByName = null;
  protected static String sql_GetUserInfoByEmail = null;
  protected static String sql_GetAllUsers = null;
  protected static String sql_AddRoleToUser = null;
  //  protected static String sql_RemoveRoleFromUser = null;
  protected static String sql_ClearRolesofUser = null;
  protected static String sql_GetRolesofUser = null;
  
  protected static String sql_SearchUsers = null;
  
  protected BrokerInitialContext(Config conConfig) {
    try {
      this.conConfig = conConfig;
      this.pool = new Properties();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}

class PreparedContext {
  
  protected BrokerInitialContext c;
  protected Connection con;
  protected Properties pool;
  
  protected PreparedContext(BrokerInitialContext c, Connection con, Properties pool) {
    this.c = c;
    this.con = con;
    this.pool = pool;
  }
  
  //  protected void exec(String sql) throws Exception {
  //    Statement stmt = con.createStatement();
  //    stmt.execute(resolve(sql));
  //    stmt.close();
  //  }
  
  protected void exec(String sql) throws Exception {
    
    Statement stmt = con.createStatement();
    String ss = new String( resolve(sql) );
    StringParser sp = new StringParser(ss,';');
    while (!sp.eos()) {
      String s = sp.nextItem();
      if (s.trim().length() > 0) {
        if (c.debug) {
          System.out.println("Update SQL :" + s);
        }
        stmt.execute( s );
      }
    }
    stmt.close();
  }
  
  protected ResultSet execQ(String sql) throws Exception {
    Statement stmt = con.createStatement();
    String resolvedSql = resolve(sql);
    resolvedSql = resolvedSql.trim();
    if (resolvedSql.endsWith(";")) {
      resolvedSql = resolvedSql.substring(0,resolvedSql.length()-1);
    }
    if (c.debug) {
      System.out.println("Select SQL :" + resolvedSql);
    }
    ResultSet rs = stmt.executeQuery(resolvedSql);
    //    stmt.close();
    return rs;
  }
  
  private String safeval(String s) {
    if (s == null) return "";
    return s;
  }
  
  private String resolve(String sql) throws SecurityBrokerException {
    int i = sql.indexOf("@{");
    while (i > -1) {
      int j = sql.indexOf("}",i+2);
      if (j > -1) {
        String sym = sql.substring(i+2,j);
        
        String val;
        if (" userid username email fullname password disabled roleid rolename roledescr cpid cpname cpdescr ".indexOf(" " + sym + " ") > -1) {
          val = pool.getProperty(sym);
          if (val == null) {
            val = safeval(pool.getProperty("_setvar_" + sym));
          }
        } else {
          val = pool.getProperty("_setvar_" + sym);
          if (val == null) {
            val = safeval(pool.getProperty(sym));
          }
        }
        sql = sql.substring(0,i) + val + sql.substring(j+1);
      } else {
        throw new SecurityBrokerException("Corrupt sql pattern. '}' is missing");
      }
      i = sql.indexOf("@{");
    }
    return sql;
  }
  
}
