/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.mmcomp.security;

import java.util.Enumeration;
import java.util.Properties;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataServiceContext;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.security.Role;
import org.moremotion.security.SecurityContext;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RoleList.java 51 2008-03-10 11:46:54Z erkan $
 */
public class RoleList implements DataService {
  
  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    try {
      
      adom.clear();
      
      MoreMotionRequest request = dsc.getRequest();
      String secdom = Utils.getSecurityDomainName(request, dsc.getDataSourceConfig());
      SecurityContext sctx = Utils.getSecurityContext(request, Utils.getSecurityDomainConfig(secdom) );
      
      Role[] roles = sctx.getRoles();
      for (int i = 0; i < roles.length; i++) {
        Role role = roles[i];
        ADOMNode node = new ADOMNode("item");
        node.setNodeValue("id",role.getId());
        node.setNodeValue("name",role.getName());
        node.setNodeValue("desc",role.getDescription());
        
        Properties attrs = role.getAttributes();
        if (attrs != null) {
          Enumeration en = attrs.propertyNames();
          while (en.hasMoreElements()) {
            String attrName = (String)en.nextElement();
            String attrValue = safeval(attrs.getProperty(attrName));
            node.setNodeValue(attrName, attrValue);
          }
        }
        
        adom.addNode(node);
      }
      
    } catch (Exception e) {
      throw new ServiceException(e);
    }
  }
  
  private String safeval(String s) {
    if (s == null) return "";
    return s;
  }
  
  
}
