/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * XmlQueryDataService.java
 *
 * Created on January 25, 2002, 4:25 PM
 */

package org.moremotion.mmcomp.utils;


import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.moremotion.adom.ADOM;
import org.moremotion.cache.XslTransformer;
import org.moremotion.config.Config;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 * Connets an XML source over HTTP and fills in the given{@link org.moremotion.adom.ADOM ADOM}
 * object with the acquired data.
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ExternalXml.java 149 2008-06-11 13:35:46Z erkan $
 */
public class ExternalXml {
  
  MoreMotionRequest request;
  Config config;
  Logger logger;
  
  public ExternalXml(Config config, MoreMotionRequest request, Logger logger) {
    this.config = config;
    this.request = request;
    this.logger = logger;
  }
  
  public String getXml() throws ServiceException, TransformerConfigurationException, TransformerException, ParserConfigurationException, SAXException, IOException, Exception {
    
    String method = config.getParameter("method").resolve(request).stringValue("GET");
    boolean post = method.equalsIgnoreCase("POST");

    DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    
    String params = config.getParameter("params").resolve(request).stringValue(null);
    String queryString = params != null ? makeQueryString(params, builder, !post) : "";
    
    if (logger.LEVEL2) {
      logger.print("Query String : \n" + queryString);
    }
    
    // Document doc = null;
    InputStream is;
    
    String urlStr = config.getParameter("url").resolve(request).stringValue();
    
    if (urlStr.indexOf("://") == -1) {
      File f;
      if (!urlStr.startsWith("/")) {
        f = request.getRealFile(urlStr);
      } else {
        f = new File(urlStr);
      }
      
      is = new FileInputStream(f);
      
    } else {
      
      String qm = "";
      if (queryString.length() > 0) {
        if (urlStr.indexOf('?') == -1) {
          qm = "?";
        } else {
          qm = "&";
        }
      }
      
      String username = config.getParameter("username").resolve(request).stringValue(null);
      String password = null;
      if (username != null) {
        password = config.getParameter("password").resolve(request).stringValue(null);
      }
      
      if (!post) {
        urlStr += qm + queryString;
      }
      URL url = new URL(urlStr);
      HttpURLConnection urlconn = (HttpURLConnection)url.openConnection();
      
      urlconn.setInstanceFollowRedirects(true);
      urlconn.setRequestMethod(method);
      
      // urlconn.setRequestProperty("Connection", "Keep-Alive");
      // urlconn.setRequestProperty("Keep-Alive", "header");
      
      if (password != null) {
        urlconn.setRequestProperty("Authorization", "Basic " +
            (new sun.misc.BASE64Encoder().encode((username + ":" + password).getBytes())));
      }
      
      urlconn.setDoOutput(true);
      
      if (!post) {
        urlconn.connect();
        
      } else {
        urlconn.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
        urlconn.setRequestProperty("Content-Length", queryString.length() + "");
        DataOutputStream stream = new DataOutputStream(urlconn.getOutputStream ());
        stream.writeBytes(queryString);
        stream.flush();
        stream.close();
        
      } 
      
      is = urlconn.getInputStream();
    }
    
    Document doc = builder.parse(is);
    Document outdoc = builder.newDocument();
    
    String xslFileName = config.getParameter("xslfile").resolve(request).stringValue(null);
    if (xslFileName != null) {
      File xslFile;
      if (!xslFileName.startsWith("/")) {
        xslFile = request.getRealFile(xslFileName);
      } else {
        xslFile = new File(xslFileName);
      }
      XslTransformer xslt = SystemContext.getXSLTransformerCache().getTransformer(xslFile);
      Element elm = outdoc.createElement("root");
      Node node = outdoc.appendChild((Node)elm);
      xslt.transform(doc,node);
      
    } else {
      outdoc = doc;
    }
    
    Transformer trans = TransformerFactory.newInstance().newTransformer();
    trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
    StringWriter sink = new StringWriter();
    trans.transform(new DOMSource(outdoc),new StreamResult(sink));
    return sink.toString();
    
  }
  
  private String makeQueryString(String params, DocumentBuilder builder, boolean encode) throws SAXException, IOException {
    
    if (params.equals("<params></params>")) return "";
    
    StringBuffer buf = new StringBuffer();
    
    ByteArrayInputStream bais = new ByteArrayInputStream(params.getBytes());
    
    Document doc = builder.parse(bais);
    
    NodeList nlist = doc.getElementsByTagName("param");
    
    for (int i = 0; i < nlist.getLength(); i++) {
      
      Node node = nlist.item(i);
      
      String name = ((Element)node).getAttribute("name");
      // String value = ((Element)node).getNodeValue();
      String value = ((Element)node).getFirstChild().getNodeValue();
      
      if (buf.length() > 0) {
        buf.append('&');
      }
      buf.append(name);
      buf.append('=');
      
      if (encode) {
        try {
          value = URLEncoder.encode(value,"UTF-8");
        } catch (Exception ignored) {}
      }
      
      buf.append(value);
      
    }
    
    return buf.toString();
    
  }
  
}