/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ADOMManager.java
 *
 * Created on 03 March 2007 Saturday, 17:36
 *
 */

package org.moremotion.adom;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletContext;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMManager.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ADOMManager {
  
  private static final String ADOM_PREFIX = "ADOM_";
  
  /**
   * Creates a new Request ADOM and saves it in the request.
   * @param request Where the ADOM is saved.
   * @param name Name of the ADOM.
   * @return Newly created ADOM.
   */
  public static ADOM newRequestADOM(HttpServletRequest request, String name) {

    ADOM result = new ADOM();
    result.setName(name);
//    result.scope = 's';
    
    request.setAttribute(ADOM_PREFIX + name, result);
    return result;
  }
  
  
  /**
   * Creates a new Session ADOM and saves it in the session.
   * @param session Where the ADOM is saved.
   * @param name Name of the ADOM.
   * @return Newly created ADOM.
   */
  public static ADOM newSessionADOM(HttpSession session, String name) {

    ADOM result = new ADOM();
    result.setName(name);
//    result.scope = 's';
    
    session.setAttribute(ADOM_PREFIX + name, result);
    return result;
  }
  
  /**
   * Creates a new Application ADOM and saves it in the servlet context.
   *
   * @param context Where the ADOM is saved.
   * @param name Name of the ADOM.
   * @return Newly created ADOM.
   */
  public static ADOM newApplicationADOM(ServletContext context, String name) {
    
    ADOM result = new ADOM();
    result.setName(name);
//    result.scope = 'a';
    
    context.setAttribute(ADOM_PREFIX + name, result);
    return result;
  }
  
  /**
   * Finds the ADOM saved in the request by the given name and returns it.
   * @param request Where the ADOM is searched
   * @param name The name of the ADOM
   * @return <CODE>null</CODE> if ADOM cannot be found.
   */
  public static ADOM getRequestADOM(HttpServletRequest request, String name) {
    return (ADOM) request.getAttribute(ADOM_PREFIX + name);
  }
  
  /**
   * Finds the ADOM saved in the session by the given name and returns it.
   * @return <CODE>null</CODE> if ADOM cannot be found.
   */
  public static ADOM getSessionADOM(HttpSession session, String name) {
    return (ADOM) session.getAttribute(ADOM_PREFIX + name);
  }
  
  /**
   * Finds the ADOM saved in the servlet context by the given name and returns it.
   * @return <CODE>null</CODE> if ADOM cannot be found.
   */
  public static ADOM getApplicationADOM(ServletContext context, String name) {
    return (ADOM) context.getAttribute(ADOM_PREFIX + name);
  }
  
  /**
   * Finds the ADOM in the request by given name and removes it.
   */
  public static void removeRequestADOM(HttpServletRequest request, String name) {
    request.removeAttribute(ADOM_PREFIX + name);
  }
  
  /**
   * Finds the ADOM in the session by given name and removes it.
   */
  public static void removeSessionADOM(HttpSession session, String name) {
    session.removeAttribute(ADOM_PREFIX + name);
  }
  
  /**
   * Finds the ADOM in the servlet context by given name and removes it.
   */
  public static void removeApplicationADOM(ServletContext context, String name) {
    context.removeAttribute(ADOM_PREFIX + name);
  }
  
}
