/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.cache;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: PageXMLCacheCleaner.java 51 2008-03-10 11:46:54Z erkan $
 */
public class PageXMLCacheCleaner implements Runnable {
  
  private Thread t;
  private boolean stop = false;
  
  private static PageXMLCacheCleaner Instance;
  private static File cacheDir = null;
  
  /**
   * Creates new PageXMLCacheCleaner
   */
  private PageXMLCacheCleaner(String moremotionHome) {
    String cachePath = moremotionHome + File.separator + "~xmlcache";
    cacheDir = new File(cachePath);
    if (!cacheDir.exists()) {
      cacheDir.mkdirs();
    }
    t = new Thread(this);
    t.setDaemon(true);
    t.start();
  }
  
  /**
   * Returns a single instance of PageXMLCacheCleaner class.
   */
  public static synchronized PageXMLCacheCleaner getInstance(String mmHome) {
    if (Instance == null) {
      Instance = new PageXMLCacheCleaner(mmHome);
    }
    return Instance;
  }
  
  public void run() {
    while(!stop) {
      try {
        synchronized (cacheDir) {
          SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmm");
          Date d = new Date();
          long l = d.getTime() - 1000L * 60 * 30;
          d.setTime(l);
          
          String prefix = "" + format.format(d);
          
          CachedFilesFilter f = new CachedFilesFilter(prefix);
          if (f != null) {
            File[] files = cacheDir.listFiles(f);
            if (files != null) {
              for (int i = 0; i < files.length;i++) {
                files[i].delete();
              }
            }
          }
        }
        t.sleep(1000L * 60 * 10);
      } catch(Exception ex) {
        ex.printStackTrace();
      }
    }
  }
  
  public void stop() {
    this.stop = true;
  }
  
  public void finalize() {
    stop();
  }
  
  class CachedFilesFilter implements java.io.FileFilter {
    private String prefix = null;
    
    CachedFilesFilter(String prefix) {
      this.prefix = prefix;
    }
    
    public boolean accept(File file) {
      String name = file.getName();
      if (name.length() <= prefix.length()) return false;
      String filePrefix = name.substring(0,prefix.length());
      int result = filePrefix.compareTo(prefix);
      if ( result < 0) {
        return true;
      } else {
        return false;
      }
    }
  }
  
}
