/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * XSLTProcessorCache.java
 *
 * Created on 01 November 2005 Tuesday, 19:39
 */

package org.moremotion.cache;

import java.io.File;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: XslTransformerCache.java 51 2008-03-10 11:46:54Z erkan $
 */
public class XslTransformerCache { 
  
  private CompiledObjectCache cache = null;
  
  public XslTransformerCache() {
    this(10);
  }
  
  public XslTransformerCache(int capacity) {
    cache = new CompiledObjectCache();
    cache.setCapacity(capacity);
  }
  
  public void setCapacity(int capacity) {
    if (cache == null) {
      cache = new CompiledObjectCache();
    }  
    cache.setCapacity(capacity);
  }
  
  public XslTransformer getTransformer(File xsl) throws Exception {
    
    XslTransformer result = (XslTransformer)cache.getCompiledObject(xsl);
    if (result == null) {
      synchronized (this) {
        result = (XslTransformer)cache.getCompiledObject(xsl);
        if (result == null) {
          result = new XslTransformer(xsl);
          cache.addCompiledObject(xsl, result);
        }
      }
    }
    
    return result;
  }
  
  public void clear() {
    cache.clear();
  }
  
}