/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Expression.java
 *
 * Created on 14 February 2006 Saturday, 04:02
 */

package org.moremotion.evaluator;

import java.util.ArrayList;
 

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Expression.java 61 2008-03-14 14:56:20Z erkan $
 */
public class Expression implements Evaluable {
  
  private Object expr;
  
  private Node[] nodes = null;
  private boolean parsed = false;
  private SymbolResolver sr = null;
  
  
  
  /** Creates a new instance of Expression */
  public Expression(String str)  {
    this.expr = str.trim();
  }
  
  public Expression(String str, SymbolResolver sr)  {
    this.expr = str.trim();
    this.sr = sr;
  }

  public Expression(Node node) {
    this.expr = new OPNode(node);
    parsed = true;
  }
  
  public Object evaluate(SymbolResolver sr) throws EvaluationException {
    parse();
    return ((OPNode) expr).evaluate(sr);
  }
  
  public String eval() throws EvaluationException {
    Object o = evaluate(this.sr);
    return EvalUtils.toStr(o);
  }
  
  public String eval(SymbolResolver sr) throws EvaluationException {
    Object o = evaluate(sr);
    return EvalUtils.toStr(o);
  }
  
  public boolean evalBool() throws EvaluationException {
    Object o = evaluate(this.sr);
    if (o instanceof Boolean) {
      return ((Boolean)o).booleanValue();
    }
    return EvalUtils.getBoolean(o.toString());
  }
  
  public boolean evalBool(SymbolResolver sr) throws EvaluationException {
    Object o = evaluate(sr);
    if (o instanceof Boolean) {
      return ((Boolean)o).booleanValue();
    }
    return EvalUtils.getBoolean(o.toString());
  }
  
  public double evalNumber() throws EvaluationException {
    Object o = evaluate(this.sr);
    if (o instanceof Double) {
      return ((Double)o).doubleValue();
    }
    return EvalUtils.getNumber(o.toString());
  }
  
  public double evalNumber(SymbolResolver sr) throws EvaluationException {
    Object o = evaluate(sr);
    if (o instanceof Double) {
      return ((Double)o).doubleValue();
    }
    return EvalUtils.getNumber(o.toString());
  }
  
  public String toString() {
    return this.expr.toString();
  }
  
  
  private void parse() throws ParsingException {
    
    if (parsed) return;
    parsed = true;
    char prevOpr = ' ';
    
    ArrayList al = null;
    boolean sf = false; // Space found
    char[] bb = new char[20]; // BraceBuffer
    int bi = -1; // BraceBuffer index
    char quote = ' ';
    Node n = null;
    boolean scopr = false;  // Operator with special chars such as !=<>&|
    
    StringBuffer opr = new StringBuffer();
    boolean oprStarted = false;
    int startPos = 0;
    int endPos = -1;
    
    int i = 0;
    
    String str = (String)this.expr;
    
    while (i < str.length()) {
      
      char cc = str.charAt(i);
      
      if (quote == ' ' && bi == -1) {
        
        if (oprStarted) {
          if (cc == ' ' || cc == '(' || (scopr && "=&|>".indexOf(cc) == -1)) {
            String o = opr.toString();
            if (prevOpr == '?' && o.charAt(0) == ';' ) {
              o = ":";  
            }
            prevOpr = o.charAt(0); 
            n.setOperator( Operator.getOperatorAsInt(o) );
            
            while (i < str.length() && str.charAt(i) == ' ') i++;
            if (i == str.length()) {
              throw new ParsingException("PREMATURE_END_OF_EXPRESSION");
            }
            startPos = i;
            oprStarted = false;
            sf = false;
            continue;
            
          } else {
            opr.append(cc);
          }
        }
        else if (cc == ' ') {
          sf = true;
          endPos = i;
        }
        else {
          scopr =  "+*&|!=<>?;".indexOf(cc) > -1;
          if ((sf && cc != '(') || scopr) {
            if (endPos == -1) endPos = i;
            if (endPos <= startPos) endPos = i; 
            oprStarted = true;
            n = new Node( str.substring(startPos,endPos) );
            if (al == null) al = new ArrayList();
            al.add(n);
            opr.setLength(0);
            opr.append(cc);
          }
        }
        
      }
      
      char ch = ' ';
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      
      if (quote == ' ') {
        if (cc == '(') ch = ')';
        if (cc == '[') ch = ']';
        else if (bi > -1 && cc == bb[bi]) {
          bi--;
          if (bi == -1) {
            sf = true;
            endPos = i+1;
          }
        }
      }
      if (ch != ' ') {
        bb[++bi] = ch;
        ch = ' ';
      }
      i++;
    }
    
    if (oprStarted) {
      throw new ParsingException("PREMATURE_END_OF_EXPRESSION");
    }
    
    n = new Node(str.substring(startPos,i).trim());
    if (al == null) al = new ArrayList();
    al.add(n);
    
    nodes = new Node[al.size()];
    al.toArray(nodes);
    
    this.expr = new OPNode(nodes,0,nodes.length);
  }
  
}
