/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Function.java
 *
 * Created on 14 January 2006 Saturday, 04:06
 */

package org.moremotion.evaluator;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;

import org.moremotion.config.ConfigurationException;


import org.moremotion.resource.ResourceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Function.java 171 2008-07-10 07:46:02Z mustafa $
 */

public class Function implements Evaluable {
  
  private String name = null;
  private Expression[] params;
  private String str;
  private char dt = ' ';
  private boolean parsed = false;
  
  /** Creates a new instance of Function */
  public Function(String str) {
    this.str = str;
  }
  
  public Function(String name, Expression[] params) {
    this.name = name;
    this.params = params;
    parsed = true;
  }
  
  public Object evaluate(SymbolResolver sr) throws EvaluationException {
    
    Object result = null;
      
    try {
      
      parse();
      
      if (name.equals("vof")) result = Funcs.vof(params, sr);
      else if (name.equals("sstr")) result = Funcs.sstr(params, sr);
      else if (name.equals("set")) result = Funcs.set(params, sr);
      else if (name.equals("trim")) result = Funcs.trim(params, sr);
      else if (name.equals("not")) result = Funcs.not(params, sr);
      else if (name.equals("pos")) result = Funcs.pos(params, sr);
      else if (name.equals("lpos")) result = Funcs.lpos(params, sr);
      else if (name.equals("len")) result = Funcs.len(params, sr);
      else if (name.equals("ismodified")) result = Funcs.ismodified(params, sr);
      else if (name.equals("sstra")) result = Funcs.sstra(params, sr);
      else if (name.equals("sstrb")) result = Funcs.sstrb(params, sr);
      else if (name.equals("repl")) result = Funcs.repl(params, sr);
      else if (name.equals("triml")) result = Funcs.triml(params, sr);
      else if (name.equals("trimr")) result = Funcs.trimr(params, sr);
      else if (name.equals("encr")) result = Funcs.encr(params, sr);
      else if (name.equals("decr")) result = Funcs.decr(params, sr);
      else if (name.equals("isnull")) result = Funcs.isnull(params, sr);
      else if (name.equals("isnum")) result = Funcs.isnum(params, sr);
      else if (name.equals("num")) result = Funcs.num(params, sr);
      else if (name.equals("round")) result = Funcs.round(params, sr);
      else if (name.equals("bool")) result = Funcs.bool(params, sr);
      else if (name.equals("exists")) result = Funcs.exists(params, sr);
      else if (name.equals("rsstr")) result = Funcs.rsstr(params, sr);
      else if (name.equals("acc")) result = Funcs.acc(params, sr);
      else if (name.equals("lcase")) result = Funcs.lcase(params, sr);
      else if (name.equals("ucase")) result = Funcs.ucase(params, sr);
      else if (name.equals("accfa")) result = Funcs.accfa(params, sr);
      else if (name.equals("accfs")) result = Funcs.accfs(params, sr);
      else if (name.equals("sstral")) result = Funcs.sstral(params, sr);
      else if (name.equals("sstrbl")) result = Funcs.sstrbl(params, sr);
      else if (name.equals("nl")) result = Funcs.nl(params, sr);
      else if (name.equals("chr")) result = Funcs.chr(params, sr);
      else if (name.equals("chrcode")) result = Funcs.chrcode(params, sr);
      else if (name.equals("echo")) result = Funcs.echo(params, sr);
      else if (name.equals("filepath")) result = Funcs.filepath(params, sr);
      else if (name.equals("res")) result = Funcs.res(params, sr);
      else if (name.equals("dq")) result = Funcs.dq(params, sr);
      else if (name.equals("now")) result = Funcs.now(params, sr);
      else if (name.equals("push")) result = Funcs.push(params, sr);
      else if (name.equals("peek")) result = Funcs.peek(params, sr);
      else if (name.equals("pop")) result = Funcs.pop(params, sr);
      else if (name.equals("incrdate")) result = Funcs.incrdate(params, sr);
      else if (name.equals("parsedate")) result = Funcs.parsedate(params, sr);
      else if (name.equals("fmtdate")) result = Funcs.fmtdate(params, sr);
      else if (name.equals("fmtnum")) result = Funcs.fmtnum(params, sr);
      else if (name.equals("parsenum")) result = Funcs.parsenum(params, sr);
      else if (name.equals("shellexec")) result = Funcs.shellexec(params, sr);
      else if (name.equals("msg")) result = Funcs.msg(params, sr);
      else if (name.equals("escape")) result = Funcs.escape(params, sr);
      else if (name.equals("dspaging")) result = Funcs.dspaging(params, sr);
      else if (name.equals("setnp")) result = Funcs.setnextpage(params, sr);
      else if (name.equals("getnp")) result = Funcs.getnextpage(params, sr);
      else if (name.equals("vov")) result = Funcs.vov(params, sr);
      else if (name.equals("cdata")) result = Funcs.cdata(params, sr);
      else if (name.equals("vore")) result = Funcs.vore(params, sr);
      else if (name.equals("sysparam")) result = Funcs.sysparam(params, sr); 
      else if (name.equals("cfgparam")) result = Funcs.cfgparam(params, sr); 
      else if (name.equals("resolve")) result = Funcs.resolve(params, sr); 
      else result = customFunc(name, params, sr);
      
    } catch (RemoveInstructionException ex) {     
      if (((MMSymbolResolver)sr).bypassRIE) {
        throw ex;
      }
      result = "";
      
    } catch (ConfigurationException ex) {
      throw new EvaluationException(ex);
    } catch (ResourceException ex) {
      throw new EvaluationException(ex);
    } catch (MScriptException ex) {
      throw new EvaluationException(ex);
    } catch (IOException ex) {
      throw new EvaluationException(ex);
    } catch (ParseException ex) {
      throw new EvaluationException(ex);
    } catch (InterruptedException ex) {
      throw new EvaluationException(ex);
    } catch (ClassCastException cce) {
      throw new EvaluationException("MSCRIPT_ERROR_INVALID_PARAMETER_TYPE",new String[]{ name },cce);
    }
    
    if (dt == 'n') {
      if (result instanceof Double) return result;
      return new Double(result.toString());
    }
    
    if (dt == 'b') {
      if (result instanceof Boolean) return result;
      return new Boolean(result.toString());
    }
    
    return result;
    
  }
  
  
  private void parseName(String s) {
    String pfx = XString.before(s,':',"");
    this.name = XString.after(s,':');
    for (int i = 0; i < pfx.length(); i++) {
      char c = pfx.charAt(i);
      switch (c) {
        case 'n':
        case 'b': dt = c;
      }
    }
    
  }
  
  public char getDataType() {
    return this.dt;
  }
  
  
  public void parse()  {
    if (parsed) return;
    parsed = true;
    int i = 0;
    outer: {
      while (str.charAt(i) != '(') {
        if (str.charAt(i) == ' ') {
          parseName(str.substring(0,i));
          i++;
          while (str.charAt(i) != '(') i++;
          break outer;
        }
        i++;
      }
      parseName(str.substring(0,i));
    }
    
    str = str.substring(i+1,str.length()-1);
    StringParser p = new StringParser(str,',');
    p.trimmed();
    int c = 0;
    ArrayList a = new ArrayList();
    while (!p.eos()) {
      String prm = p.nextItem();
      if (prm.length() == 0) a.add(null);
      else a.add(new Expression(prm));
    }
    
    params = new Expression[a.size()];
    a.toArray(params);
    
  }
  
  
  public String toString() {
    try {
      parse();
    } catch(Exception e) {
      e.printStackTrace();
    }
    StringBuffer sb = new StringBuffer();
    if (dt != ' ') sb.append(dt);
    sb.append(name);
    sb.append('(');
    for (int i = 0; i < params.length; i++) {
      sb.append((i > 0 ? ", " : "") + params[i].toString());
    }
    sb.append(')');
    return sb.toString();
  }
  
  
  private Object customFunc(String funcName, Expression[] params, SymbolResolver sr) throws ConfigurationException, MScriptFunctionException {
    
    MScriptFunctionContext msfc = new MScriptFunctionContext();
    
    MMSymbolResolver mmsr = (MMSymbolResolver)sr;
    MoreMotionRequest request = mmsr.getRequest();
    
    MScriptFunctionConfig msfconfig = MScriptFunctionManager.getMScriptFunctionConfig(funcName);
    if (msfconfig != null) {
      funcName = msfconfig.getClassName();
      msfc.setMScriptFunctionConfig(msfconfig);
      msfc.setLogger(msfconfig.getName(), msfconfig.getDebug(request),msfconfig.getUnitName(request));
    }
    
    msfc.setRequest(request);
    
    MScriptFunction msfunc = MScriptFunctionManager.loadFunction(funcName);
    
    return msfunc.execute(params, msfc);
  }
  
  
  
}
