/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MScriptResolver.java
 *
 * Created on 06 August 2004 Friday, 17:31
 */

package org.moremotion.evaluator;

import java.util.HashMap;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MScriptResolver.java 218 2008-09-15 06:13:05Z erkan $
 */

public class MScriptResolver {
  
  private static final char __uisign = '\5';
  protected static final int NONE = 0;
  protected static final int DOIF = 1;
  protected static final int DOELSEIF = 2;
  protected static final int DOELSE = 3;
  protected static final int DOEND = 4;
  private static final String SPACE = "                                                      " + 
                                      "                                                      " +
                                      "                                                      " + 
                                      "                                                      ";
  
  private MMSymbolResolver sr = null;
  private String str = null;
  private StringBuffer line = new StringBuffer(100);
  private int spos = -1;
  private int nlpos = -1;
  //  private int lcsr = -1;
  private IgnoreStack igst;
  private boolean eos = false;
  private boolean hasMScript = false;
  private int sssPos = 0;
  private String func = null;
  private String params = null;
  private boolean exit = false;
  
  private char dlm;
  private int locatePos = -1;
  private char[] bracebuffer = new char[20];
  private int braceindex = -1;
  private char quote = ' ';
  private boolean onefound = false;
  private StringBuffer nextStat = new StringBuffer(200);
  
  private boolean sqlInjectionCheck = false;
  
  private static HashMap funcPool = null;
  
  static {
    funcPool = new HashMap();
  }
  
  /**
   * Creates a new instance of MScriptResolver
   */
  
  public MScriptResolver(String str, MMSymbolResolver sr) {
    this.str = str;
    this.sr = sr;
  }
  
  public MScriptResolver(String str, MMSymbolResolver sr, char dlm) {
    this.str = str;
    this.sr = sr;
    this.dlm = dlm;
  }
  
  public void setDoubleBackSlashes(boolean value) {
    if (sr != null) sr.setDoubleBackSlashes(value);
  }
  
  public void setSqlInjectionCheck(boolean value) {
    this.sqlInjectionCheck = value;
  }
  
  public String nextStatement() throws MScriptException {
    String result = _nextStatement();
    if (result == null || result.length() == 0) return null;
    return result;
  }
  
  public String resolve() throws MScriptException {
    return _resolve();
  }
  
  public static boolean isSymbol(String s) {
    return EvalUtils.isSymbol(s);
  }
  
  
  public static String getConstant(String str) throws InvalidEntityException {
    return EvalUtils.getConstant(str);
  }
  
 
  private void nextLine(boolean join) {
    if (spos >= str.length()) {
      eos = true;
      return;
    }
    int i;
    if (spos == 0 && nlpos != -1) i = nlpos;
    else i = str.indexOf('\n',spos);
    if (i == -1) i = str.length();
    
    String s = str.substring(spos,i);
    spos = i + 1;
    
    int rempos = s.indexOf("@//");
    if (rempos > -1) {
      s = s.substring(0, rempos);
    }
    
    if (!join) line.setLength(0);
    line.append(s);
  }
  
  public int locateClosingParanthesis(int i) {
    
    char[] bracebuffer = new char[20];
    int braceindex = -1;
    char quote = ' ';
    
    while (!eos) {
      
      if (i >= line.length()) {
        nextLine(true);
      }
      
      while (i < line.length()) {
        char cc = line.charAt(i);
        char ch = ' ';
        if (cc == ')' && quote == ' ' && braceindex == -1) return i;
        if (quote == ' ') {
          if (cc == '\'' || cc == '"') quote = cc;
        } else if (quote == cc) quote = ' ';
        if (quote == ' ') {
          if (cc == '(') ch = ')';
          else if (cc == '[') ch = ']';
          else if (cc == '{') ch = '}';
          else if (braceindex > -1 && cc == bracebuffer[braceindex]) braceindex--;
        }
        if (ch != ' ') {
          bracebuffer[++braceindex] = ch;
          ch = ' ';
        }
        i++;
      }
      
    }
    
    return -1;
  }
  
  private int locateStatementDelimiter(String buffer, int i) {
    if (i > buffer.length()) return -1;
    for (; i < buffer.length(); i++) {
      char cc = buffer.charAt(i);
      char ch = ' ';
      if (onefound && cc == dlm && quote == ' ' && braceindex == -1) return i;
      if (cc != ' ') onefound = true;
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      } else if (quote == cc) quote = ' ';
      if (quote == ' ') {
        if (cc == '(') ch = ')';
        else if (cc == '[') ch = ']';
        else if (cc == '{') ch = '}';
        else if (braceindex > -1 && cc == bracebuffer[braceindex]) braceindex--;
      }
      if (ch != ' ') {
        bracebuffer[++braceindex] = ch;
        ch = ' ';
      }
      
    }
    if (dlm == ' ' && onefound && quote == ' ' && braceindex == -1) return i;
    return -1;
  }
  
  private String trimDlm(String s) {
    if (s.length() == 0) return s;
    int lpos = s.length() - 1;
    for (int j = lpos; j > -1; j--) {
      char c = s.charAt(j);
      if (c != ' ' && c != dlm) {
        if (j == lpos) return s;
        return s.substring(0,j+1);
      }
    }
    return s;
  }
  
  private String _nextStatement() throws MScriptException  {
    
    if (eos) {
      return null;
    }
    
    if (spos == -1) {
      spos = 0;
      if (str == null || str.length() == 0) {
        eos = true;
        return null;
      }
      hasMScript = str.indexOf('@') > -1;
      if (hasMScript) {
        if (str.indexOf("@do") > -1) {
          igst = new IgnoreStack(20);
        }
      }
      
      if ( str.indexOf('\n') == -1 ) {
        nextLine(false);
        boolean rol = resolveOneLine(igst);
        eos = true;
        return trimDlm(line.toString());
      }
    }
    
    nextStat.setLength(0);
    boolean first = true;
    
    braceindex = -1;
    quote = ' ';
    onefound = false;
    //    locatePos = 0;
    
    boolean rol = true;
    
    if (sssPos == 0) nextLine(false);
    while(!eos) {
      
      try {
        if (sssPos == 0) rol = resolveOneLine(igst);
        
      } catch (MScriptException mse) {
        throw new MScriptException("MSCRIPT_ERROR_SCRIPT_SYNTAX_ERROR", new String[] {line + ""}, mse);
      }
      
      if (rol) {
        String lne = line.toString();

        int ind = igst == null ? 0 : igst.getIndent();
        if (ind > 0 && lne.startsWith(SPACE.substring(0,ind) ) ) {
          lne = lne.substring(ind);
        }
            
        boolean skip = false;
        int __uisign_Pos = lne.indexOf(__uisign);  
        if (__uisign_Pos > -1) {
          skip = true;
          for (int j = 0; j < lne.length(); j++) {
            if (lne.charAt(j) != __uisign) {
              // There is a character in the line other than __uisign 
              // Cannot skip
              skip = false;
              break;
            }
          }
        }
        
        if (!skip) {
          
          if (__uisign_Pos > -1) {
            lne = remove_uisign_chars(lne);
          }

          if (!first) {
            nextStat.append('\n');
          }
          first = false;

          if (dlm != '\n') {

            int j = locateStatementDelimiter(lne,sssPos);

            if (j > -1) {
              // Statement Delimiter Found.
              String s = lne.substring(sssPos,j).trim();
              nextStat.append(s);
              sssPos = j+1;
              while (sssPos < lne.length() && lne.charAt(sssPos) == ' ') sssPos++;
              if (sssPos >= lne.length()) sssPos = 0;
              return nextStat.toString().trim();

            } else {
              nextStat.append(lne);
              sssPos = 0;
            }
          } else {
            String result = lne.toString().trim();
            if (result.length() > 0) return result;
          }

        }
        
      }
      
      nextLine(false);
    }
    
    return nextStat.toString().trim();
    
  }
  
  private String _resolve() throws MScriptException {
    
    if (str == null || str.length() == 0) return "";
    if (str.indexOf('@') == -1) return str;
    
    spos = 0;
    eos = false;
    if ( str.indexOf('\n') == -1 ) {
      /*
       * Single Line
       */
      nextLine(false);
      
      boolean rol = resolveOneLine(igst);
      
      if (!rol || line == null) {
        return "";
        
      } else {
        String s = line.toString();
        if (s.indexOf(__uisign) > -1) {
          return remove_uisign_chars(line.toString());
        } else {
          return s;
        }
      }
      
    } else {
      
      IgnoreStack ist = new IgnoreStack(20);
      StringBuffer result = new StringBuffer(str.length() * 2);
      
      boolean first = true;
      boolean rol = false;
      
      nextLine(false);
      
      while(!eos) {
        
        try {
          rol = resolveOneLine(ist);
        } catch (MScriptException mse) {
          throw new MScriptException("MSCRIPT_ERROR_SCRIPT_SYNTAX_ERROR", new String[] {line + ""}, mse);
        }
        if (rol) {
          
          String lne = line.toString();
          
          int ind = ist.getIndent();
          if (ind > 0 && lne.startsWith(SPACE.substring(0,ind) ) ) {
            lne = lne.substring(ind);
          }
            
          boolean skip = false;
          int __uisign_Pos = lne.indexOf(__uisign);  
          if (__uisign_Pos > -1) {
            skip = true;
            for (int j = 0; j < lne.length(); j++) {
              if (lne.charAt(j) != __uisign) {
                // There is a character in the line other than __uisign. 
                // Cannot skip
                skip = false;
                break;
              }
            }
          }
        
          if (!skip) {
            
            if (__uisign_Pos > -1) {
              lne = remove_uisign_chars(lne);
            }
            
            if (!first) {
              result.append('\n');
            }
            first = false;
            
            result.append(lne);
            if (exit) break;
          }
        }
        
        nextLine(false);
      }
      
      return result.toString();
    }
    
  }
  
  
  private String remove_uisign_chars(String s) {
    StringBuffer buf = new StringBuffer(s.length());
    for (int i = 0; i < s.length(); i++) {
      char c = s.charAt(i);
      if (c != __uisign) {
        buf.append(c);
      }
    }
    return buf.toString();
  }
  
  private int parseFuncAndParms(int i) throws MScriptException {
    
    int result = -1;
    func = null;
    params = "";
    
    for (int j = i + 1; j < line.length(); j++) {
      if (line.charAt(j) == '(') {
        func = line.substring(i+1,j);
        
        result = locateClosingParanthesis(++j);
        if (result == -1) {
          throw new MScriptException("MSCRIPT_ERROR_MISSING_CLOSING_PARANTHESIS",new String[]{func},null);
        }
        params = line.substring(j,result++);
        break;
      }
      if (line.charAt(j) == ' ') break;
    }
    
    return result;
  }
  
  
  private int doPos() {
    int i = line.indexOf("@do");
    while ( i > -1) {
      for (int j = i+3; j < line.length(); j++) {
        if (line.charAt(j) == '(') {
          String f = line.substring(i,j);
          if (f.equals("@doif") || f.equals("@doelse") || f.equals("@doend") || f.equals("@doelseif")) return i;
        }
      }
      i = line.indexOf("@do",i+3);
    }
    return -1;
  }
  
  private boolean isDoFunc() {
    return func.length() > 3 && func.charAt(0) == 'd' && func.charAt(1) == 'o' && (
        func.equals("doif") || func.equals("doelse") || func.equals("doend") || func.equals("doelseif")
        );
    
  }
  
  private int lastIndexOf(char c) {
    for (int i = line.length()-1; i > -1; i--) {
      if (line.charAt(i) == c) return i;
    }
    return -1;
  }
  
  private int lastIndexOf(char c, int pos) {
    for (int i = pos; i > -1; i--) {
      if (line.charAt(i) == c) return i;
    }
    return -1;
  }
  
  private boolean withinQuotes(StringBuffer s, int start) {
    boolean inquote = false;
    for (int i = 0; i < start; i++) {
      if (s.charAt(i) == '\'') inquote = !inquote ;
    }
    return inquote;
  }
  
  private static boolean isNumberResolving(StringBuffer s, int start) {
    for (int i = start; i > -1; i--) {
      char c = s.charAt(i);
      if (c == ' ') continue;
      if ("=,(".indexOf(c) > -1) return true;
      break;
    }
    return false;
  }
  
  private static void checkValidResolving(String s) throws MScriptException {
    s = s.trim();
    int dotc = 0;
    for (int i = 0; i < s.length(); i++) {
      char c = s.charAt(i);
      if (c == '.') dotc++;
      if (c == '-' && i == 0);
      else if (c == '+' && i == 0);
      else if (c == '.' && dotc == 1);
      else if (!Character.isLetterOrDigit(c)) {
        throw new MScriptException("SUSPICIOUS_VALUE_RESOLUTION",new String[]{s},null);
      }
    }
  }
  
  private boolean resolveOneLine(IgnoreStack ist) throws MScriptException {
    
    boolean ignoreFunctions = false;
    
    int i = lastIndexOf('@');
    int dopos = -1;
    
    if (ist != null && i > -1) dopos = doPos();
    
    if (ist != null && ist.peekIgnore()) {
      if (dopos == -1) {
        return false;
      }
      ignoreFunctions = true;
    }
    
    if (i == -1) return true;
    
    StringBuffer s = line;
    
    int k = 0;
    
    while (i > -1) {
      
      if (!ignoreFunctions) {
        
        k = parseFuncAndParms(i);
        
        if (func != null) {
          
          String funcResult = "";
          
          try {
            
            if (func.equals("exit")) {
              s.replace(i, k, "");
              exit = true;
              eos = true;
              return true;
            }
            
            if ( !isDoFunc() ) {
              boolean simple = false;
              String exprStr = null;
              if (sqlInjectionCheck) {
                if (withinQuotes(s,i)) {
                  sr.setDisableQSpecifier(true);
                }
              }
              if (func.equals("vof")) {
                if (EvalUtils.looksLikeSymbol(params)) {
                  simple = true;
                  funcResult = sr.getValue(params);
                } else {
                  exprStr = params;
                }
              } else {
                exprStr = func + "(" + params + ")";
              }
              if (!simple) {
                
                Expression expr = new Expression(exprStr);
                try {
                  sr.bypassRIE = true; 
                  funcResult = expr.eval(sr);
                } finally {
                  sr.bypassRIE = false; 
                }
                
              }
              
              if (sr.getDisableQSpecifier()) {
                s.replace(i, k, sr.doubleTheQuotes(funcResult) );
                sr.setDisableQSpecifier(false);
              } else {
                if (sqlInjectionCheck && isNumberResolving(s,i-1)) {
                  checkValidResolving(funcResult);
                }
                s.replace(i, k, funcResult );
              }
            }
            
          } catch (RemoveInstructionException rie) {
            String msg = rie.getMessage();
            if (msg != null) {
              if (msg.equals("LINE")) return false;
              if (msg.equals("DEFINITION")) removeDefinition(i,k);
            }
            
          } catch (EvaluationException ee) {
            throw new MScriptException("INVALID_MSCRIPT_USAGE",new String[]{line.toString()}, ee);
            
          }
          
        }
      }
      
      int ix = i;
      if (ix > line.length()) ix = line.length();
      
      i = lastIndexOf('@',ix-1);
    }
    
    if (dopos > -1) {
      i = parseFuncAndParms(dopos);
      
      // DOIF
      
      if (func.equals("doif")) {
        int instr = ist.peekInstr();
        //        if (instr != NONE && instr != DOEND ) {
        //          throw new MScriptException("MSCRIPT_ERROR_INVALID_INSTRUCTION_SEQUENCE",new String[]{"@doif()"},null);
        //        }
        if (ist.peekIgnore()) {
          ist.push(true, false, DOIF);
          
          return false;
        }
        Expression expr = new Expression(params, sr);
        
        boolean satisfied;
        try {
          satisfied = expr.evalBool();
        } catch (EvaluationException ex) {
          throw new MScriptException(ex);
        }
        
        ist.push( !satisfied, satisfied, DOIF );
        return false;
      }
      
      // DOELSEIF
      
      else if (func.equals("doelseif")) {
        
        int instr = ist.peekInstr();
        boolean ignore = ist.peekIgnore();
        if (instr != DOIF && instr != DOELSEIF ) {
          throw new MScriptException("MSCRIPT_ERROR_INVALID_INSTRUCTION_SEQUENCE",new String[]{"@doelseif()"},null);
        }
        
        boolean satisfied = ist.peekSatisfied();
        ist.pop();
        
        if (ist.peekIgnore()) {
          ist.push(true, false, DOELSEIF);
        } else {
          if (satisfied) {
            ist.push( true, satisfied, DOELSEIF );
          } else {
            Expression expr = new Expression( params, sr);
            try {
              satisfied = expr.evalBool();
            } catch (EvaluationException ex) {
              throw new MScriptException(ex);
            }
            ist.push( !satisfied, satisfied, DOELSEIF );
          }
        }
        return false;
      }
      
      // DOELSE
      
      else if (func.equals("doelse")) {
        
        boolean ignore = ist.peekIgnore();
        int instr = ist.peekInstr();
        
        if (instr != DOIF && instr != DOELSEIF ) {
          throw new MScriptException("MSCRIPT_ERROR_INVALID_INSTRUCTION_SEQUENCE",new String[]{"@doelse()"},null);
        }
        
        boolean satisfied = ist.peekSatisfied();
        ist.pop();
        
        if (ist.peekIgnore()) {
          ist.push(true, false, DOELSE);
        } else {
          ist.push(satisfied, satisfied, DOELSE);
        }
        return false;
      }
      
      // DOEND
      
      else if (func.equals("doend")) {
        int instr = ist.peekInstr();
        if (instr != DOIF && instr != DOELSEIF && instr != DOELSE ) {
          throw new MScriptException("MSCRIPT_ERROR_INVALID_INSTRUCTION_SEQUENCE",new String[]{"@doend()"},null);
        }
        ist.pop();
        return false;
        
      }
      
    }
    
    return true;
  }
  
  
  private void removeDefinition(int spos, int epos) {
    int i = spos;
    while (i > -1 && line.charAt(i) != ' ') i--;
    if (i == -1) i++;
    
    int j = epos;
    while (j < line.length() && line.charAt(j) != ' ') j++;
    
    line.delete(i,j);
  }
  
  
  
  public static boolean hasFunctions(String str) {
    if (str == null || str.length() == 0) return false;
    return str.indexOf('@') > -1;
  }
  
  public static void main(String[] args) throws Exception {
    String s = "ID";
    if (s.endsWith("ID")) {
      System.out.println("ooooooooooooooo");
    }
    
  }
  
}

class IgnoreStack {
  
  private boolean[] ignoreArray;
  private boolean[] satisfiedArray;
  private int[] instrArray;
  private int index = -1;
  
  protected IgnoreStack(int size) {
    ignoreArray = new boolean[size];
    satisfiedArray = new boolean[size];
    instrArray = new int[size];
    push(false,false,0);
  }
  
  protected int getIndent() {
    if (index < 0) return 0;
    return index * 2;
  }
  
  protected void push(boolean ignore, boolean satisfied, int instr) {
    index++;
    ignoreArray[index] = ignore;
    satisfiedArray[index] = satisfied;
    instrArray[index] = instr;
  }
  
  protected void pop() {
    if (index > 0) {
      index--;
    }
  }
  
  protected boolean peekIgnore() {
    return ignoreArray[index];
  }
  
  protected boolean peekSatisfied() {
    return satisfiedArray[index];
  }
  
  protected int peekInstr() {
    return instrArray[index];
  }
  
  protected void clear() {
    index = -1;
  }
  
}