/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Node.java
 *
 * Created on 14 January 2006 Saturday, 03:10
 */

package org.moremotion.evaluator;

import org.moremotion.evaluator.Evaluable;
import org.moremotion.evaluator.RemoveInstructionException;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Node.java 63 2008-03-14 16:29:03Z erkan $
 */

public class Node implements Evaluable {
  
  private int operator = Operator.NULL;
  private String str;
  private boolean parsed = false;
  private Object value = null;
  
  /** Creates a new instance of Node */
  public Node(String str) {
    this.str = str.trim(); 
  }
  
  public Node(Function func) {
    this.value = func;
    parsed = true;
  }
  
  public boolean isNC() {
    
    if (value == null) return false;
    
    if (value instanceof Double) return true;
    if (value instanceof Symbol && ((Symbol) value).getDataType() == 'n') return true;
    if (value instanceof Function && ((Function) value).getDataType() == 'n') return true;
    
    return false;
  }  
  
  public Object evaluate(SymbolResolver sr) throws EvaluationException {
    parse();
    if (value instanceof RemoveInstructionException) throw (RemoveInstructionException)value;
    if (value instanceof Evaluable) return ((Evaluable) value).evaluate(sr);
    return value;
  }
  
  private void parse() throws ParsingException {
    if (parsed) return;
    parsed = true;

    if (str.length() == 0) {
      throw new ParsingException("MISSING_SYMBOL_OR_CONSTANT");
    }
    
    if (str.equals("\\rd")) {
      value = new RemoveInstructionException("DEFINITION");
      return;
    }
    
    if (str.equals("\\rl")) {
      value = new RemoveInstructionException("LINE");
      return;
    }
    
    if (str.equals("true")) {
      value = new Boolean(true);
      return;
    }
    
    if (str.equals("false")) {
      value = new Boolean(false);
      return;
    }
    
    if (EvalUtils.isNumber(str)) {
      try {
        value = new Double(str);
        return;
      } catch(Exception e) {}
    }
    
    if (str.charAt(0) == '[') {
      value = new Array( str.substring(1,str.length()-1) );
      return;
    }

    if (str.charAt(0) == '(') {
      value = new Expression( str.substring(1,str.length()-1) );
      return;
    }

    if (str.charAt(0) != '\'' && str.indexOf('(') > -1) {
      value = new Function(str);
      return;
    }
    
    if (EvalUtils.looksLikeSymbol(str)) {
      Symbol sym = new Symbol(str);
      if (sym.getError() != null) {
        throw new ParsingException(sym.getError(),new String[]{str},null);
      }
      value = sym;
      return;
    }
    
    value = new String( getStringValue());
  }
  
  public String toString() {
    
    try {
      if (!parsed) parse();
    } catch(Exception e) {
      e.printStackTrace();
    }
    
    if (value instanceof Evaluable) {
      return value.toString();
    }
    
    if (value instanceof String) {
      return "'" + value.toString().replaceAll("'","''") + "'";
    }
    
    if (value instanceof Double) {
      return EvalUtils.toStr(value);
    }
    
    if (value == null) {
      return this.str;
    }
    
    return value.toString();
  }
  
  public void setOperator(int value) {
    this.operator = value;
  }
  
  public int getOperator() {
    return this.operator;
  }
  
  public String getStringValue() throws InvalidEntityException {
    return EvalUtils.getValueOfStringConstant(this.str,'\'');
  }
  
  public static void main(String[] args) {
  }
  
}
