/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * OPNode.java
 *
 * Created on 07 February 2006 Tuesday, 17:32
 */

package org.moremotion.evaluator;

import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: OPNode.java 137 2008-05-11 17:24:45Z erkan $
 */

public class OPNode {
  
  private int opr;
  private OPNode n1 = null;
  private OPNode n2 = null;
  private Node n = null;
  
  public OPNode(Node node) {
    this.n = node;
  }
  
  public boolean isNC() {
    return n != null && n.isNC();
  }
  
  public String toString() {
    if (n != null) return n.toString();
    if (n2 != null){
      return n1.toString() + Operator.getOperatorAsString(opr) + n2.toString();
    }
    return n1.toString();
  }
  
  public boolean isThenElse() throws EvaluationException {
    return opr == Operator.COLUMN;
  }
  
  public OPNode(Node[] nodes, int fi, int ti) {
    
    int len = ti - fi;
    if ( len == 1 ) {
      n = nodes[fi];
      return;
    }
    
    for (int i = Operator.MAXLEVEL; i >= 0; i--) {
      for (int j = fi; j < ti-1;  j++) {
        opr = nodes[j].getOperator();
        if (i == Operator.getOperatorLevel(opr)) {
          n1 = new OPNode(nodes,fi,j+1);
          n2 = new OPNode(nodes,j+1,ti);
          return;
        }
      }
    }
  }
  
  
  private Double math(Object o1, Object o2, int opr) {
    
    Double d1 = EvalUtils.doubleValue(o1);
    Double d2 = EvalUtils.doubleValue(o2);
    
    switch (opr) {
      case Operator.ADD: return new Double(d1.doubleValue() + d2.doubleValue());
      case Operator.SUBTRACT: return new Double(d1.doubleValue() - d2.doubleValue());
      case Operator.MULTIPLY: return new Double(d1.doubleValue() * d2.doubleValue());
      case Operator.DIVIDE: return new Double(d1.doubleValue() / d2.doubleValue());
    }
    return null;
    
  }

  /*
  private String toStr(Object o) {
    if (o instanceof Double) {
      Double d = (Double)o;
      if (d.longValue() == d.doubleValue()) {
        return d.longValue() + "";
      }
      return d.toString();
      
    }
    return o.toString();
  }
   *
   **/
  
  private Boolean compare(Object o1, Object o2, int opr) throws EvaluationException {
    
    if( o1 instanceof Boolean && o2 instanceof Boolean) {
      boolean b1 = ((Boolean)o1).booleanValue();
      boolean b2 = ((Boolean)o2).booleanValue();
      switch (opr) {
        case Operator.EQ: return new Boolean(b1 == b2);
        case Operator.NE: return new Boolean(b1 != b2);
        case Operator.GT: return new Boolean(false);
        case Operator.GE: return new Boolean(true);
        case Operator.LT: return new Boolean(false);
        case Operator.LE: return new Boolean(true);
      }
    }
    
    if( o1 instanceof Double && o2 instanceof Double) {
      double d1 = ((Double)o1).doubleValue();
      double d2 = ((Double)o2).doubleValue();
      switch (opr) {
        case Operator.EQ: return new Boolean(d1 == d2);
        case Operator.NE: return new Boolean(d1 != d2);
        case Operator.GT: return new Boolean(d1 > d2);
        case Operator.GE: return new Boolean(d1 >= d2);
        case Operator.LT: return new Boolean(d1 < d2);
        case Operator.LE: return new Boolean(d1 <= d2);
      }
      
    }
    
    if (n1.isNC() || n2.isNC() ) {
      /* 
      if (o1 == null || o1.toString().length() == 0 || o2 == null || o2.toString().length() == 0) {
        return new Boolean(false);
      } */
        double d1;
        double d2;
        try {
          d1 = (new Double(o1.toString())).doubleValue();
        } catch (Exception ex) {
          throw new EvaluationException("INVALID_OPERAND_FOR_NUMERIC_COMPARISON",new String[]{n1.toString(), o1.toString()},null);
        }
        try {
          d2 = (new Double(o2.toString())).doubleValue();
        } catch (Exception ex) {
          throw new EvaluationException("INVALID_OPERAND_FOR_NUMERIC_COMPARISON",new String[]{n2.toString(), o2.toString()},null);
        }
        
        switch (opr) {
          case Operator.EQ: return new Boolean(d1 == d2);
          case Operator.NE: return new Boolean(d1 != d2);
          case Operator.GT: return new Boolean(d1 > d2);
          case Operator.GE: return new Boolean(d1 >= d2);
          case Operator.LT: return new Boolean(d1 < d2);
          case Operator.LE: return new Boolean(d1 <= d2);
        }
    }
    
    
    String s1 = EvalUtils.toStr(o1);
    String s2 = EvalUtils.toStr(o2);
    
    switch (opr) {
      case Operator.EQ: return new Boolean(s1.equals(s2));
      case Operator.NE: return new Boolean(!s1.equals(s2));
      case Operator.GT: return new Boolean(s1.compareTo(s2) > 0);
      case Operator.GE: return new Boolean(s1.compareTo(s2) >= 0);
      case Operator.LT: return new Boolean(s1.compareTo(s2) < 0);
      case Operator.LE: return new Boolean(s1.compareTo(s2) <= 0);
    }
    
    return null;
  }
  
  private Boolean specialCompare(Object o1, Object o2, int opr) throws EvaluationException {
    String s1 = EvalUtils.toStr(o1);
    String s2 = EvalUtils.toStr(o2);
    
    switch (opr) {
      case Operator.MATCH: return new Boolean( match(s1,s2)  );
      case Operator.WMATCH: return new Boolean( wmatch(s1,s2)  );
      case Operator.LIKE: return new Boolean( isLike(s1,s2)  );
      case Operator.INCLUDE: return new Boolean( include(s1,s2)  );
    }
    return null;
    
  }
  
  public Object evaluate(SymbolResolver sr) throws EvaluationException {
    
    if (n != null) return n.evaluate(sr);
    
    Object o1 = null;
    Object o2 = null;
    
    if (opr != Operator.COLUMN) {
      o1 = n1.evaluate(sr);
      if (opr != Operator.OR && opr != Operator.AND && opr != Operator.QMARK && opr != Operator.SEMICOL) o2 = n2.evaluate(sr);
    }
    
    //    Object o2 = n2.evaluate();
    
    switch(opr) {
      
      case Operator.SEMICOL:
        if (o1 instanceof String && ((String)o1).length() == 0) {
          return n2.evaluate(sr);
        } else {
          return o1;
        }
        
      case Operator.SUBTRACT:
      case Operator.MULTIPLY:
      case Operator.DIVIDE:
        return math(o1,o2,opr);
        
      case Operator.ADD:
        if ( n1.isNC() || n2.isNC() || (o1 instanceof Double && o2 instanceof Double) ) {
          return math(o1,o2,opr);
        }
        return EvalUtils.toStr(o1) + EvalUtils.toStr(o2);
        
      case Operator.EQ:
      case Operator.NE:
      case Operator.GT:
      case Operator.GE:
      case Operator.LT:
      case Operator.LE:
        return compare(o1,o2,opr);
        
      case Operator.LIKE:
      case Operator.MATCH:
      case Operator.WMATCH:
      case Operator.INCLUDE:
        return specialCompare(o1,o2,opr);
        
      case Operator.OR: {
        if (getBoolean(o1)) return new Boolean(true);
        o2 = n2.evaluate(sr);
        return new Boolean(getBoolean(o2));
      }
      
      case Operator.AND: {
        if (!getBoolean(o1)) return new Boolean(false);
        o2 = n2.evaluate(sr);
        return new Boolean( getBoolean(o2) );
      }
      
      case Operator.COLUMN: {
        return new ThenElse(n1,n2);
      }
      
      case Operator.QMARK: {
        boolean b = getBoolean(o1);
        
        if (n2.isThenElse()) {
          o2 = n2.evaluate(sr);
          ThenElse te = ((ThenElse)o2);
          return te.eval(sr,b);
        } else {
          if (b) {
            o2 = n2.evaluate(sr);
            return o2;
          } else {
            return new String("");
          }
        }
        
      }
    }
    
    return null;
    
  }
  
  private static boolean getBoolean(Object o) {
    
    if (o instanceof Boolean) {
      return ((Boolean)o).booleanValue();  
    }
    return EvalUtils.getBoolean(o.toString());

  }  
  
  private static boolean isLike(String inStr, String sstr) {
    
    int i = sstr.indexOf('%');
    if (i == -1 || sstr.length() < 2 || (i != 0 && i != sstr.length()-1) ) {
      return inStr.equalsIgnoreCase(sstr);
    }
    int ln = sstr.length();
    if (sstr.charAt(ln-1) == '%') {
      if (sstr.charAt(0) == '%') {
        // "%ABC%"
        String s1 = inStr.toUpperCase();
        String s2 = sstr.substring(1,ln-1).toUpperCase();
        return s1.indexOf(s2) > -1;
      } else {
        // "ABC%"
        return inStr.substring(0,ln-1).equalsIgnoreCase(sstr.substring(0,ln-1));
      }
    } else {
      // "%ABC"
      return inStr.substring(inStr.length()-ln+1).equalsIgnoreCase(sstr.substring(1));
    }
  }
  
  private static boolean include(String s, String s1) {
    return Keys.keys(s1).includedIn(s);
  }
  
  private static boolean match(String s, String s1) {
    Pattern p = Pattern.compile(s1);
    Matcher m = p.matcher(s);
    return m.matches();
  }
  
  private static boolean wmatch(String s, String s1) {
    WildCard wc = new WildCard(s1);
    return wc.match(s);
  }
  
  
  
  
}
