/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Operator.java
 *
 * Created on 14 January 2006 Saturday, 03:12
 */

package org.moremotion.evaluator;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Operator.java 51 2008-03-10 11:46:54Z erkan $
 */

public class Operator {
  
  public static final int NULL     = 0;
 
  public static final int MULTIPLY = 1;
  public static final int DIVIDE   = 2;
  
  public static final int ADD      = 11;
  public static final int SUBTRACT = 12;
  
  public static final int EQ       = 21;
  public static final int NE       = 22;
  public static final int GT       = 23;
  public static final int GE       = 24;
  public static final int LT       = 25;
  public static final int LE       = 26;
  public static final int MATCH    = 27;
  public static final int WMATCH   = 28;
  public static final int LIKE     = 29;
  public static final int INCLUDE  = 30;

  public static final int AND      = 41;
  public static final int OR       = 42;

  public static final int QMARK    = 51;
  public static final int SEMICOL  = 52;
  public static final int COLUMN   = 53;
  
  public static final int MAXLEVEL  = 8;
  
  public static int getOperatorLevel(int opr) {
    
    switch(opr) {
      
      case SEMICOL:
        return 1;
      
      case MULTIPLY: 
      case DIVIDE:
        return 2;
      
      case ADD:
      case SUBTRACT:
        return 3;
      
      case AND:
        return 5;
        
      case OR:
        return 6;
        
      case COLUMN:
        return 7;
        
      case QMARK:
        return 8;
        
      case NULL:
        return 0;
    }
    
    return 4;
    
  }
  
  public static int getOperatorAsInt(String opr) throws InvalidEntityException {
    
    if (opr.length() == 1) {
      switch (opr.charAt(0)) {
        case '=': return EQ;
        case '+': return ADD;
        case '-': return SUBTRACT;
        case '*': return MULTIPLY;
        case '/': return DIVIDE;
        case '>': return GT;
        case '<': return LT;
        case '?': return QMARK;
        case ';': return SEMICOL;
        case ':': return COLUMN;
      }
    }
    else if (opr.equals("&&")) return AND;
    else if (opr.equals("||")) return OR;
    else if (opr.equals("and")) return AND;
    else if (opr.equals("or")) return OR;
    else if (opr.equals(">=")) return GE;
    else if (opr.equals("<=")) return LE;
    else if (opr.equals("!=")) return NE;
    else if (opr.equals("<>")) return NE;
    else if (opr.equals("==")) return EQ;
    else if (opr.equals("like")) return LIKE;
    else if (opr.equals("include")) return INCLUDE;
    else if (opr.equals("match")) return MATCH;
    else if (opr.equals("wmatch")) return WMATCH;
    
    throw new InvalidEntityException("Invalid operator: '" + opr + "'");
    
  }
  
  public static String getOperatorAsString(int opr) {
    switch (opr) {
      case NULL: return "";
      case AND: return "&&";
      case OR: return "||";
      case EQ: return "=";
      case NE: return "!=";
      case GT: return ">";
      case GE: return ">=";
      case LT: return "<";
      case ADD: return "+";
      case SUBTRACT: return "-";
      case MULTIPLY: return "*";
      case DIVIDE: return "/";
      case MATCH: return "match";
      case WMATCH: return "wmatch";
      case LIKE: return "like";
      case INCLUDE: return "include";
      case QMARK: return "?";
      case SEMICOL: return ";";
      case COLUMN: return ":";
    }
    return "";
  }
  
}
