/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Symbol.java
 *
 * Created on 07 February 2006 Tuesday, 16:55
 */
package org.moremotion.evaluator;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Symbol.java 146 2008-06-04 21:23:59Z erkan $
 */
public class Symbol implements Evaluable {

  private String str;
  private String pfx;
  private String name;
  private char sts = ' ';
  private char dt = ' ';
  private int depth = -1;
  private boolean isAdomName = false;
  private String error = null;
  
  public static String SYMBOL_VALID_FIRST_CHARS = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM_@/";
  public static String SYMBOL_VALID_BODY_CHARS = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM_@0123456789/.";
  
  public static final String MISUSED_DEPTH_SPECIFIER = "MSCRIPT_ERROR_MISUSED_DEPTH_SPECIFIER";
  public static final String INVALID_DATA_TYPE_SPECIFIER = "MSCRIPT_ERROR_INVALID_DATA_TYPE_SPECIFIER";
  public static final String INVALID_SOURCE_TARGET_SPECIFIER = "MSCRIPT_ERROR_INVALID_SOURCE_TARGET_SPECIFIER";
  public static final String INVALID_CHARACTER = "MSCRIPT_ERROR_INVALID_CHARACTER";
  public static final String INVALID_SOURCE_SPECIFIER = "MSCRIPT_ERROR_INVALID_SOURCE_SPECIFIER";
  public static final String INVALID_TARGET_SPECIFIER = "MSCRIPT_ERROR_INVALID_TARGET_SPECIFIER";
  
  public static final String MISSING_NAME = "MSCRIPT_ERROR_MISSING_SYMBOL_NAME";
  public static final String INVALID_SYMBOL_NAME = "MSCRIPT_ERROR_INVALID_SYMBOL_NAME";

  public Symbol(String str) {

    boolean dotFound = false;
    boolean charFound = false;

    this.str = str.replace('^', '_');
    int x = this.str.indexOf(':');
    if (x == -1) {
      this.name = str;
      this.pfx = "";
    } else {
      pfx = str.substring(0, x);
      this.name = str.substring(x + 1);
    }

    for (int i = 0; i < pfx.length(); i++) {
      char c = pfx.charAt(i);

      if (c == '.') {
        if (charFound) {
          error = MISUSED_DEPTH_SPECIFIER;
        }
        depth++;
        dotFound = true;
        
      } else {
        charFound = true;

        if (c == 'n' || c == 'b' || c == 'q') {
          if (dt != ' ') {
            error = INVALID_DATA_TYPE_SPECIFIER;
          }
          if (dotFound && sts == ' ') {
            error = MISUSED_DEPTH_SPECIFIER;
          } 
          dt = c;

        } else if (c == 'i' || c == 'v' || c == 'a' || c == 'p' || c == 's' || c == 'f' || c == 'c' || c == 'r') {
          if (sts == ' ') {
            sts = c;
          } else {
            error = INVALID_SOURCE_TARGET_SPECIFIER;
          }

        } else {
          error = INVALID_CHARACTER;
        }
      }
    }
    
    if (depth > -1 && sts != 'f' && sts != 'c') {
      error = MISUSED_DEPTH_SPECIFIER;
    }
    
    if (name.length() == 0) {
      error = MISSING_NAME;
      return;
    }
    
    char c = name.charAt(0);
    
    if (name.length() == 1) {
      
      if (c == '.') {
        if (sts != ' ' || depth > -1) {
          error = MISSING_NAME;
        }
        return;
      }
      
      if (SYMBOL_VALID_FIRST_CHARS.indexOf(c) == -1 || c == '/') {
        error = INVALID_SYMBOL_NAME;
        return;
      }
      
      return;
    }
    
    if (SYMBOL_VALID_FIRST_CHARS.indexOf(c) == -1) {
      error = INVALID_SYMBOL_NAME;
      return;
    }
    
    if (c == '/') isAdomName = true;
    
    for (int i = 1; i < name.length(); i++) {
      c = name.charAt(i);
      if (SYMBOL_VALID_BODY_CHARS.indexOf(c) == -1) {
        error = INVALID_SYMBOL_NAME;
        return;
      }
      if (c == '/' || c == '.') {
        char pc = name.charAt(i-1);  
        if (pc == '/' || pc == '.' || i == name.length()-1) {
          error = INVALID_SYMBOL_NAME;
          return;
        }
        isAdomName = true;
      }
    }
    
    if (isAdomName && " sra".indexOf(sts) == -1) {
      error = INVALID_SYMBOL_NAME;
    }

  }
  
  public boolean isAdomName() {
    return isAdomName;
  }
  
  public String getError() {
    if (error == null) return null;
    else {
      if (error.equals(INVALID_SOURCE_TARGET_SPECIFIER)) return INVALID_SOURCE_SPECIFIER;
      return error;
    }
  }
  
  public String getErrorForTargetSymbol() {
    if (error != null) {
      if (error.equals(INVALID_SOURCE_TARGET_SPECIFIER)) return INVALID_TARGET_SPECIFIER;
      return error;
    }
    if (sts == 'i' || sts == 'c') return INVALID_TARGET_SPECIFIER;
    if (depth > -1 && sts != 'f') return MISUSED_DEPTH_SPECIFIER;
    return null;
  }
  
  public Object evaluate(SymbolResolver sr) throws EvaluationException {
    int type = sr.getType(str);
    String result = sr.getValue(str);
    if (type == SymbolResolver.DT_BOOL) {
      return new Boolean(result);
    }
    if (type == SymbolResolver.DT_NUMBER) {
      return EvalUtils.doubleValue(result);
    }
    return new String(result);
  }

  public String getName() {
    return this.name;
  }

  public char getSTS() {
    return this.sts;
  }

  public char getDataType() {
    return this.dt;
  }
  
  public int getDepth() {
    return depth;  
  }

  public String toString() {
    return this.str;
  }
  
  public static void main(String[] args) throws Exception{
    Symbol sym = new Symbol(".");
    System.out.println(sym.getError());
  }
  
}
