/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Vars.java
 *
 * Created on 14 June 2006 Wednesday, 23:05
 */

package org.moremotion.evaluator;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import javax.servlet.http.HttpSession;

import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Vars.java 225 2008-09-30 20:56:35Z erkan $
 */

public class Vars {
  
  PageTable ptable = null;
  // MoreMotionRequest request = null;  
  String currPage = null;
  PageVars pvars = null;
  PageVars svars = null;
  
  public Vars(MoreMotionRequest request) {
    HttpSession ses = request.getSession();
    ptable = (PageTable)ses.getAttribute("moremotion.page.table");
    if (ptable == null) {
      ptable = new PageTable();
      ses.setAttribute("moremotion.page.table", ptable);
    }
    
    String cp = request.getPlainParameter("_pg",null); // ProcessForm iindeki hidden alandan geliyor
    if (cp != null) cp = cp.replace('/','_').replace('\\','_');
    
    pvars = ptable.getPageVars(cp);
    svars = ptable.getSessionVars();
  }

  public void adjustToPage(String pageName) {
    currPage = pageName.replace('/','_').replace('\\','_');
    this.pvars = ptable.getPageVars(currPage);
    ptable.setLastPage(currPage);
  }
  
  public String getPageVar(String name) {
   return pvars.get(name);  
  }
  
  public void setPageVar(String name, String value) { 
    if (value == null) {
      pvars.remove(name);  
    } else {
      pvars.put(name,value);  
    }
  }
  
  public void deletePageVar(String name) {
    if (name.indexOf(',') == -1) {
      pvars.remove(name);
      return;
    }
    
    String[] vars = name.split(","); 
    for (int i = 0; i < vars.length; i++) {
      String var = vars[i].trim();
      pvars.remove(var);
    }
  }
  
  public Properties getPageVars() {
    return pvars.getProperties();
  }
  
  public void removeAllPageVars() {
    pvars.clear();
  }
  
  public String getSessionVar(String name) {
   return svars.get(name);  
  }
  
  public void setSessionVar(String name, String value) {
    if (value == null) {
      svars.remove(name);  
    } else {
      svars.put(name,value);  
    }
  }
  
  public void deleteSessionVar(String name) {
    if (name.indexOf(',') == -1) {
      svars.remove(name);
      return;
    }
    
    String[] vars = name.split(","); 
    for (int i = 0; i < vars.length; i++) {
      String var = vars[i].trim();
      svars.remove(var);
    }
  }
  
  public Properties getSessionVars() {
    return svars.getProperties();
  }

  public void removeAllSessionVars() {
    svars.clear();
  }
  
  private class PageTable {
    Hashtable pages = null;
    String lastPage = null;
    
    PageTable() {
      pages = new Hashtable();
    }
    
    void clear() {
      pages.clear();
    }
    
    void setLastPage(String pageName) {
      lastPage = pageName;
    }
    
    String getLastPage() {
      return lastPage;
    }
    
    PageVars getPageVars(String pageName) {
      if (pageName == null) pageName = lastPage;
      if (pageName == null) pageName = "*";
      PageVars pvars = (PageVars)pages.get(pageName);
      if (pvars == null) {
        pvars = new PageVars();
        pages.put(pageName, pvars);
      }
      return pvars;
    }
    
    PageVars getSessionVars() {
      PageVars pvars = (PageVars)pages.get("*session");
      if (pvars == null) {
        pvars = new PageVars();
        pages.put("*session", pvars);
      }
      return pvars;
    }
    
  }
  
  
  private class PageVars {
    Properties props = null;
    
    PageVars() {
    }
    
    Properties getProperties() {
      return this.props;
    }
    
    void clear() {
      props = null;
    }
    
    void put(String key, String value) {
      if (props == null) props = new Properties();
      props.setProperty(key, value);
    }
    
    String get(String key) {
      if (props == null) return null;
//      if (key == null) return null;
      return props.getProperty(key);
    }
    
//    String get(String key, String defaultValue) {
//      if (key == null) return defaultValue;
//      return props.getProperty(key, defaultValue);
//    }
    
    void remove(String key) {
      
      if (props == null || key == null || key.length() == 0) return;
      
      if (key.charAt(key.length()-1) == '*') {
        ArrayList list = new ArrayList();
        key = key.substring(0,key.length()-1); 
        Enumeration e = props.keys();
        while (e.hasMoreElements()) {
          String name = (String)e.nextElement();
          if (name.startsWith(key)) {
            list.add(name);
          }
        }
        for (int i = 0; i < list.size(); i++) {
          String name = (String)list.get(i);
          props.remove(name);
        }
        
      } else {
        props.remove(key);
      }
      
    }
    
    
  }
  
  
}


