/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * PageUtils.java
 *
 */

package org.moremotion.page;

import java.text.DecimalFormatSymbols;
import java.util.Locale;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.servlet.MoreMotionRequest;

/** 
 * @author Yusuf
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: PageUtils.java 51 2008-03-10 11:46:54Z erkan $
 */
public class PageUtils {
  
  public static boolean Debug = false;
 
  public static String getTillSemicolumn(String str, int position) {
    for (int i = position; i < str.length(); i++) {
      if (i-position > 6) break;
      if (str.charAt(i) == ';') {
        return str.substring(position,i);
      }
    }
    return null;
  }
  
  /** Encodes characters <CODE>&amp;, &lt;, &quot;</CODE> to XML entities
   * <CODE>&amp;amp;, &amp;lt;, &amp;quot;</CODE> in the given string.
   */
  public static String encodeXMLAttributeValue(String source) { 
    
    if (source == null || source.length() == 0) {
      return source;
    }
    
    int i;
    StringBuffer buffer = null;
    for(i = 0; i < source.length(); i++) {
      char c = source.charAt(i);
      if (c == '&' || c == '<' || c == '"') {
        buffer = new StringBuffer( source.length() + 10 );
        buffer.append(source.substring(0,i));
        break;
      }
    }
    
    if (buffer == null) return source;
    
    for(; i < source.length(); i++) {
      
      char c = source.charAt(i);
      
      switch(c) {
        
        case '&':
          
          buffer.append("&amp;");
          break;
          
        case '<':
          buffer.append("&lt;");
          break;
          
        case '"':
          buffer.append("&quot;");
          break;
          
        default:
          buffer.append(c);
      }
    }
    return buffer.toString();
  }
  
  /** Encodes characters <CODE>&amp;, &gt;, &lt; </CODE> to XML entities
   * <CODE>&amp;amp;, &amp;gt;, &amp;lt;</CODE> in the given string.
   */
  public static String encodeXMLElementValue(String source) { 
    
    if (source == null || source.length() == 0) {
      return source;
    }
    
    if (source.startsWith("<![CDATA[")) {
      return source;
    }
    
    if (source.startsWith("<!-- #NO_ESCAPE -->")) {
      return source.substring(19);
    }
    
    int i;
    StringBuffer buffer = null;
    for(i = 0; i < source.length(); i++) {
      char c = source.charAt(i);
      if (c == '&' || c == '<' || c == '>') {
        buffer = new StringBuffer( source.length() + 10 );
        buffer.append(source.substring(0,i));
        break;
      }
    }
    
    if (buffer == null) return source;
    
    for(; i < source.length(); i++) {
      
      char c = source.charAt(i);
      
      switch (c) {   
        case '&':
          buffer.append("&amp;");
          break;

        case '>':
          buffer.append("&gt;");
          break;
          
        case '<':
          buffer.append("&lt;");
          break;
          
        default:
          buffer.append(c);
      }
    }
    return buffer.toString();
  }
  
  /** Decodes XML entities <CODE>&amp;amp;, &amp;apos;, &amp;gt;, &amp;lt;, &amp;quot;</CODE>
   * to <CODE>&amp;, ', &gt;, &lt;, &quot;</CODE> in the given string.
   */
  public static String decodeXMLChars(String source) {
    
    if (source == null || source.length() == 0) {
      return source;
    }
    
    StringBuffer buffer = new StringBuffer();
    for(int i = 0; i < source.length(); i++) {
      char c = source.charAt(i);
      if (c == '&') {
        String s = getTillSemicolumn(source,i+1);
        if (s != null) {
          if ( s.equals("amp") ) {
            buffer.append('&');
            i += 4;
          }
          else if ( s.equals("apos") ) {
            buffer.append('\'');
            i += 5;
          }
          else if ( s.equals("gt") ) {
            buffer.append('>');
            i += 3;
          }
          else if ( s.equals("lt") ) {
            buffer.append('<');
            i += 3;
          }
          else if ( s.equals("quot") ) {
            buffer.append('"');
            i += 5;
          }
          else {
            buffer.append(c);
          }
        }
        else {
          buffer.append(c);
        }
      } else {
        buffer.append(c);
      }
    }
    return buffer.toString();
  }
  
  public static String getPageInfoXml(MoreMotionRequest request) {
    StringBuffer buf = new StringBuffer();
    buf.append("<pageinfo>");
    String lang = request.getLanguage();
    Locale locale = new Locale(lang);
    DecimalFormatSymbols dfs = new DecimalFormatSymbols(locale);
    buf.append("name:" + request.getCurrentPage().getPageName());
    buf.append(";lang:" + lang);
    String encoding = "iso-8859-1";
    try {
      encoding = request.getCurrentPage().getEncoding();
    } catch (ConfigParameterException ex) {
      ex.printStackTrace();
    }
    buf.append(";charset:" + encoding);
    buf.append(";GS:" + dfs.getGroupingSeparator());
    buf.append(";DS:" + dfs.getDecimalSeparator());
    buf.append("</pageinfo>");
    return buf.toString();
  }
  
  
}