/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * BlockQueryProcess.java
 *
 * Created on 28 July 2007 Saturday, 18:42
 *
 */

package org.moremotion.process;

import org.moremotion.config.ConfigParameterException;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: BlockQueryProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
public class BlockQueryProcess extends QueryProcess {
  
  private FormProcessBlock formBlock;
  private ProcessRecord prec;
  private int recordIndex;
  
  /** Creates a new instance of BlockQueryProcess */
  public BlockQueryProcess() {
  }

  public String[] getFieldNames() throws ProcessException {
    return formBlock.getFieldNames();
  }

  public MemoryProcessRecord getNextResultRecord() throws ProcessException {
    while (this.recordIndex < this.formBlock.getRecordCount()) {
      String recNumStr = this.formBlock.getRecordStatus(this.recordIndex).getParentRecNum();
      if (recNumStr != null) {
        try {
          if (Integer.parseInt(recNumStr) == this.prec.getIndex()) {
            MemoryProcessRecord record = getRecord();
            this.recordIndex++;
            return record;
          }
        } catch (Exception e) {}
      }
      this.recordIndex++;
    }
    return null;
  }
  
  private MemoryProcessRecord getRecord() {
    String[] fieldNames = formBlock.getFieldNames();
    MemoryProcessRecord result = new MemoryProcessRecord();
    result.fields = new String[fieldNames.length];
    for (int i = 0; i < fieldNames.length; i++) {
      result.fields[i] = this.formBlock.getField(fieldNames[i],this.recordIndex);
    }
    result.status = this.formBlock.getRecordStatus(this.recordIndex);
    return result;
  }

  public void prepare() throws ProcessException {
    try {
      ProcessConfig config = callDef.getProcessConfig();
      String blockName = config.getParameter("blockName").stringValue();
      ProcessBlock block = ProcessBlockPool.getPool(request).getBlock(blockName);
      if (block instanceof FormProcessBlock) {
        formBlock = (FormProcessBlock) block;
      }
      
    } catch (ProcessManagingException ex) {
      throw new ProcessException(ex);
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
  }

  public void processRecord(ProcessRecord prec) throws ProcessException {
    this.prec = prec;
    this.recordIndex = 0;
  }

  public void finalizeProcess() throws ProcessException {
  }
  
}
