/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ProcessManager.java
 *
 * Created on March 19, 2002, 11:14 AM
 */

package org.moremotion.process;

import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.moremotion.cache.VariablePool;
import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConditionalListItem;

import org.moremotion.evaluator.MScriptException;
import org.moremotion.page.CurrentPage;
import org.moremotion.page.PageGenerationException;
import org.moremotion.page.TemplateCall;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.page.UserAgentAnalyzer;

/**
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ProcessManager.java 186 2008-07-31 18:23:46Z erkan $
 */
public class ProcessManager {
  
  public static void doService(MoreMotionRequest request) throws ServiceException, IOException {
    
    try {
      
      String cmdName = request.getParameter("_cmd"); // The process command
      String scope = request.getParameter("_scope"); // The command scope, usually the name of the process form.
      String originPage = request.getOriginPage();
      
      request.adjustVarsToPage(originPage);
      
      if (cmdName == null || cmdName.length() == 0) {
        sendHistoryBack(request);
        return;
      }
      
      String fullCommandName = cmdName;
      if (scope != null && scope.trim().length() > 0) {
        fullCommandName = scope + "." + fullCommandName;
      }
      if (originPage != null && originPage.trim().length() > 0) {
        fullCommandName = originPage + "." + fullCommandName;
      }
      
      ProcessCommand cmd = ProcessCommandManager.getProcessCommand(fullCommandName);
      if (cmd == null) {
        throw new ServiceException("PROCESS_COMMAND_MISSING",new String[]{cmdName, scope, originPage},null);
      }
      
      String method = cmd.getMethod(request);
      if (method != null && method.equalsIgnoreCase("POST") && !request.getMethod().equalsIgnoreCase("POST")) {
        throw new ServiceException("METHOD_MUST_BE_POST",null,null);
      }
      
      if (originPage == null) {
        originPage = "index";
      }
      
      String nextPage = cmd.getNextPage();
      if (nextPage == null) {
        nextPage = originPage;
      }
       
      String errorPage = cmd.getErrorPage();
      if (errorPage == null) {
        errorPage = SystemConfig.getParameter("page-of-processerror", "moremotion/error/ProcessErrorPage");
      }
      
      String continuePage = cmd.getContinuePage();
      if (continuePage == null) {
        continuePage = originPage;
      }
      
      VariablePool vpool = request.getVPool();
      // vpool.put("_originpage", originPage);
      vpool.put("_nextpage", nextPage);
      vpool.put("_errorpage", errorPage);
      vpool.put("_continuepage", continuePage);
      
      vpool.put("_maxlcc", "0");
      
      ExecutionManager em = ExecutionManager.getExecutionManager(
          cmd.getTemplateName(request), originPage, request, cmd.getDebugLevel(request));
      
      int cc;
      try {
        cc = em.execute();
      } catch (ProcessManagingException ex) {
        cc = 20;
      }
      
      vpool.put("_maxcc",cc + "");
      
      if (cc <= 4) {
        nextPage = vpool.get("_nextpage");
        nextPage = request.resolve(nextPage);
        displayNextPage( nextPage, request );
        
      } else if (cc <= 8) {
        displayNextPage( originPage, request );
        
      } else {
        setContinuePage(request);
        errorPage = vpool.get("_errorpage","");
        errorPage = request.resolve(errorPage);
        displayNextPage( errorPage, request );
        
      }
      
    } catch (Exception ex) {
      
      ex.printStackTrace();
      throw new ServiceException(ex);
      
    }
    
  }
  
  private static void displayNextPage(String pageName, MoreMotionRequest request) throws ServiceException, IOException, MScriptException {
    
    if (request.getResponded()) {
      return;
    }
    
    if (pageName.indexOf("?") > -1 || pageName.indexOf("http://") > -1 || pageName.indexOf(".") > -1) {
      
      if (pageName.endsWith("_rand=")) {
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        String time = format.format(new java.util.Date());
        pageName = pageName + time;
      }
      
      request.getResponse().sendRedirect(pageName);
      
    } else {
      request.generatePage(pageName);
    }
    
  }
  
  
  /**
   * Executes the Page Process Templates and returns <CODE>true</CODE> if caller can continue normally
   * and <CODE>false</CODE> if caller should return since there was an error and ProcessManager has
   * already handled it and redirected to the error page.
   * @param currpage The CurrectPage Object
   * @param request MoreMotionRequest object
   * @param beforeDS Before DataSources Flag. If <CODE>true</CODE> specified the template calls that are marked as
   * 'Execute Before Data Sources', and if <CODE>false</CODE> specified the templates that are NOT marked as
   * 'Execute Before Data Sources' will be executed.
   */
  public static boolean executePageProcessTemplates(CurrentPage currpage, MoreMotionRequest request, boolean beforeDS) throws PageGenerationException {
    
    try {
      
      if (!currpage.getPageInfo().getPageConfig().getAvoidCaching(request)) {
        if (isSameRequest(request,beforeDS == false)) {
          return true;
        }
      }      
      
      ConditionalList templateCallList = currpage.getPageInfo().getPageConfig().getProcessTemplateCalls();
      if (templateCallList == null) {
        return true;
      }
      
      ConditionalListItem[] templateCalls = templateCallList.getItems(request);
      
      for (int i = 0; i < templateCalls.length; i++) {
        
        TemplateCall call = (TemplateCall)templateCalls[i];
        
        if (call.isBeforeDS(request) == beforeDS) {
          
          String originPage = call.getOriginPage();
          
          String errorPage = call.getErrorPage();
          if (errorPage == null) {
            errorPage = SystemConfig.getParameter("page-of-processerror", "moremotion/error/ProcessErrorPage");
          }
          String continuePage = call.getContinuePage();
          if (continuePage == null) {
            continuePage = originPage;
          }
          
          VariablePool vpool = request.getVPool();
          vpool.put("_originpage", originPage);
          vpool.put("_errorpage", errorPage);
          vpool.put("_continuepage", continuePage);
          
          vpool.put("_maxlcc", "0"); 
          
          ExecutionManager em = ExecutionManager.getExecutionManager(
              call.getName(), currpage.getPageName(true), request, call.getDebugLevel(request));
          
          int cc = em.execute();
          
          vpool.put("_maxcc",cc+"");
          
          if (cc <= 4) {
            return true;
            
          } else if (cc <= 8) {
            displayNextPage( originPage, request );
            return false;
            
          } else {
            setContinuePage(request);
            displayNextPage( vpool.get("_errorpage"), request );
            return false;
          }
          
        }
      }
      
      return true;
      
    } catch (Exception ex) {
      request.generateDefaultErrorPage();
      return false;
    }
    
  }
  
  private static void setContinuePage(MoreMotionRequest request) throws MScriptException {
    
    String pageName = null;
    VariablePool vpool = request.getVPool();
    
    pageName = vpool.get("_continuepage");
    pageName = request.resolve(pageName);
    vpool.put("_continuepage", pageName);
    
  }
  
  
  private static boolean isSameRequest(MoreMotionRequest request, boolean saveRand) {
    
    if (request.getMethod().equalsIgnoreCase("GET")) {
      
      String _rand = request.getParameter("_rand",null);
      if (_rand != null) {
        HttpSession ses = request.getSession();
        String _prev_rand = (String)ses.getAttribute("moremotion.user.request.rand");
        if (saveRand) ses.setAttribute("moremotion.user.request.rand", _rand);
        if (_prev_rand != null) {
          if (_rand.compareTo(_prev_rand) <= 0) {
            return true;
          }
        }
      }
    }
    
    return false;
  }
  
  
  private static void sendHistoryBack(MoreMotionRequest request) throws ServiceException {
    
    try {
      
      HttpServletResponse response = request.getResponse();
      
      if (UserAgentAnalyzer.isXSLTEnabled(request)) {
        
        String pageName = SystemConfig.getParameter("page-of-invalidaction", "moremotion/error/InvalidActionPage");
        
        request.generatePage(pageName);
        return;
        
      } else {
        request.setNoCache();
        response.setContentType("text/html");
        PrintWriter out = response.getWriter();
        out.println("<html><head><script language=\"javascript\">");
        out.println("  history.go(1);");
        out.println("</script></head></html>");
        out.close();
      }
      
    }  catch(Exception e) {
      throw new ServiceException("CANNOT_CREATE_HISTORY_BACK");
    }
  }
  
  
}