/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * QueryProcessCallDefinition.java
 *
 * Created on 12 March 2008 Wednesday, 16:55
 */

package org.moremotion.process;

import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: QueryProcessCallDefinition.java 51 2008-03-10 11:46:54Z erkan $
 */
public class QueryProcessCallDefinition extends CallDefinition {
  
  private ConfigParameter outputTo;
  private ConditionalList callList;
  
  public QueryProcessCallDefinition(Element elm, Object parent) throws ConfigurationException {
    super(elm, parent);
    parse(elm);
    prepareProcessConfig();
  }

  
  private void parse(Element elm) throws ConfigurationException {
    
    outputTo = new ConfigParameter("outputTo",elm.getAttribute("outputTo"));
    
    NodeList nlist = elm.getChildNodes();
    
    for (int i = 0; i < nlist.getLength(); i++) {
      
      Node node = nlist.item(i);
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        
        if (node.getNodeName().equals("callProcess")) {
          if (callList == null) callList = new ConditionalList();
          callList.add( new ProcessCallDefinition( (Element)node, this ) );
        }
        
        else if (node.getNodeName().equals("callQueryProcess")) {
          if (callList == null) callList = new ConditionalList();
          callList.add( new QueryProcessCallDefinition( (Element)node, this) );
          
        }
        
        else if (node.getNodeName().equals("callTemplate")) {
          if (callList == null) callList = new ConditionalList();
          callList.add( new ProcessTemplateCallDefinition( (Element)node, this) );
          
        }
        
      }
      
    }
    
  } 
  
  public String getOutputTo(MoreMotionRequest request) throws ConfigParameterException {
    return outputTo.resolve(request).stringValue(null);
  }

  protected ConditionalList getCallList() {
    return this.callList;
  }
  
  
}
