/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               *
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      *
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License *
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 *
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * AccessControlDefinition.java
 *
 * Created on 24 December 2005 Saturday, 13:03
 */

package org.moremotion.security;

import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: AccessControlDefinition.java 228 2008-10-14 08:18:04Z erkan $
 */
public class AccessControlDefinition {

  private String secdom = "main";
  private String checkPoint = null;
  private String loginPage = "/index";
  private String failurePage = null;

  public AccessControlDefinition(String secdom, String checkPoint,
      String loginPage, String failurePage) {

    if (secdom != null) {
      this.secdom = secdom;
    }

    this.checkPoint = checkPoint;
    if (loginPage != null) {
      this.loginPage = loginPage;
    }

    if (failurePage != null) {
      this.failurePage = failurePage;
    }

  }

  public AccessControlDefinition(String checkPoint, String loginPage, String failurePage) {

    this.checkPoint = checkPoint;
    if (loginPage != null) {
      this.loginPage = loginPage;
    }

    if (failurePage != null) {
      this.failurePage = failurePage;
    }

  }

  /** Creates a new instance of AccessControlDefinition.
   * <p>Sample Creation: <br/><code>AccessControlDefinition acd = new AccessControlDefinition(&quot;main:CAN_DO_THIS,LoginPage,AuthorizationFailurePage&quot;)</code></p>
   */
  public AccessControlDefinition(String acdStr) {

    if (acdStr != null && acdStr.length() > 0) {
      if (acdStr.indexOf(',') != -1) {
        String[] array = XString.split(acdStr,',');
        try {
          parseCheckPointStr(array[0]);
          loginPage = array[1];
          failurePage = array[2];
        } catch(Exception ignored) {}
      } else {
        checkPoint = acdStr;
      }
    }
  }

  private void parseCheckPointStr(String cpStr) {
    int i = cpStr.indexOf(':');
    if (i == -1) {
      checkPoint = cpStr;
    } else {
      checkPoint = cpStr.substring(i+1);
      secdom = cpStr.substring(0,i);
    }
  }


  public String getSecurityDomain() {
    return secdom;
  }

  public String getCheckPoint() {
    return checkPoint;
  }

  public boolean hasCheckPoint() {
    return checkPoint != null && checkPoint.length() > 0;
  }

  public String getLoginPage() {
    return loginPage;
  }

  public String getFailurePage() {
    return failurePage;
  }

  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("AccessControlDefinition [ securityDomain: " + secdom);
    buf.append(", checkPoint: " + checkPoint );
    buf.append(", loginPage: " + loginPage );
    buf.append(", failurePage: " + failurePage );
    buf.append(" ]");
    return buf.toString();
  }

  public static void main(String[] args) throws Exception {

  AccessControlDefinition acd = new AccessControlDefinition("CAN_DO_THIS");
  System.out.println(acd + "");

  }


}
