/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Encryption.java
 *
 */

package org.moremotion.util;

/**
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Encryption.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Encryption {
  
  public static final String DECRYPT_ERROR = "DECRYPT_ERROR!";
  
  /** A table of hex digits */
  private static final char[] hexDigit = {
    '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
  };
  
  private static char toHex(int nibble) {
    return hexDigit[(nibble & 0xF)];
  }
  
  private static String stringToHex(String source) {
    StringBuffer out = new StringBuffer();
    for (int i = source.length() - 1; i >= 0 ; i--) {
      char aChar = source.charAt(i);
      char c1 = toHex((aChar >>  4) & 0xF);
      char c2 = toHex( aChar & 0xF);
      out.append(c2);
      out.append(c1);
    }
    return out.toString();
  }
  
  private static String hexToString(String source) {
    StringBuffer out = new StringBuffer();
    if (source.length() % 2 != 0) {
      return DECRYPT_ERROR;
    }
    for (int x=source.length() - 1; x >= 0; ) {
      int value=0;
      
      char[] cbuf = new char[2];
      cbuf[0] = source.charAt(x--);
      cbuf[1] = source.charAt(x--);
      
      for (int i = 0; i < 2; i++) {
        char aChar = cbuf[i];
        
        switch (aChar) {
          case '0': case '1': case '2': case '3': case '4':
          case '5': case '6': case '7': case '8': case '9':
            value = (value << 4 ) + aChar - '0';
            break;
          case 'a': case 'b': case 'c':
          case 'd': case 'e': case 'f':
            value = (value << 4 ) + 10 + aChar - 'a';
            break;
          case 'A': case 'B': case 'C':
          case 'D': case 'E': case 'F':
            value = (value << 4 ) + 10 + aChar - 'A';
            break;
          default:
            return DECRYPT_ERROR;
        }
      }
      out.append((char) value);
    }
    return out.toString();
  }
  
  
  /** Encrypts a given string.
   */
  public static String encrypt(String s, int SC, int PC1, int PC2, int PC3) {
    if (s == null || s.equals("")) return s;
    
//    if (hasSignature( _decrypt(s, SC, PC1, PC2, PC3) )) {
//      return s;
//    }
    s = addSignature(s);
    
    return _encrypt(s, SC, PC1, PC2, PC3);
  }
  
  /** Decrypts a given string.
   */
  public static String decrypt(String s, int SC, int PC1, int PC2, int PC3) {
    if (s == null || s.equals("")) return s;
    String result = _decrypt(s, SC, PC1, PC2, PC3);
    return removeSignature(result);
  }
  
  private static String _decrypt(String str, int SC, int PC1, int PC2, int PC3) {
    if (str == null || str.equals("")) return str;
    StringBuffer out = new StringBuffer();
    String source = hexToString(str);
    if (source.equals(DECRYPT_ERROR)) return DECRYPT_ERROR;
    int curIndex = PC3 * (PC1 + PC2);
    for (int i=0; i < source.length(); i++) {
      char curCh = (char) (source.charAt(i) & 0xFF);
      int curIndShr = curIndex >> SC;
      int decCh = ((int) (curCh ^ curIndShr) & 0xFF);
      curIndex = ( curCh + curIndex ) * PC1 + PC2;
      out.append((char) decCh);
    }
    return out.toString();
  }
  
  private static String _encrypt(String str, int SC, int PC1, int PC2, int PC3) {
    StringBuffer out = new StringBuffer();
    int curIndex = PC3 * (PC1 + PC2);
    
    for (int i=0; i < str.length(); i++) {
      char curCh = (char) (str.charAt(i) & 0xFF);
      int curIndShr = curIndex >> SC;
      char encCh = (char) (curCh ^ curIndShr);
      curIndex = ( (encCh & 0xFF) + curIndex ) * PC1 + PC2;
      out.append(encCh);
    }
    return stringToHex(out.toString());
    
  }
  
  private static final String SIGNATURE = "$ver1.";
  
  private static boolean hasSignature(String plain) {
    return plain.startsWith(SIGNATURE);
  }
  
  private static String removeSignature(String plain) {
    if (plain.startsWith(SIGNATURE)) {
      return plain.substring(SIGNATURE.length());
    }
    return DECRYPT_ERROR;
  }
  
  private static String addSignature(String plain) {
    return SIGNATURE + plain;
  }
  
  public static boolean isEncrypted(String pass, int SC, int PC1, int PC2, int PC3) {
    String s = _decrypt(pass, SC, PC1, PC2, PC3);
    return hasSignature(s);
  }
  
  private static String toUnsignedString(int i, int shift) {
    char[] buf = new char[32];
    int charPos = 32;
    int radix = 1 << shift;
    int mask = radix - 1;
    do {
      buf[--charPos] = AlphaDigits[i & mask];
      i >>>= shift;
    } while (i != 0);
    
    return new String(buf, charPos, (32 - charPos));
  }
  
  final static char [] AlphaDigits = {
    '@', '0', '|', 'V', '4', '5', '6', '7', '$', '9',
    'Z', 'w', 'L', 'r', 't', 'y', 'J', 'o', 'p', 'a',
    'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'O', 'P', 'A',
    's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'i', 'z',
    'S', 'D', 'F', 'G', 'H', 'u', 'K', 'e', 'I', 'q',
    'x', 'c', 'v', 'b', '8', 'm', '1',
    'X', 'C', '3', 'B', 'N', 'M', '2'
  } ;
  
  
  public static String toAlphaString(int value) {
    String s = toUnsignedString(value,6);
    return s;
  }
  
  private static int getAlphaCode(char c) {
    for (int i = 0; i < AlphaDigits.length; i++) {
      if (c == AlphaDigits[i]) return i;
    }
    return 0;
  }
  
  public static int parseAlphaString(String value) {
    
    switch (value.length()) {
      
      case 3:
        return getAlphaCode(value.charAt(0)) * 4096 +
                getAlphaCode(value.charAt(1)) * 64 +
                getAlphaCode(value.charAt(2));
        
      case 2:
        return getAlphaCode(value.charAt(0)) * 64 +
                getAlphaCode(value.charAt(1));
        
      default:
        return getAlphaCode(value.charAt(0));
        
    }
    
  }
  
  public static String encstr(String str, String ref) {
    
    if (str == null) return null;
    String hash = "" + (str + ref).hashCode();
    hash = hash.substring(hash.length()-3);
    String s = hash + "?" + str;
    byte[] bb = s.getBytes();
    StringBuffer bf = new StringBuffer();
    
    int diff = 1;
    for (int i = 0; i < bb.length; i++) {
      byte b = bb[i];
      
      int iv = b > -1 ? b : (b * -1) + 256;
      iv -= diff;
      
      String x = toAlphaString(iv);
      
      switch (x.length()) {
        
        case 1:
          bf.append(x.charAt(0));
          break;
          
        case 2:
          bf.append('n');
          bf.append(x.charAt(0));
          bf.append(x.charAt(1));
          break;
          
        default :
          bf.append(':');
          bf.append(x.charAt(0));
          bf.append(x.charAt(1));
          bf.append(x.charAt(2));
      }
      
      diff++;
      if (diff > 8) diff = 1;
    }
    return "!" + bf;
  }
  
  public static String decstr(String str, String ref) {
    
    try {
      
      if (str == null) return str;
      if (str.length() > 0 && str.charAt(0) == '!') {
        int ln = str.length() - 1;
        int sze = ln;
        byte[] bb = new byte[sze];
        int diff = 1;
        int j = 1;
        int k = 0;
        
        while (j <= ln) {
          
          int iv;
          
          switch (str.charAt(j)) {
            
            case ':':
              j++;
              iv = parseAlphaString( str.substring(j,j+3) );
              j += 3;
              break;
              
            case 'n':
              j++;
              iv = parseAlphaString( str.substring(j,j+2) );
              j += 2;
              break;
              
            default:
              iv = parseAlphaString( str.substring(j,j+1) );
              j++;
              
          }
          
          iv += diff;
          if (iv > 255) {
            iv -= 256;
            iv *= -1;
          }
          bb[k] = (byte)iv;
          k += 1;
          diff++;
          if (diff > 8) diff = 1;
        }
        String ss = new String(bb,0,k);
        int pos = ss.indexOf('?');
        if (pos > 0) {
          String hsh = ss.substring(0,pos);
          String val = ss.substring(pos+1);
          String hash = "" + (val + ref).hashCode();
          hash = hash.substring(hash.length()-3);
          if (hsh.equals(hash)) {
            return val;
          }
        }
        
      }
    } catch (Exception e) {}
    
    return "DECRYPT_ERROR!";
    
  }
  
  
}
