/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * FileUtils.java
 *
 * Created on March 18, 2002, 6:13 PM
 */

package org.moremotion.util;

import java.io.File;

import javax.servlet.*;
import javax.servlet.http.*;
import org.moremotion.page.CurrentPage;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * This class provides static utility methods to access MoreMotion specific files.
 *
 * @author  Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: FileUtils.java 51 2008-03-10 11:46:54Z erkan $
 */
public class FileUtils {
  
  
  /**
   * Returns the file associated with the given page name. If variant parameter is not null and the variant file
   * exits then it is returned. If variant file missing and the base file exists then the base file is returned.
   * Otherwise <CODE>false</CODE> is returned.
   * @param request The MoreMotionRequest object
   * @param page The page name
   * @param variant The page variant.
   */
  public static File getPageFile(MoreMotionRequest request, String page, String variant) {
    
    if (page.lastIndexOf('.') > -1) {
      return getVariantFile( request.getRealFile(page), variant );
    }
    
    File hfile = getVariantFile( request.getRealFile(page + ".html"), variant);
    File xfile = getVariantFile( request.getRealFile(page + ".xsl"), variant);
    
    if (hfile != null && xfile != null) {
      if (xfile.lastModified() > hfile.lastModified()) {
        return xfile;
      } else {
        return hfile;
      }
    }
    
    if (xfile != null) {
      return xfile;
    }
    
    if (hfile != null) {
      return hfile;
    }
    
    return null;
  }
  
  
  private static final String PageConfigDir = "WEB-INF/MM-INF/config/pages/";
  
  public static File getPageConfigFile(MoreMotionRequest request, String page, String variant) {
    
    int x = page.lastIndexOf('.');
    if (x > -1) {
      page = page.substring(0,x);
    }
    
    return getVariantFile( request.getRealFile(PageConfigDir + page + ".xml"), variant);
  }
  
  private static File getVariantFile(File file, String variant) {
    
    if (variant != null && variant.trim().length() > 0 && !variant.equals("*")) {
      
      String name = file.getName();
      String ext = "";
      int x = name.lastIndexOf('.');
      if (x > -1) {
        ext = name.substring(x);
        name = name.substring(0,x);
      }
      
      File variantFile = new File(file.getParentFile(), name + File.separator + "_." + variant + ext);
      if (variantFile.exists()) {
        return variantFile;
      }
    }
    
    if (file.exists()) {
      return file;
    }
    return null;
  }
  
  
//  public static boolean Debug = false;
  
//  public static String getWebFilename(String absolutePath, File webRootFile) {
//    if (webRootFile == null) {
//      return null;
//    }
//    return getWebFilename(absolutePath, webRootFile.getAbsolutePath());
//  }
//
//  public static String getWebFilename(File file, String webRoot) {
//    if (file == null) {
//      return null;
//    }
//    return getWebFilename(file.getAbsolutePath(), webRoot);
//  }
//
//  public static String getWebFilename(File file, File webRootFile) {
//    if (file == null) {
//      return null;
//    }
//    return getWebFilename(file.getAbsolutePath(), webRootFile.getAbsolutePath());
//  }
//
//  public static String getWebFilename(String absolutePath, String webRoot) {
//    String result = null;
//    if (webRoot.endsWith("/") || webRoot.endsWith("\\")) {
//      result = absolutePath.substring(webRoot.length() - 1);
//    } else {
//      result = absolutePath.substring(webRoot.length());
//    }
//    return result.replace('\\', '/');
//  }
//
//  public static boolean endsWithCaseInsensitive(String s, String suffix) {
//    // WATCH-OUT: lowercasing in default locale
//    String lowered = s.toLowerCase();
//    return lowered.endsWith(suffix);
//  }
  
  
  
  /**
   * Splits a file name into two parts extension and the part before the extension.
   * @param filename File name.
   * @return String array. First element of the array contains the part before the extension
   * Second element contains the extension.
   */
  public static String[] splitByExtension(String filename) {
    if (filename == null) {
      return new String[0];
    }
    int index = filename.lastIndexOf('.');
    if (index == -1) {
      return null;
    }
    String beforeExtension = filename.substring(0, index);
    String extension = filename.substring(index);
    String[] result = { beforeExtension, extension };
    return result;
  }


  
  /** Returns current page variant.
   */
  public static String getCurrentPageVariant(MoreMotionRequest request) {
    String result = (String) request.getSession().getAttribute("moremotion.current.page.variant");
    if (result == null) {
      result = request.getCookieValue("moremotion.current.page.variant");
      if (result == null || result.equals("*")) {
        result = null;
      }
      request.getSession().setAttribute("moremotion.current.page.variant", result);
    }
    return result;
  }
  
  /** Sets current page variant
   */
  public static void setCurrentPageVariant(MoreMotionRequest request, String variant) {
    if (variant == null) variant = "";
    request.getSession().setAttribute("moremotion.current.page.variant", variant);
    request.setCookieValue("moremotion.current.page.variant", variant, 946080000);
  }
  
}