/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ViewADOM.java
 *
 * Created on July 29, 2002, 7:36 PM
 */

package org.moremotion.utilityservices;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.util.Iterator;
import javax.servlet.http.HttpServletResponse;
import org.moremotion.action.ActionServiceContext;

import org.moremotion.adom.ADOM;
import org.moremotion.page.PageDataRequest;
import org.moremotion.page.PageXMLData;
import org.moremotion.page.ADOMFormatter;
import org.moremotion.action.ActionService;

import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.adom.ADOMUtils;

/**
 * Makes a view of an ADOM.
 * @author  Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ViewADOM.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ViewADOM implements ActionService {
  
  private static final String REQ_PARAM_ADOM = "adom";
  private static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
  private static final String EMPTY = "<root/>";
  private static final String CONTENT_TYPE_TEXT_XML = "text/xml; charset=UTF-8";
  
  public void doService(ActionServiceContext asc) throws ServiceException, IOException {
    
    MoreMotionRequest request = asc.getRequest();
    
    if (!Utils.isUtilityServiceActive(request,"ViewADOM")) return;
    
    PageXMLData xmldata = new PageXMLData();
    
    String[] names = request.getParameterValues(REQ_PARAM_ADOM);
    if (names != null && names.length != 0) {
      
      for (int i = 0; i < names.length; i++) {
        
        String nodePath = new String(names[i]);
        
        if (nodePath != null && nodePath.length() != 0 ) {
          
          String adomName = ADOMUtils.getPureADOMName(nodePath);
          ADOM adom = ADOMUtils.findADOM(request, nodePath);
          
          if (adom != null) {
//            PageDataRequest pdr = new PageDataRequest(adomName);
//            pdr.addFilter(ADOMUtils.getNodePath(nodePath));
            ADOMFormatter xf = new ADOMFormatter(adom,request,adomName,null);
            xmldata.append(xf.format());
          }
        }
      }
    }
    
    // write it out
    HttpServletResponse response = request.getResponse();
    response.setHeader("Cache-Control","no-cache"); //HTTP 1.1
    response.setHeader("Pragma","no-cache"); //HTTP 1.0
    response.setDateHeader("Expires", -1); //prevents caching at the proxy server
    response.setContentType(CONTENT_TYPE_TEXT_XML);
    
    BufferedWriter bw = new BufferedWriter(
        new OutputStreamWriter(response.getOutputStream(), "UTF-8")
        );
    
    bw.write(xmldata.getXml());
    bw.flush();
    bw.close();
    
  }
  
}